/*
 *= CMU (UNIVERSAL RADIO COMMUNICATION TESTER VISA) ===========================
 * LabWindows/CVI 5.0 Instrument Driver
 * Original Release : June, 1999
 * By : ROHDE & SCHWARZ FRANCE
 *      CONTACTS :
 *
 *              Fran?s Ribiere
 *              Telephone : 00 33 1 41 36 10 83
 *              Fax       : 00 33 1 41 36 11 10
 *              e-Mail    : Francois.Ribiere@rsf.rohde-schwarz.com
 *
 *              Rohde & Schwarz France
 *              Parc Tertiaire de Meudon
 *              9-11, rue Jeanne Braconnier
 *              92366 Meudon-la-Foret Cedex
 *
 * Version : V5.00.2, 03/2009
 *
 *  Modification History:
 *    see ChangeLog
 *-----------+------------+----------------------------------------------------

 *===========================================================================*/

/****************************************************************************/
/* WARNING:                                                                 */
/* ======================================================================== */
/* When using the new functinality of the status checking please try to not */
/* use the RSMU_Read and RSMU_Write functions. If it is neccessary call     */
/* RSCMU_setCheckStatus function to restore your desired checking status.   */
/* After each call of the RSCMU_Read function the status checking is always */
/* off. This note does not regards to RSCMU_Query and RSCMU_QueryArray      */
/* functions.                                                               */
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <visa.h>
#include <locale.h>
#include "rscmu200.h"

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS======================================*/
/*****************************************************************************/
static ViString mode_On_Off[] = {"OFF","ON",VI_NULL}; // Common for every
static ViString StrFreqUnit[] = {"HZ","KHZ","MHZ","GHZ",VI_NULL};
static ViString StrTimeUnit[] = {"US","MS","S",VI_NULL};
static ViString StrRepetition[] = {"CONT","SING",VI_NULL};
static ViString StrStopCondition[] = {"NONE","SON",VI_NULL};
static ViString StrStepMode[] = {"NONE","STEP",VI_NULL};
static ViString StrModulation[] = {"",":GMSK",":EPSK",VI_NULL};
static ViString StrCommand[] = {"CONF","INIT","ABOR","STOP","CONT","READ","FETC","SAMP","CALC",VI_NULL};
static ViString StrObject[] = {":POW",":MOD",":SPEC",":SPOW",":WPOW",":SPEC:MOD",":SPEC:SWIT",":POW:MPR",":POW:SLOT",
                               ":POW:XSL",":POW:FRAM",":POW:PCL",":MOD:XPER",":MOD:OVER",":MOD:EVM",":MOD:PERR",
                               ":MOD:MERR", ":POW:ABUR",":POW:MSL",":NPOW",":NPOW",":SPECtrum:MSWitching",VI_NULL};
static ViString StrScopeMeas [] = {"SCAL","ARR",VI_NULL};
static ViReal64 unitExpLow[] = {1.0E-6, 1.0E-3, 1.0};
static ViString measurementStatusArr[] = {"OFF","RUN","STOP","ERR","STEP","RDY",VI_NULL};
static ViString modeArr[] = {"OFF","SRQ","SOPC","SRSQ",VI_NULL};
static ViString genStatArr[] = {"OFF", "RUN", "ERR", VI_NULL};
static ViString AFInputArr[] = {"PRIM","SEC", VI_NULL};
static ViString matchingArr[] = {"OK","INV","NMAU","NMAL", VI_NULL};
static ViReal64 freqMin[] = {5.0E+4, 50.0, 0.05, 5.0E-5,RSCMU_NAN};
static ViReal64 freqMax[] = {2.7E+9, 2.7E+6, 2.7E+3, 2.7,RSCMU_NAN};
static ViString StrCAMMMode[] = {"CURR","AVER","MAX","MIN"};
static ViString StrStat[] = {"ALL","ARIT","MIN","MAX","IVAL"};
static ViString audioAnalyzerChanArr[]  = {"PRIM","SEC", VI_NULL};
static ViString bandPassDCCouplingArr[] = {"","BP01","BP02","BP03","BP04","BP05","BP06",
                                           "BP07","BP08","BP09","BP10","BP11","BP12",
                                           "BP13","BP14","BP15","BP16","BP17","BP18","BP19", VI_NULL};

/*---------------------------------------------------------------------------*/
/*
/*---------------------------------------------------------------------------*/
typedef enum {
  RSCMU_READ,
  RSCMU_FETC,
  RSCMU_SAMP
} MeasCommand;

/*****************************************************************************/
/*  LOCAL FUNCTIONS DEFINITION                                               */
/*****************************************************************************/

ViStatus RSCMU_RF_DefaultSettings    (ViSession, ViChar*, ViInt32, ViStatus, ViChar[]);

/***************************************************************
**
**   My Locals
**
***************************************************************/

/* ----------------------------------------------------------------------------
 * lokal function
 * ----------------------------------------------------------------------------*/
ViStatus _VI_FUNC RSCMU_ConfIPAddress (ViSession instrSession,
                                       ViChar   *StrCommand,
                                       ViInt32  _1st,
                                       ViInt32  _2nd,
                                       ViInt32  _3rd,
                                       ViInt32  _4th,
                                       ViChar   errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  GET_WRT_BUF(instrSession);

  // Check if the function is correctly used:
  CHECK_VI_INT32_RANGE(instrSession, _1st, 0, 255, RSCMU_ERROR_PARAMETER2, errorMessage);
  CHECK_VI_INT32_RANGE(instrSession, _2nd, 0, 255, RSCMU_ERROR_PARAMETER3, errorMessage);
  CHECK_VI_INT32_RANGE(instrSession, _3rd, 0, 255, RSCMU_ERROR_PARAMETER4, errorMessage);
  CHECK_VI_INT32_RANGE(instrSession, _4th, 0, 255, RSCMU_ERROR_PARAMETER5, errorMessage);

  sprintf (wrtBuf, "%s %d,%d,%d,%d", StrCommand, _1st, _2nd, _3rd, _4th);

  RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}


/* ----------------------------------------------------------------------------
 * lokal function
 * ----------------------------------------------------------------------------*/
ViStatus _VI_FUNC RSCMU_GetIPAddress (ViSession instrSession,
                                       ViChar   *StrCommand,
                                       ViInt32  *_1st,
                                       ViInt32  *_2nd,
                                       ViInt32  *_3rd,
                                       ViInt32  *_4th,
                                       ViChar   errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViChar      tab[100][100];
  GET_BUFS(instrSession);

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

  sprintf (wrtBuf, "%s", StrCommand);
  CHECKRETMSG(RSCMU_getViString(instrSession,wrtBuf,rdBuf,RD_BUFFER_SIZE,0));

  // Separate the CMU response:
  RSCMU_SeparateDataValues (rdBuf, ",", tab);

  if (strcmp (tab[0], "NAN") != 0)
    sscanf (tab[0], "%lf", _1st);
  else *_1st = RSCMU_NAN_VI_INT32;

  if (strcmp (tab[1], "NAN") != 0)
    sscanf (tab[1], "%lf", _2nd);
  else *_2nd = RSCMU_NAN_VI_INT32;

  if (strcmp (tab[1], "NAN") != 0)
    sscanf (tab[1], "%lf", _3rd);
  else *_3rd = RSCMU_NAN_VI_INT32;

  if (strcmp (tab[1], "NAN") != 0)
    sscanf (tab[1], "%lf", _4th);
  else *_4th = RSCMU_NAN_VI_INT32;

  return RSCMU_Status;
}




/* ----------------------------------------------------------------------------
 * lokal function
 * ----------------------------------------------------------------------------*/
ViStatus _VI_FUNC RSCMU_MeasArray (ViSession  instrSession,
                          ViChar    *StrCommand,
                          ViReal64   values[],
                          ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    ViInt32     NbOfMeasValues;
    ViChar      valuesTmp[1000][20];

    sprintf (wrtBuf, "%s?", StrCommand);

    if ((RSCMU_Status = RSCMU_Query_Array (instrSession, wrtBuf, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
        return RSCMU_Status;

    // Find all values by breaking the read buffer:
    NbOfMeasValues = RSCMU_scanStringBurst (rdBuf, valuesTmp);

    RSCMU_Status = RSCMU_ReturnCMUResponse (NbOfMeasValues, valuesTmp, values);

    return RSCMU_Status;
}


/* ----------------------------------------------------------------------------
 * lokal function
 * ----------------------------------------------------------------------------*/
ViStatus _VI_FUNC RSCMU_MeasArrayInt (ViSession  instrSession,
                             ViChar    *StrCommand,
                             ViInt32    values[],
                             ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    ViInt32     NbOfMeasValues;
    ViChar      valuesTmp[669][20];
    ViInt32 i;

    sprintf (wrtBuf, "%s?", StrCommand);

    if ((RSCMU_Status = RSCMU_Query_Array (instrSession, wrtBuf, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
        return RSCMU_Status;

    // Find all values by breaking the read buffer:
    NbOfMeasValues = RSCMU_scanStringBurst (rdBuf, valuesTmp);

    for (i=0; i < (NbOfMeasValues); i++)
    {
      if (strncmp (valuesTmp[i], "NAN",3) != 0)
        sscanf (valuesTmp[i], "%d", &values[i]);
      else  values[i] = RSCMU_NAN_VI_INT32;
    }

    return RSCMU_Status;
}







/*****************************************************************************/
/*========== CLASS: BASIC UNIT ==============================================*/
/*****************************************************************************/
/*****************************************************************************/

/*
 * ----------------------------------------------------------------------------
 * Initialize
 * ----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: Initialize
/* Purpose : This function opens the instrument, queries the instrument
/*           for its ID, and initializes the instrument to a known state.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Init_Base (ViPSession  instrSession,
                                   ViInt32     SCPIConnection,
                                   ViInt32     ConnectionNb,
                                   ViInt32     PrimAddr,
                                   ViBoolean   IDQuery,
                                   ViBoolean   reset,
                                   ViChar      errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViSession   globalSession = 0;
    ViChar      rdBuf[RD_BUFFER_SIZE]= "";
    ViChar      Board[5]= "", resourceName[30] = "";
    ViChar      tmpBuf[10]="";
    ViUInt16    i;
    ViInt32     Version;

    struct RSCMU_Base_Globals *p_globalvarBase;

    /* --- Set standard "C" locale --- */
    {
       char *locale;

       locale = setlocale(LC_ALL, "C");
    }

    // Init the Error Message
    strcpy (errorMessage, "No Error");

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE (*instrSession, SCPIConnection, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE (*instrSession, ConnectionNb, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE (*instrSession, PrimAddr, 0, 30, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_BOOLEAN_RANGE (*instrSession, IDQuery, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_BOOLEAN_RANGE (*instrSession, reset, RSCMU_ERROR_PARAMETER6, errorMessage);

    // Hardware Connection
    switch (SCPIConnection)
    {
    case RSCMU_GPIB:
        strcpy (Board, "GPIB");
        sprintf(resourceName, "%s%ld::%ld::0::INSTR", Board, ConnectionNb, PrimAddr);
        break;
    case RSCMU_COM:
        strcpy (Board, "ASRL");
        sprintf(resourceName, "%s%ld::INSTR", Board, ConnectionNb);
        break;
    }

    // Open instrument session
    CHECKRET(viOpenDefaultRM (&globalSession));

    if ((RSCMU_Status = viOpen (globalSession, resourceName, VI_NULL, 3000,instrSession)) != 0)
    {
        viClose ( *instrSession );
        return RSCMU_Base_errorMessage (globalSession, RSCMU_Status, errorMessage);
    }

    if (SCPIConnection == RSCMU_COM)
    {
        CHECKRET(viSetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, 19200));
        CHECKRET(viSetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, 8));
        CHECKRET(viSetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE));
        CHECKRET(viSetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS,VI_ASRL_STOP_ONE));
        CHECKRET(viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL, VI_ASRL_FLOW_RTS_CTS));
    }

    // Timeout set to 20 seconds
    CHECKRET(viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, RSCMU_TMO_VALUE));

    // Allocate space for "struct RSCMU_Base_Globals"
    if ((p_globalvarBase = (struct RSCMU_Base_Globals *)malloc(sizeof(struct RSCMU_Base_Globals))) == 0)
        return RSCMU_initCleanUp (globalSession, *instrSession, RSCMU_Status);

    memset ( p_globalvarBase, 0, sizeof ( struct RSCMU_Base_Globals ) );
    if ((RSCMU_Status = viSetAttribute(*instrSession, VI_ATTR_USER_DATA,(ViAttrState)p_globalvarBase)) < 0)
    {
        free( p_globalvarBase );
        return RSCMU_initCleanUp (globalSession, *instrSession, RSCMU_Status);
    }

    sprintf(tmpBuf, "%s%ld", Board, ConnectionNb);

    // Set initial values of variables for function Group 'BASE_Unit'.
    strcpy (p_globalvarBase->RSCMU_tGlobal.FGrp_Name, "BASE");
    p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address = PrimAddr;
    p_globalvarBase->RSCMU_tGlobal.RSCMU_secondary_address = 0;
    strncpy (p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, tmpBuf, strlen(tmpBuf));
    p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection[strlen(tmpBuf)] = 0;
    p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE = RSCMU_OFF;
    p_globalvarBase->RSCMU_tGlobal.RSCMU_RmSessionDefault = globalSession;
    p_globalvarBase->RSCMU_tGlobal.RSCMU_SessionNb = *instrSession;

    // Record the execution of this Init Function.
    p_globalvarBase->RSCMU_tSecAddr[0].SessionNb = *instrSession;

    for (i=1 ; i<30 ; i++)
        p_globalvarBase->RSCMU_tSecAddr[i].SessionNb = RSCMU_OFF;

    // Clear error stack
    CHECKERR(viPrintf(*instrSession, "*CLS\r\n"));

    // Get the current options (= Function Group Names) in the CMU.

    CHECKERR(RSCMU_getTabSecAdd (*instrSession, errorMessage));

    if (RSCMU_Status < 0)
        return RSCMU_Base_errorMessage (*instrSession,RSCMU_Status,errorMessage);

    // initialize type and default FW version of CMU
    if (IDQuery == RSCMU_DO_QUERY)
    {
        CHECKRETMSG(RSCMU_getViString (*instrSession, "*IDN?", rdBuf,RD_BUFFER_SIZE,0));

        // Translates characters to uppercase:
        for (i=0; i<strlen (rdBuf); i++)
          rdBuf[i] = (ViChar) toupper(rdBuf[i]);

        // Check the CMU response
        if ((strstr(rdBuf, "ROHDE&SCHWARZ,CMU") == NULL))
        {
            if ((strstr(rdBuf, "ROHDE&SCHWARZ,CBT") == NULL))
                return RSCMU_Base_errorMessage (*instrSession, RSCMU_DO_FAIL_IF_NO_QUERY, errorMessage);
            else
                p_globalvarBase->RSCMU_tGlobal.RSCMU_Type = RSCBT;
        }
        else
        {
            // Base CMU FW available
            if ( strstr ( rdBuf, "CMU300" ) || strstr ( rdBuf, "CMU 300" ) )
                p_globalvarBase->RSCMU_tGlobal.RSCMU_Type = RSCMU_300;
            else
                p_globalvarBase->RSCMU_tGlobal.RSCMU_Type = RSCMU_200;
        }

         // initialize FW Number
        CHECKRET(RSCMU_Base_RevisionQuery (*instrSession, p_globalvarBase->RSCMU_tGlobal.DrvVersion, p_globalvarBase->RSCMU_tGlobal.CmuVersion, errorMessage));
        CHECKRET(RSCMU_CurrentVersion (*instrSession, &Version, errorMessage));

        p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version = Version;
    }
    else
    {
        // Base CMU FW available
        p_globalvarBase->RSCMU_tGlobal.RSCMU_Type = RSCMU_200;
        strcpy (p_globalvarBase->RSCMU_tGlobal.CmuVersion, "V2.10");
        p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version = RSCMU200_V2_10;
    }

    // Reset instrument
    if (reset == RSCMU_RESET_ON)
    {
        CHECKRET(RSCMU_Reset (*instrSession,  errorMessage));
    }

    CHECKRET(RSCMU_setCheckStatus(*instrSession, VI_TRUE));
    // Setting of registers and Remote/Local Switching

        RETMSG(RSCMU_sendCmd(*instrSession, "*SRE 0;*ESE 0"));

}

/*
 *-----------------------------------------------------------------------------
 * Common Commands
 * ------------------
 *  The common commands are taken from the IEEE 488.2 (IEC 625-2) standard.
 *  These commands have the same effect on different devices. The headers of
 *  these commands consist of "*" followed by three letters.
 *  Many common commands are related to the status reporting system which is
 * described in detail in chapter 5.
 *-----------------------------------------------------------------------------
*/

ViStatus _VI_FUNC RSCMU_SystGtl (ViSession instrSession,
                                 ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;

    CHECKRET(RSCMU_viWrite (instrSession, "*GTL", strlen("*GTL")));
    return RSCMU_Status;
}

ViStatus _VI_FUNC RSCMU_Systllo (ViSession instrSession,
                                 ViBoolean enable,
                                 ViChar    errorMessage[])
{
    ViString    list[]= {"FALSe", "TRUE", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "*LLO",
                         enable, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 * STATUS FUNCTIONS
 * ------------------
 *  The STATus functions control the SCPI-defined status reporting
 *  structures.
 *  The purpose and definition of status registers is given in
 *  chapter 5, section "Status Reporting System". Unless otherwise
 *  stated, all the following commands are SCPI-confirmed.
 *
 *-----------------------------------------------------------------------------
 *    OPERATION REGISTERS
 *    These functions control the SCPI-defined Operation registers.
 *-----------------------------------------------------------------------------
 *
 *-----------------------------------------------------------------------------
 *      STATus OPERation Register
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Event Part
/* Purpose : This function queries and deletes the contents of the EVENt
/*           part of the STATus:OPERation register.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOper (ViSession  instrSession,
                                       ViInt32   *eventPart,
                                       ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViInt32(instrSession, "STAT:OPER?", eventPart,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Condition Part
/* Purpose : This function queries the contents of the CONDition part of
/*           the STATus:OPERation register.
/*           Reading this register is nondestructive.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCond (ViSession  instrSession,
                                           ViInt32   *conditionPart,
                                           ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViInt32 (instrSession,
                           "STAT:OPER:COND?",
                           conditionPart,
                           RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Positive Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the PTRansition part of the STATus:OPERation register.
/*           If a bit is set the corresponding 0-to-1 transition in the
/*           CONDition part (indicating that an event in the hardware
/*           occurred) is entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperPtr (ViSession instrSession,
                                          ViInt32   positiveTranstionPart,
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                          "STAT:OPER:PTR",
                          positiveTranstionPart,
                          0,
                          RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Negative Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the NTRansition part of the STATus:OPERation register. If a
/*           bit is set the corresponding 0-to-1 transition in the CONDition
/*           part (indicating that an event in the hardware disappeared) is
/*           entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperNtr (ViSession instrSession,
                                          ViInt32   negativeTranstionPart,
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                          "STAT:OPER:NTR",
                          negativeTranstionPart,
                          0, RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Enable Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the ENABLe part of the STATus:OPERation register. If a bit is
/*           set the corresponding event is reported in the summary bit of
/*           the status byte.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperEnab (ViSession instrSession,
                                           ViInt32   enablePart,
                                           ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                          "STAT:OPER:ENAB",
                          enablePart,
                          0, RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *      STATus:OPERation:CMU:SUM1|2 Registers
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Event Part
/* Purpose : This function queries and deletes the contents of the EVENt part
/*           of the STATus:OPERation:CMU:SUM1|2 register.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSum (ViSession  instrSession,
                                             ViInt32    Register,
                                             ViInt32   *eventPart,
                                             ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld?", Register);
    RETMSG(RSCMU_getViInt32(instrSession, wrtBuf, eventPart, RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Condition Part
/* Purpose : This function queries the contents of the CONDition part of the
/*           STATus:OPERation:CMU:SUM1|2 register.
/*           Reading this register is nondestructive.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumCond (ViSession  instrSession,
                                                 ViInt32    Register,
                                                 ViInt32   *conditionPart,
                                                 ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:COND?", Register);
    RETMSG(RSCMU_getViInt32(instrSession, wrtBuf, conditionPart, RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Set Positive Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the PTRansition part of the STATus:OPERation:CMU:SUM1|2
/*           register.
/*           If a bit is set the corresponding 0-to-1 transition in the
/*           CONDition part (indicating that an event in the hardware
/*           occurred) is entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumPtr (ViSession instrSession,
                                                ViInt32   Register,
                                                ViInt32   positiveTranstionPart,
                                                ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:PTR", Register);

    RETMSG(RSCMU_setViInt32(instrSession,
                          wrtBuf,
                          positiveTranstionPart,
                          0, RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Set Negative Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the NTRansition part of the STATus:OPERation:CMU:SUM1|2
/*           register. If a bit is set the corresponding 0-to-1 transition in
/*           the CONDition part (indicating that an event in the hardware
/*           disappeared) is entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumNtr (ViSession instrSession,
                                                ViInt32   Register,
                                                ViInt32   negativeTranstionPart,
                                                ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:NTR", Register);

    RETMSG(RSCMU_setViInt32(instrSession,
                          wrtBuf,
                          negativeTranstionPart,
                          0,RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Set Enable Part
/* Purpose : This command sets a number to be interpreted as a bit pattern in
/*           the ENABle part of the STATus:OPERation:CMU:SUM1|2 register. If
/*           a bit is set the corresponding event is reported in the summary
/*           bit of the status byte.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumEnab (ViSession instrSession,
                                                 ViInt32   Register,
                                                 ViInt32   enablePart,
                                                 ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:ENAB", Register);

    RETMSG(RSCMU_setViInt32(instrSession,
                          wrtBuf,
                          enablePart,
                          0,RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER3));
}

/*
 *-----------------------------------------------------------------------------
 *        STATus:OPERation:CMU:SUM1|2:CMU1|to|15 Registers
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Event Part
/* Purpose : This function queries and deletes the contents of the EVENt part
/*           of the STATus:OPERation:CMU:SUM1|2:CMU1|to|15 register.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumCmu (ViSession  instrSession,
                                                ViInt32    Register,
                                                ViInt32    SubRegister,
                                                ViInt32   *eventPart,
                                                ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, SubRegister, 1, RSCMU_SUB_REG_MAX, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:CMU%ld?", Register, SubRegister);

    RETMSG(RSCMU_getViInt32(instrSession,
                          wrtBuf,
                          eventPart,
                          RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Condition Part
/* Purpose : This function queries the contents of the CONDition part of the
/*           STATus:OPERation:CMU:SUM1|2:CMU1|to|15 register.
/*           Reading this register is nondestructive.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumCmuCond (ViSession  instrSession,
                                                    ViInt32    Register,
                                                    ViInt32    SubRegister,
                                                    ViInt32   *conditionPart,
                                                    ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, SubRegister, 1, RSCMU_REG_MAX, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:CMU%ld:COND?", Register, SubRegister);
    RETMSG(RSCMU_getViInt32(instrSession,
                          wrtBuf,
                          conditionPart,
                          RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Set Positive Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the PTRansition part of the
/*           STATus:OPERation:CMU:SUM1|2:CMU1|to|15 register.
/*           If a bit is set the corresponding 0-to-1 transition in the
/*           CONDition part (indicating that an event in the hardware
/*           occurred) is entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumCmuPtr (ViSession instrSession,
                                                   ViInt32   Register,
                                                   ViInt32   SubRegister,
                                                   ViInt32   positiveTranstionPart,
                                                   ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, SubRegister, 1, RSCMU_SUB_REG_MAX, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:CMU%ld:PTR", Register, SubRegister);

    RETMSG(RSCMU_setViInt32(instrSession,
                          wrtBuf,
                          positiveTranstionPart,
                          0, RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Set Negative Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the NTRansition part of the
/*           STATus:OPERation:CMU:SUM1|2:CMU1|to|15 register. If a bit is set
/*           the corresponding 0-to-1 transition in the CONDition part
/*           (indicating that an event in the hardware disappeared) is
/*           entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumCmuNtr (ViSession instrSession,
                                                   ViInt32   Register,
                                                   ViInt32   SubRegister,
                                                   ViInt32   negativeTranstionPart,
                                                   ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, SubRegister, 1, RSCMU_SUB_REG_MAX, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:CMU%ld:NTR", Register, SubRegister);

    RETMSG(RSCMU_setViInt32(instrSession,
                          wrtBuf,
                          negativeTranstionPart,
                          0, RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Set Enable Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the ENABLe part of the STATus:OPERation:CMU:SUM1|2:CMU1|to|15
/*           register. If a bit is set the corresponding event is reported in
/*           the summary bit of the status byte.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuSumCmuEnab (ViSession instrSession,
                                                    ViInt32   Register,
                                                    ViInt32   SubRegister,
                                                    ViInt32   enablePart,
                                                    ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Register, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, SubRegister, 1, RSCMU_SUB_REG_MAX, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "STAT:OPER:CMU:SUM%ld:CMU%ld:ENAB", Register, SubRegister);

    RETMSG(RSCMU_setViInt32(instrSession,
                          wrtBuf,
                          enablePart,
                          0, RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Query All Operation Registers
/* Purpose : This command queries the EVENt parts of all
/*           STATus:OPERation:CMU:SUM1|2:CMU1| to |15 registers. The result
/*           is returned as a list of 30 integer values separated by commas.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuAll (ViSession instrSession,
                                             ViInt32   eventParts[],
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar* rdBuf = VI_NULL;
    ViChar* p2buf;
    ViInt16 i;
    GET_WRT_BUF(instrSession);

    strcpy (wrtBuf,"STAT:OPER:CMU:ALL?");
    CHECKERR (RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));
    CHECKERR (RSCMU_ReadDataUnknownLength(instrSession, &rdBuf, VI_NULL));
    CHECKERR (RSCMU_sysStatus(instrSession, VI_NULL));
    if (RSCMU_Status < 0)
    {
        if (rdBuf) free(rdBuf);
        return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
    }

    if (rdBuf && (RSCMU_Status >= 0))
    {
        p2buf = strtok (rdBuf, ",");
        for (i = 0; i<30; i++)
        {
            eventParts[i] = atol (p2buf);
            p2buf = strtok(VI_NULL, ",");
        }
    }

    if (rdBuf) free(rdBuf);

    return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
}

/*===========================================================================*/
/* Function: Clear All Operation Registers
/* Purpose : This command clears the EVENt parts of all
/*           STATus:OPERation:CMU:SUM1|2:CMU1| to |15 registers.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperCmuCle (ViSession instrSession,
                                             ViChar    errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "STAT:OPER:CMU:CLE"));
}

/*
 *-----------------------------------------------------------------------------
 *        Questionnable Register
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Event Part
/* Purpose : This function queries and deletes the contents of the EVENt part
/*           of the STATus:QUEStionable register.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatQues (ViSession  instrSession,
                                       ViInt32   *eventPart,
                                       ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViInt32(instrSession,
                          "STAT:QUES?",
                          eventPart,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Condition Part
/* Purpose : This function queries the contents of the CONDition part of the
/*           STATus:QUEStionable register.
/*           Reading this register is nondestructive.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatQuesCond (ViSession  instrSession,
                                           ViInt32   *conditionPart,
                                           ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViInt32(instrSession,
                          "STAT:QUES:COND?",
                          conditionPart,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Positive Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the PTRansition part of the STATus:QUEStionable register.
/*           If a bit is set the corresponding 0-to-1 transition in the
/*           CONDition part (indicating that an event in the hardware
/*           occurred) is entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatQuesPtr (ViSession instrSession,
                                          ViInt32   positiveTranstionPart,
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                          "STAT:QUES:PTR",
                          positiveTranstionPart,
                          0,RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Negative Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the NTRansition part of the STATus:QUEStionable register. If
/*           a bit is set the corresponding 0-to-1 transition in the
/*           CONDition part (indicating that an event in the hardware
/*           disappeared) is entered into the EVENt part.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatQuesNtr (ViSession instrSession,
                                          ViInt32   negativeTranstionPart,
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                          "STAT:QUES:NTR",
                          negativeTranstionPart,
                          0,RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Enable Part
/* Purpose : This Function sets a number to be interpreted as a bit pattern
/*           in the ENABLe part of the STATus:QUEStionable register. If a bit
/*           is set the corresponding event is reported in the summary bit of
/*           the status byte.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatQuesEnab (ViSession instrSession,
                                           ViInt32   enablePart,
                                           ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                          "STAT:QUES:ENAB",
                          enablePart,
                          0,RSCMU_REG_MAX,
                          RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Reset Status Registers
/* Purpose : This command sets the edge detectors (PTRansition and
/*           NTRansition parts) and the ENABle registers of the
/*           STATus:OPERation and the STATus:QUEStionable registers to
/*           defined values:
/*           * PTRansition is set to 32767 (0x7FFF), i.e. all hardware events
/*           are detected and transferred to the EVENt register.
/*           * NTRansition is set to 0, i.e. a hardware event that disappears
/*           does not cause any change in the EVENt register.
/*           * The ENABle registers are also set to 0 so that events are not
/*           reported in the status byte.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatPres (ViSession instrSession,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "STAT:PRES"));
}

/*
 *-----------------------------------------------------------------------------
 *        Symbolic Status Event Register Evaluation
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function:
/* Purpose : This function returns the next function group where a event was
/*           reported
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperEvenSadd (ViSession instrSession,
                                               ViInt32   *nextsecondaryaddress,
                                               ViChar    functiongroupname[],
                                               ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      buffer[RD_BUFFER_SIZE];

    memset (buffer, 0, RD_BUFFER_SIZE);

    // Fetch the status of the RF spectrum measurement:
    CHECKERR(RSCMU_getViString(instrSession, "STAT:OPER:EVEN:SADD?", buffer, RD_BUFFER_SIZE, 0));

    CHECKERR(sscanf(buffer,"%ld,%[^,]", nextsecondaryaddress, functiongroupname) != 2 ?
            RSCMU_ERROR_INTERPRETING_RESPONSE:
            RSCMU_Status);

    return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
}

/*===========================================================================*/
/* Function:
/* Purpose : This function returns the next function group where an event was
/*           reported
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperSymb (ViSession instrSession,
                                           ViChar    events[],
                                           ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_RD_BUF( instrSession );

    // Fetch the status of the RF spectrum measurement:
    CHECKRETMSG(RSCMU_getViString (instrSession, "STAT:OPER:SYMB?", rdBuf,
             RD_BUFFER_SIZE,0));

    sscanf (rdBuf, "%[^\r\n]", events);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function:
/* Purpose : This function enables event reporting for one or several events
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_StatOperSymbEnab (ViSession instrSession,
                                               ViChar    event[],
                                               ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViString(instrSession,
                             "STAT:OPER:SYMB:ENAB",
                             event,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Synchronization
/* Purpose : This function sets the reference frequency
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_ConfSyncFreqRef (ViSession  instrSession,
                                              ViReal64   Frequency,
                                              ViInt32    Mode,
                                              ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrMode[]= {"INT","EXT", VI_NULL};

    CHECKRETMSG(RSCMU_setViReal64(instrSession,
                               "CONF:SYNC:FREQ:REF",
                               Frequency*1.0E+6,
                               0.01E+6, 52.0E+6,
                               RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_setEnum(instrSession,
                         "CONF:SYNC:FREQ:REF:MODE",
                         Mode,
                         StrMode,
                         RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Synchronization Reference Frequency is locked
/* Purpose : This function indicates whether the CMU is synchronized to the
/*           (external) reference frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SenseSyncFreqRefLock (ViSession instrSession,
                                                   ViInt32  *RefFreqLocked,
                                                   ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_getEnum(instrSession,
                         "SENS:SYNC:FREQ:REF:LOCK?",
                         RefFreqLocked, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));

}

/*
 *-----------------------------------------------------------------------------
 * SETUP - BASIC DEVICE SETTINGS
 * ------------------
 * The SETup subsystem contains the functins for global
 * configuration of the remote control parameters, the serial
 * interfaces, the options, date and time, the keyboard and beeper.
 * It corresponds to the Setup menu opened via the SETUP key on the
 * front panel.
 *-----------------------------------------------------------------------------
 *  REMOTE
 *  The REMote subsystem contains the functions for configuration of
 *  the remote control parameters.
 *  It corresponds to the Remote index card in the Setup menu opened
 *  via the SETUP key on the front panel.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Primary Address
/* Purpose : This function sets the primary address of the GPIB driver which
/*           is used to address the device (CMU).
/*           It is equivalent to SYSTem:COMMunicate:GPIB[:SELF]:ADDRess (see
/*           after).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystRemAddrPrim (ViSession instrSession,
                                              ViInt32   primaryAddress,
                                              ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                            "SYST:REM:ADDR:PRIM",
                            primaryAddress,
                            0, RSCMU_ADDR_PRIM_MAX,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Secondary Address
/* Purpose : This function assigns the secondary addresses to the remote
/*           control modes (function groups) of the GPIB driver (compare next
/*           command).
/*           If a secondary address is successively assigned to different
/*           function groups the previous assignments are overwritten.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystRemAddrSec (ViSession instrSession,
                                             ViInt32   secondaryAddress,
                                             ViInt32   functionGroupName,
                                             ViChar    errorMessage[])
{

    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      FctGrpBuffer[256] = "";
    struct      RSCMU_Base_Globals *p_globalvarBase;
    int         i_count;
    ViInt32     status_byte;
    GET_WRT_BUF(instrSession);

    CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

  // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, secondaryAddress, 0, RSCMU_ADDR_SEC_MAX, RSCMU_ERROR_PARAMETER2, errorMessage);

    if ( pvar_base_globals->RSCMU_Type == RSCMU_200 )
    {
        switch (functionGroupName)
        {
        case RSCMU_NOT_MAPPED:
            strcpy (FctGrpBuffer, "NONE");
            break;
        case RSCMU_BASE:
            strcpy (FctGrpBuffer, "\"BASE\"");
            break;
        case RSCMU_RF_NSIG:
            strcpy (FctGrpBuffer, "\"RF_NSig\"");
            break;
        case RSCMU_AUDIO_NSIG:
            strcpy (FctGrpBuffer, "\"AUDIO_NSig\"");
            break;
        case RSCMU_GSM400MS_NSIG:
            strcpy (FctGrpBuffer, "\"GSM400MS_NSig\"");
            break;
        case RSCMU_GSM400MS_SIG:
            strcpy (FctGrpBuffer, "\"GSM400MS_Sig\"");
            break;
        case RSCMU_GSM850MS_NSIG:
            strcpy (FctGrpBuffer, "\"GSM850MS_NSig\"");
            break;
        case RSCMU_GSM850MS_SIG:
            strcpy (FctGrpBuffer, "\"GSM850MS_Sig\"");
            break;
        case RSCMU_GSM900MS_NSIG:
            strcpy (FctGrpBuffer, "\"GSM900MS_NSig\"");
            break;
        case RSCMU_GSM900MS_SIG:
            strcpy (FctGrpBuffer, "\"GSM900MS_Sig\"");
            break;
        case RSCMU_GSM1800MS_NSIG:
            strcpy (FctGrpBuffer, "\"GSM1800MS_NSig\"");
            break;
        case RSCMU_GSM1800MS_SIG:
            strcpy (FctGrpBuffer, "\"GSM1800MS_Sig\"");
            break;
        case RSCMU_GSM1900MS_NSIG:
            strcpy (FctGrpBuffer, "\"GSM1900MS_NSig\"");
            break;
        case RSCMU_GSM1900MS_SIG:
            strcpy (FctGrpBuffer, "\"GSM1900MS_Sig\"");
            break;
        case RSCMU_IS136800MS_NSIG:
            strcpy (FctGrpBuffer, "\"IS136800MS_NSig\"");
            break;
        case RSCMU_IS136800MS_SIG:
            strcpy (FctGrpBuffer, "\"IS136800MS_Sig\"");
            break;
        case RSCMU_IS1361900MS_NSIG:
            strcpy (FctGrpBuffer, "\"IS1361900MS_NSig\"");
            break;
        case RSCMU_IS1361900MS_SIG:
            strcpy (FctGrpBuffer, "\"IS1361900MS_Sig\"");
            break;
        case RSCMU_AMPSMS_NSIG:
            strcpy (FctGrpBuffer, "\"AMPSMS_NSig\"");
            break;
        case RSCMU_AMPSMS_SIG:
            strcpy (FctGrpBuffer, "\"AMPSMS_Sig\"");
            break;
        case RSCMU_CDMA800MS_NSIG:
            strcpy (FctGrpBuffer, "\"CDMA_Cell_MS_NSig\"");
            break;
        case RSCMU_CDMA800MS_SIG:
            strcpy (FctGrpBuffer, "\"CDMA_Cell_MS_Sig\"");
            break;
        case RSCMU_CDMA1900MS_NSIG:
            strcpy (FctGrpBuffer, "\"CDMA_PCS_MS_NSig\"");
            break;
        case RSCMU_CDMA1900MS_SIG:
            strcpy (FctGrpBuffer, "\"CDMA_PCS_MS_Sig\"");
            break;
        case RSCMU_BLUETOOTH_NSIG:
            strcpy( FctGrpBuffer, "\"BLUETOOTH_NSig\"" );
            break;
        case RSCMU_BLUETOOTH_SIG:
            strcpy( FctGrpBuffer, "\"BLUETOOTH_Sig\"" );
            break;
        case RSCMU_WCDMA1900FDDMS_NSIG:
            strcpy( FctGrpBuffer, "\"WCDMA19UEFDD_NSig\"" );
            break;
        case RSCMU_WCDMA1900FDDMS_SIG:
          strcpy( FctGrpBuffer, "\"WCDMA19UEFDD_Sig\"" );
          break;
        case RSCMU_CDMA2K450MS_NSIG:
            strcpy( FctGrpBuffer, "\"CDMA2K450MS_NSig\"" );
            break;
        case RSCMU_CDMA2K450MS_SIG:
            strcpy( FctGrpBuffer, "\"CDMA2K450MS_Sig\"" );
            break;
        case RSCMU_CDMA2KCELLMS_NSIG:
            strcpy( FctGrpBuffer, "\"CDMA2KCellMS_NSig\"" );
            break;
        case RSCMU_CDMA2KCELLMS_SIG:
            strcpy( FctGrpBuffer, "\"CDMA2KCellMS_Sig\"" );
            break;
        case RSCMU_CDMA2KPCSMS_NSIG:
            strcpy( FctGrpBuffer, "\"CDMA2KPCSMS_NSig\"" );
            break;
        case RSCMU_CDMA2KPCSMS_SIG:
            strcpy( FctGrpBuffer, "\"CDMA2KPCSMS_Sig\"" );
            break;
        case RSCMU_CDMA2KIMT2KMS_NSIG:
            strcpy( FctGrpBuffer, "\"CDMA2KIMT2KMS_NSig\"" );
            break;
        case RSCMU_CDMA2KIMT2KMS_SIG:
            strcpy( FctGrpBuffer, "\"CDMA2KIMT2KMS_Sig\"" );
            break;
        case RSCMU_EVDO1xAT_NSIG:
            strcpy( FctGrpBuffer, "\"EVDO1XAT_NSig\"" );
            break;
        case RSCMU_EVDO1xAT_SIG:
            strcpy( FctGrpBuffer, "\"EVDO1XAT_Sig\"" );
            break;
        case RSCMU_WCDMA_NB_NSIG:
            strcpy( FctGrpBuffer, "\"WCDMA1900NBFDD_NSig\"" );
            break;
        case RSCMU_WCDMA_NB_SIG:
            strcpy( FctGrpBuffer, "\"WCDMA1900NBFDD_Sig\"" );
            break;
        default:
            return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_PARAMETER3, errorMessage);
        }
    }
    else
    {
        switch (functionGroupName)
        {
        case RSCMU300_NOT_MAPPED:
          strcpy (FctGrpBuffer, "NONE");
          break;
        case RSCMU300_BASE:
          strcpy (FctGrpBuffer, "\"BASE\"");
          break;
        case RSCMU300_RF_NSIG:
          strcpy (FctGrpBuffer, "\"RF_NSig\"");
          break;
        case RSCMU300_AUDIO_NSIG:
          strcpy (FctGrpBuffer, "\"AUDIO_NSig\"");
          break;
        case RSCMU300_GSM400BS_NSIG:
          strcpy (FctGrpBuffer, "\"GSM400BS_NSig\"");
          break;
        case RSCMU300_GSM400BS_SIG:
          strcpy (FctGrpBuffer, "\"GSM400BS_Sig\"");
          break;
        case RSCMU300_GSM850BS_NSIG:
          strcpy (FctGrpBuffer, "\"GSM850BS_NSig\"");
          break;
        case RSCMU300_GSM850BS_SIG:
          strcpy (FctGrpBuffer, "\"GSM850BS_Sig\"");
          break;
        case RSCMU300_GSM900BS_NSIG:
          strcpy (FctGrpBuffer, "\"GSM900BS_NSig\"");
          break;
        case RSCMU300_GSM900BS_SIG:
          strcpy (FctGrpBuffer, "\"GSM900BS_Sig\"");
          break;
        case RSCMU300_GSM1800BS_NSIG:
          strcpy (FctGrpBuffer, "\"GSM1800BS_NSig\"");
          break;
        case RSCMU300_GSM1800BS_SIG:
          strcpy (FctGrpBuffer, "\"GSM1800BS_Sig\"");
          break;
        case RSCMU300_GSM1900BS_NSIG:
          strcpy (FctGrpBuffer, "\"GSM1900BS_NSig\"");
          break;
        case RSCMU300_GSM1900BS_SIG:
          strcpy (FctGrpBuffer, "\"GSM1900BS_Sig\"");
          break;
        case RSCMU300_BLUETOOTH_NSIG:
          strcpy( FctGrpBuffer, "\"BLUETOOTH_NSig\"" );
          break;
        case RSCMU300_BLUETOOTH_SIG:
          strcpy( FctGrpBuffer, "\"BLUETOOTH_Sig\"" );
          break;
        case RSCMU_WCDMA_NB_NSIG:
            strcpy( FctGrpBuffer, "\"WCDMA1900NBFDD_NSig\"" );
            break;
        case RSCMU_WCDMA_NB_SIG:
            strcpy( FctGrpBuffer, "\"WCDMA1900NBFDD_Sig\"" );
            break;
        default:
            return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_PARAMETER3, errorMessage);
        }
    }

    sprintf (wrtBuf, "SYST:REM:ADDR:SEC %ld,", secondaryAddress);
    CHECKERR(RSCMU_setViString(instrSession,
                             wrtBuf,
                             FctGrpBuffer,
                             RSCMU_ERROR_PARAMETER3));

    if (RSCMU_Status == RSCMU_ERROR_STATUS_BYTE)
    {
        RSCMU_Status = RSCMU_getViInt32(instrSession,
                                        "*STB?",
                                        &status_byte,
                                        0);
        if (status_byte == 4)
            return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);
    }

    // Version Manager
    if (p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version >= RSCMU200_V2_12)
    {
        // Put 'tab' in struct GMU_globals
        CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

        for(i_count=1; i_count<=30; i_count++)
            if(strncmp(p_globalvarBase->RSCMU_tSecAddr[i_count].FctGrp, FctGrpBuffer, strlen(FctGrpBuffer))==0)
                if(i_count!=secondaryAddress)
                    strcpy (p_globalvarBase->RSCMU_tSecAddr [i_count].FctGrp, "NONE");

        strcpy (p_globalvarBase->RSCMU_tSecAddr [secondaryAddress].FctGrp, FctGrpBuffer);
    }
    else
        CHECKERR(RSCMU_getTabSecAdd (instrSession, errorMessage));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Task Priority Management
/* Purpose : This function determines the behavior of the R&S CMU if
/*           conflicting measurements or generators are run in parallel.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystRemTpm (ViSession instrSession,
                                         ViBoolean taskPriorityManagement,
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    RETMSG (RSCMU_setEnum (instrSession,
                           "SYST:REM:TPM",
                           taskPriorityManagement,
                           mode_On_Off,
                           VI_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Remote Debug Mode
/* Purpose : This function enables or disables the remote debug mode.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystRemRdm (ViSession instrSession,
                                         ViBoolean remoteDebugMode,
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    RETMSG (RSCMU_setEnum (instrSession,
                           "SYST:REM:RDM",
                           remoteDebugMode,
                           mode_On_Off,
                           VI_ERROR_PARAMETER2));
}

/*
 * ----------------------------------------------------------------------------
 *  COMMUNICATE
 *  The COMMunicate subsystem contains the functions for
 *  configuration of the remote serial interfaces.
 *  It corresponds to the Communicate index card in the Setup menu
 *  opened via the SETUP key on the front panel. The parameters set
 *  in this subsystem are explained in more detail in chapter 8,
 *  section Hardware Interfaces.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Primary Address
/* Purpose : This function sets the primary address of the GPIB driver which
/*           is used to address the device (CMU).
/*           It is equivalent to SYSTem:REMote:ADDRess:PRIMary (see above).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystCommGpibAddr (ViSession instrSession,
                                               ViInt32   primaryAddress,
                                               ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,
                            "SYST:COMM:GPIB:ADDR",
                            primaryAddress,
                            0, RSCMU_ADDR_PRIM_MAX,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Application
/* Purpose : This function returns the current application (connection) of
/*           the serial interface. The numeric suffix distinguishes the two
/*           interfaces COM1 and COM2.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystCommSerAppl (ViSession  instrSession,
                                              ViInt32    interface,
                                              ViInt32   *currentConnection,
                                              ViChar     errorMessage[])
{
    ViString    list[]={"TRAN", "REM", "PRIN", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

  // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, interface, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "SYST:COMM:SER%ld:APPL?", interface);

    RETMSG(RSCMU_getEnum(instrSession,
                         wrtBuf,
                         currentConnection,
                         list,0));
}

/*===========================================================================*/
/* Function: Set Baud Rate
/* Purpose : This function sets the baud rate of the serial interface no. 1
/*           or 2 (connectors COM1 or COM2).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystCommSerBaud (ViSession instrSession,
                                              ViInt32   interface,
                                              ViInt32   baudRate,
                                              ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, interface, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "SYST:COMM:SER%ld:BAUD", interface);

    RETMSG(RSCMU_setViInt32(instrSession,
                            wrtBuf,
                            baudRate,
                            110, 19200,
                            RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Set Data & Stop Bits
/* Purpose : This function sets the numbers of data bits and stop bits for
/*           the serial interface no. 1 or 2 (connectors COM 1 or COM 2).
/*           The sum of data bits and stop bits numbers must be equal to 9.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystCommSerBit (ViSession instrSession,
                                             ViInt32   interface,
                                             ViInt32   dataBitsNumber,
                                             ViInt32   stopBitsNumber,
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, interface, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "SYST:COMM:SER%ld:BITS", interface);
    CHECKRETMSG (RSCMU_setViInt32(instrSession,
                              wrtBuf,
                              dataBitsNumber,
                              RSCMU_DATABITS_MIN, RSCMU_DATABITS_MAX,
                              RSCMU_ERROR_PARAMETER3));

    sprintf (wrtBuf, "SYST:COMM:SER%ld:STOP", interface);

    RETMSG(RSCMU_setViInt32(instrSession,
                            wrtBuf,
                            stopBitsNumber,
                            RSCMU_STOPBITS_MIN, RSCMU_STOPBITS_MAX,
                            RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Set Parity
/* Purpose : This function sets the parity of the serial interface no. 1 or 2
/*           (connectors COM 1 or COM 2).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystCommSerPar (ViSession instrSession,
                                             ViInt32   interface,
                                             ViInt32   parity,
                                             ViChar    errorMessage[])
{
    ViString    list[] = {"NONE", "ODD", "EVEN", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, interface, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "SYST:COMM:SER%ld:PAR", interface);

    RETMSG(RSCMU_setEnum(instrSession,wrtBuf, parity, list, RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Set protocol
/* Purpose : This function sets the handshake protocol of the serial
/*           interface no. 1 or 2 (connectors COM 1 or COM 2).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystCommSerTranPace (ViSession instrSession,
                                                  ViInt32   interface,
                                                  ViInt32   protocol,
                                                  ViChar    errorMessage[])
{
    ViString    list[] = {"NONE", "XON", "ACK", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, interface, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "SYST:COMM:SER%ld:TRAN:PACE", interface);

    RETMSG(RSCMU_setEnum(instrSession, wrtBuf, protocol, list, RSCMU_ERROR_PARAMETER3));
}

/*
 * ----------------------------------------------------------------------------
 *  OPTIONS
 *  The Options subsystem contains the functions for querying
 *  information on the instrument and the available options.
 *  It corresponds to the Options index card in the Setup menu
 *  opened via the SETUP key on the front panel.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Device Informations
/* Purpose : This function returns the information on the device comprising
/*           the manufacturer, model, serial number and firmware version.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystOptInf (ViSession instrSession,
                                         ViChar    deviceInformation[],
                                         ViChar    errorMessage[])
{
    RETMSG(RSCMU_getViString(instrSession, "*IDN?", deviceInformation, RD_BUFFER_SIZE,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Device Informations
/* Purpose : This function returns the information on the device comprising
/*           the manufacturer, model, serial number, firmware version with date,
/*           and Functiongroup.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SystOptInfCurrent (ViSession instrSession,
                                           ViChar deviceInformation[],
                                           ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_getViString(instrSession,
                             "SYST:OPT:INFO:CURR?",
                             deviceInformation, RD_BUFFER_SIZE,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Options
/* Purpose : This function returns a list of all options and equipment
/*           available in the instrument. It is always a query.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystOpt (ViSession instrSession,
                                      ViChar    options[],
                                      ViChar    errorMessage[])
{
    RETMSG(RSCMU_getViString(instrSession, "SYST:OPT?", options, MID_BUFFER_SIZE,
                           RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Hardware equipment
/* Purpose : This function returns a list of the hardware equipment of the CMU
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystMequ (ViSession instrSession,
                                       ViChar    hardwareEquipment[],
                                       ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_50, RSCMU_ERROR_FW_VERSION_3_50));

    RETMSG(RSCMU_getViString(instrSession,
                             "SYSTem:OPTions:MEQuipment?",
                             hardwareEquipment, RD_BUFFER_SIZE,
                             RSCMU_ERROR_PARAMETER2));
}

/*
 * ----------------------------------------------------------------------------
 *  TIME
 *  The Time subsystem contains the functions for the current time
 *  and date.
 *  It corresponds to the Time index card in the Setup menu opened
 *  via the SETUP key on the front panel.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Time
/* Purpose : This function defines the current time and date.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystTim (ViSession instrSession,
                                      ViInt32   timeZone,
                                      ViChar    currentTime[],
                                      ViChar    date[],
                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViInt32     Hour, Minute, Second, Year, Month, Day;
    GET_WRT_BUF(instrSession);

    if (sscanf (currentTime, "%ld:%ld:%ld",&Hour, &Minute, &Second) != 3)
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_PARAMETER3, errorMessage);

    CHECK_VI_INT32_RANGE(instrSession, Hour, 0, 23, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, Minute, 0, 59, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, Second, 0, 59, RSCMU_ERROR_PARAMETER3, errorMessage);

    if (sscanf (date, "%ld.%ld.%ld", &Month, &Day, &Year) != 3)
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_PARAMETER4, errorMessage);

    CHECK_VI_INT32_RANGE(instrSession, Month, 1, 12, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, Day, 1, 31, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, Year, 1980, 2099, RSCMU_ERROR_PARAMETER4, errorMessage);

    // Set Time Zone
    CHECKRETMSG(RSCMU_setViInt32(instrSession, "SYST:TZON", timeZone, -12, 12,
                              RSCMU_ERROR_PARAMETER2));

    // Set Current Time
    sprintf (wrtBuf, "SYST:TIME %ld, %ld, %ld", Hour, Minute, Second);
    CHECKRETMSG(RSCMU_sendCmd (instrSession, wrtBuf));

    // Set Date
    sprintf (wrtBuf, "SYST:DATE %ld, %ld, %ld", Year, Month, Day);
    RETMSG(RSCMU_sendCmd (instrSession, wrtBuf));
}

/*
 * ----------------------------------------------------------------------------
 *  MISCELLANEOUS
 *  The Miscellaneous subsystem contains the functions for setting
 *  the acoustic signal and selecting the external keyboard
 *  assignment.
 *  It corresponds to the Misc. index card in the Setup menu opened
 *  via the SETUP key on the front panel.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set KeyBeep (ON/OFF)
/* Purpose : This function switches the acoustic signal of the CMU on or off.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystMiscKbe (ViSession instrSession,
                                          ViInt32   beeper,
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "SYST:MISC:KBE", beeper, mode_On_Off,
                       RSCMU_ERROR_PARAMETER2));
}

/*
 * ----------------------------------------------------------------------------
 *  GTRMode
 *  The Time subsystem determines the behavior of the CMU in a local remote transition.
 *  the sub-sytem has no equivalent in manual control.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set GTRMode
/* Purpose : This command defines the behavior of the CMU in a local to remote transition.
 * The command is valid for all function groups and test modes, however,
 * its effect depends on the test mode (Signalling or Non Signalling tests):
 *      Signalling In the ON setting:
 *           the connection or call is dropped and the CMU returns to its default
 *           signalling state (e.g. SOFF in the GSMxxx-MS function groups).
 *      Signalling In the OFF setting:
 *           all signalling states are maintained. This makes it possible to switch the instrument
 *           to remote control without dropping a call or connection.
 *           See also operating manual for the different network tests.
 *      Non Signalling In the ON setting:
 *           all generators are switched off.
 *      Non Signalling In the OFF setting, the current operating state of all generators is maintained.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystGtrmComp (ViSession instrSession,
                                           ViInt32   enable,
                                           ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_05, RSCMU_ERROR_FW_VERSION_3_05));

    RETMSG(RSCMU_setEnum(instrSession, "SYST:GTRM:COMP", enable, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Disable Nonvolatile RAM
/* Purpose : This function prevents the CMU from saving measurement settings
/*           to the nonvolatile ram.
/*           This improves the system performance but implies that the current
/*           settings will not be saved for later sessions. Disabling the nonvolatile ram
/*           is recommended on instruments equipped with an FMR 6 Front Module controller.
/*
/*           Note: There is no way to cancel the effect of the function within the current
/*           session, even is the CMU is switched to manual control (Local).
/*           To reenable the nonvolatile ram, the CMU must be rebooted.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystNvlDisable (ViSession instrSession,
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_sendCmd(instrSession, "SYSTem:NONVolatile:DISable"));
}

/*===========================================================================*/
/* Function: Set Keyboard Assignment
/* Purpose : This function selects the driver for the external keyboard.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystMiscKeyb (ViSession instrSession,
                                           ViInt32   keyboard,
                                           ViChar    errorMessage[])
{
    ViString    list[] = {"US","GR", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SYST:MISC:KEYB", keyboard, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Get complete measurement queue
/* Purpose : This function selects the driver for the external keyboard.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystMqu (ViSession instrSession,
                                      ViChar    fctgrp[],
                                      ViChar    rdymeas[],
                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[MID_BUFFER_SIZE];
    ViChar*     str;
    ViChar*     p2buf1;
    ViChar*     p2buf2;

    CHECKRETMSG(RSCMU_getViString(instrSession, "SYST:MQU?", rdBuf, MID_BUFFER_SIZE,
                               0));

    fctgrp[0] = '\0';
    rdymeas[0] = '\0';
    p2buf1 = fctgrp;
    p2buf2 = rdymeas;
    str = strtok(rdBuf, ",");
    while(str)
    {
        p2buf1 += sprintf(p2buf1, "%s,", str);
        str = strtok(NULL, ",");
        p2buf2 += sprintf(p2buf2, "%s,", str);
        str = strtok(NULL, ",");
    }

    /* remove trailing commas */
    *--p2buf1 = '\0';
    *--p2buf2 = '\0';

    return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
}

/*===========================================================================*/
/* Function: Get next measurement queue
/* Purpose : This function selects the driver for the external keyboard.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystMquItem (ViSession instrSession,
                                          ViChar    fctgrp[],
                                          ViChar    rdymeas[],
                                          ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];

    CHECKRETMSG(RSCMU_getViString(instrSession, "SYST:MQU:ITEM?", rdBuf, RD_BUFFER_SIZE,
                               0));

    if (sscanf(rdBuf,"%[^,],%[^\r\n]", fctgrp, rdymeas) != 2)
        RSCMU_Status = RSCMU_ERROR_INTERPRETING_RESPONSE;

    return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
}

/*===========================================================================*/
/* Function: Sense Temperature System
/* Purpose : This function get the internal tempearture of CMU.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SensTempSyst (ViSession instrSession,
                                           ViReal64  *temperature,
                                           ViChar    errorMessage[])
{
    RETMSG(RSCMU_getViReal64(instrSession, "SENS:TEMP:SYST?", temperature,
                             RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------
 * TCP/IP
 * ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------*/



/*===========================================================================*/
/* Primary LH; TCP/IP Default Setting
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPPrimStaticIDefSet (ViSession instrSession,
                                                     ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG (RSCMU_sendCmd (instrSession, "SYSTem:TCPip:PRIMary:DEFault ON"));
}

/*===========================================================================*/
/* Primary LH; Static IP Address
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPPrimStaticIPAddress (ViSession instrSession,
                                                      ViInt32 _1st,
                                                      ViInt32 _2nd,
                                                      ViInt32 _3rd,
                                                      ViInt32 _4th,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_ConfIPAddress (instrSession, "SYSTem:TCPip:PRIMary:STATic:IPADdress",_1st, _2nd, _3rd, _4th, errorMessage);
}


/*===========================================================================*/
/* Primary LH; Static Subnet Mask
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPPrimStaticSubMask (ViSession instrSession,
                                                      ViInt32 _1st,
                                                      ViInt32 _2nd,
                                                      ViInt32 _3rd,
                                                      ViInt32 _4th,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_ConfIPAddress (instrSession, "SYSTem:TCPip:PRIMary:STATic:SMASk",_1st, _2nd, _3rd, _4th, errorMessage);
}


/*===========================================================================*/
/* Primary LH; Static Default Gateway
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPPrimStaticDefGatew (ViSession instrSession,
                                                      ViInt32 _1st,
                                                      ViInt32 _2nd,
                                                      ViInt32 _3rd,
                                                      ViInt32 _4th,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_ConfIPAddress (instrSession, "SYSTem:TCPip:PRIMary:STATic:DGATeway",_1st, _2nd, _3rd, _4th, errorMessage);
}


/*===========================================================================*/
/* Secondary LH; TCP/IP Default Setting
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecStaticIDefSet (ViSession instrSession,
                                                    ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG (RSCMU_sendCmd (instrSession, "SYSTem:TCPip:SECondary:DEFault ON"));
}

/*===========================================================================*/
/* Secondary LH; Mode:
/* This function specifies whether the IP Address information is
/* assigned manually or automatically.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecMode (ViSession instrSession,
                                           ViInt32   mode,
                                           ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"DYNamic","STATic", VI_NULL};

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "SYSTem:TCPip:SECondary:MODE",
                         mode, list,
                         RSCMU_ERROR_PARAMETER2));

}

/*===========================================================================*/
/* Secundary LH; Static IP Address
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecStaticIPAddress (ViSession instrSession,
                                                      ViInt32 _1st,
                                                      ViInt32 _2nd,
                                                      ViInt32 _3rd,
                                                      ViInt32 _4th,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_ConfIPAddress (instrSession, "SYSTem:TCPip:SECondary:STATic:IPADdress",_1st, _2nd, _3rd, _4th, errorMessage);
}

/*===========================================================================*/
/* Secundary LH; Static Subnet Mask
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecStaticSubMask (ViSession instrSession,
                                                      ViInt32 _1st,
                                                      ViInt32 _2nd,
                                                      ViInt32 _3rd,
                                                      ViInt32 _4th,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_ConfIPAddress (instrSession, "SYSTem:TCPip:SECondary:STATic:SMASk",_1st, _2nd, _3rd, _4th, errorMessage);
}

/*===========================================================================*/
/* Secundary LH; Static Default Gateway
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecStaticDefGatew (ViSession instrSession,
                                                      ViInt32 _1st,
                                                      ViInt32 _2nd,
                                                      ViInt32 _3rd,
                                                      ViInt32 _4th,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_ConfIPAddress (instrSession, "SYSTem:TCPip:SECondary:STATic:DGATeway",_1st, _2nd, _3rd, _4th, errorMessage);
}

/*===========================================================================*/
/* Secundary LH; Dynamic Default Gateway
/* This function sets the Hostname, that is assignet to the CMU and
/* transfered to DHCP Server if dynamic IP Addressing is selected.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecDynamicHName (ViSession instrSession,
                                                   ViChar    hostname[],
                                                   ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    sprintf (wrtBuf, "SYSTem:TCPip:SECondary:DYNamic:HNAMe \"%s\"",hostname);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));

}


/*===========================================================================*/
/* Secundary LH; Dynamic Failure Code
/* This function returns the possible cuese for an error during dynamic
/* allocation of the CMU's IP Address. The response unless dynamic IP Address
/*  allocation failed.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecDynamicFCode (ViSession instrSession,
                                                     ViInt32 *failureCode,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));
    RETMSG(RSCMU_getViInt32(instrSession, "SYSTem:TCPip:SECondary:DYNamic:FCODE?",
                                           failureCode,
                                           0));
}


/*===========================================================================*/
/* Secundary LH; Get dynamic DNS
/* This function returns the dynamically acqired CMU Domain Name Server.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecDynamicDNS (ViSession instrSession,
                                                        ViInt32 *_1st,
                                                        ViInt32 *_2nd,
                                                        ViInt32 *_3rd,
                                                        ViInt32 *_4th,
                                                        ViChar  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_GetIPAddress (instrSession, "SYSTem:TCPip:SECondary:DYNamic:DNS?",_1st, _2nd, _3rd, _4th, errorMessage);
}

/*===========================================================================*/
/* Secundary LH; Get dynamic IP Address
/* This function returns the dynamically acqired IP Address of the CMU..
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecDynamicIPAddress (ViSession instrSession,
                                                        ViInt32 *_1st,
                                                        ViInt32 *_2nd,
                                                        ViInt32 *_3rd,
                                                        ViInt32 *_4th,
                                                        ViChar  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_GetIPAddress (instrSession, "SYSTem:TCPip:SECondary:DYNamic:IPADdress?",_1st, _2nd, _3rd, _4th, errorMessage);
}

/*===========================================================================*/
/* Secundary LH; Get dynamic Subnet Mask
/* This function returns the dynamically acqired Subnet Mask of the CMU.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecDynamicSubMask (ViSession instrSession,
                                                        ViInt32 *_1st,
                                                        ViInt32 *_2nd,
                                                        ViInt32 *_3rd,
                                                        ViInt32 *_4th,
                                                        ViChar  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_GetIPAddress (instrSession, "SYSTem:TCPip:SECondary:DYNamic:SMASk?",_1st, _2nd, _3rd, _4th, errorMessage);
}

/*===========================================================================*/
/* Secundary LH; Get dynamic default Gateway
/* This function returns the dynamically acqired default Gateway of the CMU.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_TCPIPSecDynamicDefGatew (ViSession instrSession,
                                                        ViInt32 *_1st,
                                                        ViInt32 *_2nd,
                                                        ViInt32 *_3rd,
                                                        ViInt32 *_4th,
                                                        ViChar  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    return RSCMU_GetIPAddress (instrSession, "SYSTem:TCPip:SECondary:DYNamic:DGATeway?",_1st, _2nd, _3rd, _4th, errorMessage);
}

/* ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------
 * Reset
 * ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------*/
/*===========================================================================*/
/* Function: PReset
/* Purpose : This function sets all parameters of the instrument to their
/*           default values.
/*           It corresponds to the Reset menu opened via the RESET key on the
/*           front panel.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystPres (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "SYST:PRES"));
}

/*===========================================================================*/
/* Function: Reset All
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SystRes (ViSession instrSession, ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_sendCmd(instrSession, "SYST:RES:ALL"));
}

/*===========================================================================*/
/* Function: Reset Current
/* Purpose : This function sets all parameters of the
/*           current function group and test mode to default values.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SystResCurrent (ViSession instrSession, ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_sendCmd(instrSession, "SYST:RES:CURR"));
}

/* ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------
 * File Manager (MMEMory)
 * ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------*/

/*===========================================================================*/
/* Function: Save / Recall Configuration File all Functiongroups
/* Purpose : This command saves the configuration of all function groups and
/* test modes to a single configuration file.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SaveRecallAll (ViSession instrSession,
                                      ViInt32   saveRecall,
                                      ViChar    fileName[],
                                      ViInt32   storageDevice,
                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSaveRecall[] = {"SAVE","RECall", VI_NULL};
    ViString    StrStorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, saveRecall, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER4, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:%s:ALL \"%s\"" , StrSaveRecall[saveRecall], fileName);
    else
      sprintf (wrtBuf, "MMEM:%s:ALL \"%s\",%s" , StrSaveRecall[saveRecall], fileName , StrStorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Save / Recall Configuration File current Functiongroup
/* Purpose : This command saves the configuration of the current function group and
/* test mode to a single configuration file.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SaveRecallCurrent (ViSession instrSession,
                                          ViInt32   saveRecall,
                                          ViChar    fileName[],
                                          ViInt32   storageDevice,
                                          ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSaveRecall[] = {"SAVE","RECall", VI_NULL};
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, saveRecall, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER4, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:%s:CURR \"%s\"" , StrSaveRecall[saveRecall], fileName);
    else
      sprintf (wrtBuf, "MMEM:%s:CURR \"%s\",%s" , StrSaveRecall[saveRecall], fileName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SetDefaultStorageDevice (ViSession instrSession,
                                                        ViInt32 storageDevice,
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "MMEM:MSIS %s" , StrSorageDevice[storageDevice]);
    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SetFileComment (ViSession instrSession,
                                          ViChar    fileComment[],
                                          ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    sprintf (wrtBuf, "MMEM:COMM \"%s\"",fileComment);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_GetFileInfo (ViSession instrSession,
                                             ViChar fileName[],
                                            ViInt32 storageDevice,
                                             ViChar fileInformation[],
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "";

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:INFO? \"%s\"" , fileName);
    else
      sprintf (wrtBuf, "MMEM:INFO? \"%s\",%s" , fileName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_getViString(instrSession, wrtBuf,
                               fileInformation, RD_BUFFER_SIZE,
                               RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_ChangeDirectory (ViSession instrSession,
                                              ViChar    directoryName[],
                                              ViInt32   storageDevice,
                                              ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:CDIR \"\\%s\"" , directoryName);
    else
      sprintf (wrtBuf, "MMEM:CDIR \"\\%s\",%s" , directoryName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));

}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_GetCurrDirectory (ViSession instrSession,
                                               ViChar    currentDirectory[],
                                               ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_getViString(instrSession, "MMEM:DIR?", currentDirectory, RD_BUFFER_SIZE,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_ScanDirectory (ViSession instrSession,
                                            ViChar    directoryList[],
                                            ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_getViString(instrSession, "MMEM:SCAN?", directoryList, RD_BUFFER_SIZE,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_MakeDir (ViSession instrSession,
                                      ViChar    directoryName[],
                                      ViInt32   storageDevice,
                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:MKDir \"%s\"" , directoryName);
    else
      sprintf (wrtBuf, "MMEM:MKDir \"%s\",%s" , directoryName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_RemoveDir (ViSession instrSession,
                                        ViChar    directoryName[],
                                        ViInt32   storageDevice,
                                        ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:RMDir \"%s\"" , directoryName);
    else
      sprintf (wrtBuf, "MMEM:RMDir \"%s\",%s" , directoryName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_DeleteFile (ViSession instrSession,
                                         ViChar    fileName[],
                                         ViInt32   storageDevice,
                                         ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:DEL \"%s\"" , fileName);
    else
      sprintf (wrtBuf, "MMEM:DEL \"%s\",%s" , fileName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_CopyFile (ViSession instrSession,
                                       ViChar    sourceFileName[],
                                       ViInt32   sourceStorageDevice,
                                       ViChar    destFileName[],
                                       ViInt32   destStorageDevice,
                                       ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", VI_NULL};
    GET_WRT_BUF(instrSession);

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, sourceStorageDevice, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, destStorageDevice, 0, 1, RSCMU_ERROR_PARAMETER5, errorMessage);

    sprintf (wrtBuf, "MMEM:COPY \"%s\",%s,\"%s\",%s" , sourceFileName, StrSorageDevice[sourceStorageDevice]
                                                     , destFileName, StrSorageDevice[destStorageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_MoveFile (ViSession instrSession,
                                       ViChar    sourceFileName[],
                                       ViInt32   sourceStorageDevice,
                                       ViChar    destFileName[],
                                       ViInt32   destStorageDevice,
                                       ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", VI_NULL};
    GET_WRT_BUF(instrSession);

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, sourceStorageDevice, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, destStorageDevice, 0, 1, RSCMU_ERROR_PARAMETER5, errorMessage);

    sprintf (wrtBuf, "MMEM:MOVE \"%s\",%s,\"%s\",%s" , sourceFileName, StrSorageDevice[sourceStorageDevice]
                                                     , destFileName, StrSorageDevice[destStorageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_RenameFile (ViSession instrSession,
                                         ViChar    sourceFileName[],
                                         ViChar    destFileName[],
                                         ViInt32   storageDevice,
                                         ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrSorageDevice[] = {"INTERNAL","EXTERNAL", "", VI_NULL};
    GET_WRT_BUF(instrSession);

    //Version Check
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_10, RSCMU_ERROR_FW_VERSION_3_10));

    CHECK_VI_INT32_RANGE(instrSession, storageDevice, 0, 2, RSCMU_ERROR_PARAMETER5, errorMessage);

    if (storageDevice == RSCMU_DEF_STOR_DEVICE)
      sprintf (wrtBuf, "MMEM:REN \"%s\",\"%s\"" , sourceFileName, destFileName);
    else
      sprintf (wrtBuf, "MMEM:REN \"%s\",\"%s\",%s" , sourceFileName, destFileName, StrSorageDevice[storageDevice]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/* ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------
 * Class: CMU Remote Trace
 * ----------------------------------------------------------------------------
 * ----------------------------------------------------------------------------*/

/*===========================================================================*/
/* Function: Remote Trace on Display
/* Purpose : This function qualifies whether the remote report is displayed
/*           on screen.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_RemTraceDisplay (ViSession instrSession,
                                              ViInt32   enable,
                                              ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_07, RSCMU_ERROR_FW_VERSION_3_07));

    RETMSG(RSCMU_setEnum(instrSession, "TRAC:REM:MODE:DISP",
                                        enable, mode_On_Off,
                                        RSCMU_ERROR_PARAMETER2));

}

/*===========================================================================*/
/* Function: Remote Trace on File
/* Purpose : This command qualifies whether the remote report is written to a
/*           file named Remote.trc in the CMU directory: internal\userdata\remote
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_RemTraceFile (ViSession instrSession,
                                           ViInt32   enable,
                                           ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_07, RSCMU_ERROR_FW_VERSION_3_07));

    RETMSG(RSCMU_setEnum(instrSession, "TRAC:REM:MODE:FILE",
                                        enable, mode_On_Off,
                                        RSCMU_ERROR_PARAMETER2));

}

/*===========================================================================*/
/* Function: Default Setting
/* Purpose : This function sets all parameters of the user correction
/*           loss subsystem to their default values.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_DefUserCorrLoss (ViSession instrSession,
                                              ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    RETMSG (RSCMU_sendCmd (instrSession, "DEF:USER:CORR:LOSS ON"));
}

/*===========================================================================*/
/* Function: User Corr Table Enable
/* Purpose : This function enables or disables the entire user correction
/*           table.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_ConfUserCorrLossTablEnab (ViSession instrSession,
                                                       ViBoolean userCorrectionTable,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    RETMSG (RSCMU_setEnum (instrSession,
                           "CONF:USER:CORR:LOSS:TABL:ENAB",
                           userCorrectionTable,
                           mode_On_Off,
                           VI_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: User Corr at Specified Freq
/* Purpose : This function defines the attenuation correction values for each
/*           of the RF connectors at the frequency specified by means of the
/*           numeric suffix <nr>.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_ConfUserCorrLossTablLine (ViSession instrSession,
                                                       ViInt32 tableLine,
                                                       ViBoolean frequencySetting,
                                                       ViReal64 frequencyValue,
                                                       ViBoolean attCorrection_forRF1,
                                                       ViReal64 RF1CorrectionValue,
                                                       ViBoolean attCorrection_forRF2,
                                                       ViReal64 RF2CorrectionValue,
                                                       ViBoolean attCorrection_forRF3,
                                                       ViReal64 RF3CorrectionValue,
                                                       ViBoolean attCorrection_forRF4,
                                                       ViReal64 RF4CorrectionValue,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     p2buf;

    GET_WRT_BUF(instrSession);

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    CHECK_VI_INT32_RANGE(instrSession, tableLine, 1, 50, RSCMU_ERROR_PARAMETER2, errorMessage);

    p2buf = wrtBuf + sprintf (wrtBuf, "CONF:USER:CORR:LOSS:TABL:LINE%ld ", tableLine);

    if (frequencySetting)
        p2buf += sprintf (p2buf,"%.12f,", frequencyValue);
    else
        p2buf += sprintf (p2buf, "OFF,");

    if (attCorrection_forRF1)
        p2buf += sprintf (p2buf,"%.12f,", RF1CorrectionValue);
    else
        p2buf += sprintf (p2buf, "OFF,");

    if (attCorrection_forRF2)
        p2buf += sprintf (p2buf,"%.12f,", RF2CorrectionValue);
    else
        p2buf += sprintf (p2buf, "OFF,");

    if (attCorrection_forRF3)
        p2buf += sprintf (p2buf,"%.12f,", RF3CorrectionValue);
    else
        p2buf += sprintf (p2buf, "OFF,");

    if (attCorrection_forRF4)
        p2buf += sprintf (p2buf,"%.12f", RF4CorrectionValue);
    else
        p2buf += sprintf (p2buf, "OFF");

    RETMSG (RSCMU_sendCmd (instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Get User Corr Table
/* Purpose : This function returns the whole content of the user correction
/*           loss table.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_ConfUserCorrLossTabl (ViSession instrSession,
                                                   ViBoolean frequencySetting[],
                                                   ViReal64 frequencyValue[],
                                                   ViBoolean attCorrection_forRF1[],
                                                   ViReal64 RF1CorrectionValue[],
                                                   ViBoolean attCorrection_forRF2[],
                                                   ViReal64 RF2CorrectionValue[],
                                                   ViBoolean attCorrection_forRF3[],
                                                   ViReal64 RF3CorrectionValue[],
                                                   ViBoolean attCorrection_forRF4[],
                                                   ViReal64 RF4CorrectionValue[],
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     p2buf;
    ViInt32     i = 0;

    GET_BUFS( instrSession );

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    strcpy (wrtBuf,"CONF:USER:CORR:LOSS:TABL?");
    CHECKERR (RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));
    CHECKERR (RSCMU_ReadDataUnknownLength(instrSession, &rdBuf, VI_NULL));
    CHECKERR (RSCMU_sysStatus(instrSession, VI_NULL));

    p2buf = rdBuf;

    for (i = 0; i < 25; i++)
    {
        frequencySetting[i] = (ViBoolean) sscanf (p2buf, "%le", &frequencyValue[i]);

        p2buf = strchr (++p2buf, ',');

        if (!p2buf)
        {
            if (rdBuf) free(rdBuf);
            RETMSG (RSCMU_ERROR_INTERPRETING_RESPONSE);
        }

        attCorrection_forRF1[i] = (ViBoolean) sscanf (p2buf, ",%le", &RF1CorrectionValue[i]);

        p2buf = strchr (++p2buf, ',');

        if (!p2buf)
        {
            if (rdBuf) free(rdBuf);
            RETMSG (RSCMU_ERROR_INTERPRETING_RESPONSE);
        }

        attCorrection_forRF2[i] = (ViBoolean) sscanf (p2buf, ",%le", &RF2CorrectionValue[i]);

        p2buf = strchr (++p2buf, ',');

        if (!p2buf)
        {
            if (rdBuf) free(rdBuf);
            RETMSG (RSCMU_ERROR_INTERPRETING_RESPONSE);
        }

        attCorrection_forRF3[i] = (ViBoolean) sscanf (p2buf, ",%le", &RF3CorrectionValue[i]);

        p2buf = strchr (++p2buf, ',');

        if (!p2buf)
        {
            if (rdBuf) free(rdBuf);
            RETMSG (RSCMU_ERROR_INTERPRETING_RESPONSE);
        }

        attCorrection_forRF4[i] = (ViBoolean) sscanf (p2buf, ",%le", &RF4CorrectionValue[i]);

        if (i < 24)
        {
            p2buf = strchr (++p2buf, ',');
            if (!p2buf)
        {
            if (rdBuf) free(rdBuf);
            RETMSG (RSCMU_ERROR_INTERPRETING_RESPONSE);
        }

            p2buf++;
        }
    }
    if (rdBuf) free(rdBuf);
    RETMSG (0);
}

/*===========================================================================*/
/* Function: Sort Table
/* Purpose : This function sorts the frequency table. Sorting has no effect
/*           to the user correction.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_SortUserCorrLossTabl (ViSession instrSession,
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_60, RSCMU_ERROR_FW_VERSION_3_60));

    RETMSG (RSCMU_sendCmd (instrSession, "SORT:USER:CORR:LOSS:TABL"));
}

/*===========================================================================*/
/* Function: Close
/* Purpose : This function close the Basic Unit session and release the
/*           resources
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Close_Base (ViSession BaseSession, ViChar errorMessage[])
{
    ViStatus RSCMU_Status = VI_SUCCESS;

    struct RSCMU_Base_Globals *p_globalvarBase;

    if ((RSCMU_Status = viGetAttribute (BaseSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase)) < 0)
        return RSCMU_Base_errorMessage (BaseSession, RSCMU_Status, errorMessage);

    // Deallocate all system resources allocated to the Base Session:
    CHECKERR(viClose (BaseSession));

    // Deallocate the space pointed by p_globalvarBase:
    if (p_globalvarBase)
        free (p_globalvarBase);

    return RSCMU_Status;
}

/*****************************************************************************/
/*****************************************************************************/
/*========== Class: RFNonSignalling =========================================*/
/*****************************************************************************/
/*****************************************************************************/
/*
 * ----------------------------------------------------------------------------
 * Initialize
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Initialize
/* Purpose : This function can be called only after the initialization
/*           of the "Basic Unit".It opens the instrument, queries the
/*           instrument for its ID, and initializes the instrument
/*           to a known state.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Init_RF_NSig (ViSession   BaseSession,
                                      ViPSession  RFNSigSession,
                                      ViChar      errorMessage[])
{

  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViSession   globalSession = 0;
  ViInt32     secondAddr = RSCMU_SECONDARY_ADDRESS_INVALID;
  ViInt32     firstFreeSecAddr = RSCMU_SECONDARY_ADDRESS_INVALID;
  ViInt32     comp = -1;
  ViChar      resource[25];
  ViBoolean   Flag = RSCMU_OFF;

  struct RSCMU_Base_Globals *p_globalvarBase;
  struct RSCMU_RF_NSig_Globals *p_globalvarRF;

  /* --- Set standard "C" locale --- */
  {
    char *locale;

    locale = setlocale(LC_ALL, "C");
  }

  // Init the Error Message
  strcpy (errorMessage, "No Error");

  // Get the current options (= Function Group Names) in the CMU.
  if ((RSCMU_Status = viGetAttribute (BaseSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Base_errorMessage (BaseSession, RSCMU_Status, errorMessage);

  //Find the Secondary address of function Group 'RF_NSig'
  while ((comp != 0) && (secondAddr <= RSCMU_SECONDARY_ADDRESS_MAX))
  {
    secondAddr++;
    comp = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp, "\"RF_NSig\"", 9);
    if (comp == 0)
    {
      Flag = RSCMU_ON;
    }
    else
    {
      ViInt32 comp1;
      comp1 = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp, "NONE", 4);
      if (comp1 == 0 && firstFreeSecAddr == RSCMU_SECONDARY_ADDRESS_INVALID)
      {
        firstFreeSecAddr = secondAddr;
      }
    }
  }

  // If not found try to assign or error
  if (Flag == RSCMU_OFF)
  {
    if ((RSCMU_Status = RSCMU_Base_SystRemAddrSec (BaseSession, firstFreeSecAddr, RSCMU_RF_NSIG, errorMessage)) < 0)
      return RSCMU_Base_errorMessage (BaseSession, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);
    Flag = RSCMU_ON;
    secondAddr = firstFreeSecAddr;
  }

  // Hardware Connection
  if (strncmp ("GPIB",p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, 4) == 0)
    sprintf (resource, "%s::%ld::%ld::INSTR",p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address, secondAddr);
  else
    sprintf (resource, "%s::INSTR",p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);

  // take default instrument session
  globalSession = p_globalvarBase->RSCMU_tGlobal.RSCMU_RmSessionDefault;

  if ((RSCMU_Status = viOpen (globalSession, resource, VI_NULL, 3000, RFNSigSession)) != 0)
    return RSCMU_Base_errorMessage (globalSession, RSCMU_Status, errorMessage);

  // Allocate space for "struct RSCMU_RF_NSig_Globals"
  if ((p_globalvarRF = (struct RSCMU_RF_NSig_Globals *)malloc(sizeof(struct RSCMU_RF_NSig_Globals))) == 0)
    return RSCMU_initCleanUp (globalSession, *RFNSigSession, RSCMU_Status);

  memset ( p_globalvarRF, 0, sizeof ( struct RSCMU_RF_NSig_Globals ) );
  if ((RSCMU_Status = viSetAttribute(*RFNSigSession, VI_ATTR_USER_DATA,(ViAttrState)p_globalvarRF)) < 0)
  {
    free( p_globalvarRF );
    return RSCMU_initCleanUp (globalSession, *RFNSigSession, RSCMU_Status);
  }

  // Set initial values of variables for function group 'RF_NSig'.
  strcpy (p_globalvarRF->RSCMU_tGlobal.FGrp_Name, "RF_NSig");
  strcpy (p_globalvarRF->RSCMU_tGlobal.RSCMU_SCPIConnection, p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);
  p_globalvarRF->RSCMU_tGlobal.RSCMU_primary_address = p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address;
  p_globalvarRF->RSCMU_tGlobal.RSCMU_secondary_address = secondAddr;
  p_globalvarRF->RSCMU_tGlobal.RSCMU_TRACE = p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE;
  p_globalvarRF->RSCMU_tGlobal.RSCMU_SessionNb = *RFNSigSession;

    // Base CMU FW available
  strcpy (p_globalvarRF->RSCMU_tGlobal.CmuVersion, p_globalvarBase->RSCMU_tGlobal.CmuVersion);
  p_globalvarRF->RSCMU_tGlobal.RSCMU_FW_Version = p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version;

  if (p_globalvarRF->RSCMU_tGlobal.RSCMU_TRACE == RSCMU_ON)
    p_globalvarRF->RSCMU_tGlobal.RSCMU_AddrTargetFile = p_globalvarBase->RSCMU_tGlobal.RSCMU_AddrTargetFile;

  // Record the execution of this Init Function.
  p_globalvarBase->RSCMU_tSecAddr[secondAddr].SessionNb = *RFNSigSession;

  // Timeout set to RSCMU_TMO_VALUE defined in rscmu200.h (10 s)
  RSCMU_Status = viSetAttribute (*RFNSigSession, VI_ATTR_TMO_VALUE, RSCMU_TMO_VALUE);

  CHECKRET(RSCMU_setCheckStatus(*RFNSigSession, VI_TRUE));
  return RSCMU_Status;
}


/*
 *-----------------------------------------------------------------------------
 * GROUP CONFIGURATION
 * ------------------
 * The functions presented in this section configure the
 * measurements in the function group RF Non Signalling globally,
 * i.e. for all subsequent measurements.
 * They correspond to the settings in the popup menu of the Group
 * Config. softkey located to the left of the header of each main
 * menu.
 *
 *-----------------------------------------------------------------------------
 *    INPUT LEVEL
 *    This subsystem contains the functions for determination or
 *    modification of the input level for the currently used input.
 *    It corresponds to the index card Input Level in the popup menu
 *    Group Configuration.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Input Lev - Maximum
/* Purpose : This Function sets the maximum input level. The permissible
/*           value range depends on the RF input used and the external
/*           attenuation set.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensLevMax (ViSession instrSession,
                                            ViReal64  Level,
                                            ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SENS:LEV:MAX",
                             Level, -37.0,47.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Input Level - Reference
/* Purpose : This Function sets the reference input level. The permissible
/*           value range depends on the RF input used and the external
/*           attenuation set.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensLevRef (ViSession  instrSession,
                                            ViReal64   ReferenceLevel,
                                            ViChar     errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SENS:LEV:REF",
                             ReferenceLevel, -100.0,53.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Input level - Attenuation
/* Purpose : This function sets the attenuation or gain factor for the
/*           input measurement Path.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensLevAtt (ViSession instrSession,
                                            ViInt32   Mode,
                                            ViChar    errorMessage[])
{
    ViString    list[] = {"NORM","LNO","LDIS", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SENS:LEV:ATT",
                         Mode, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Input level - Mode
/* Purpose : This function sets the attenuation or gain factor for the
/*           input measurement Path.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensLevMode (ViSession instrSession,
                                             ViInt32   mode,
                                             ViChar    errorMessage[])
{
    ViString    list[] = {"MAN","AUT", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SENS:LEV:MODE",
                         mode, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *    TRIGGER MODE
 *    The subsystem TRIGger contains the functions for determination
 *    of the trigger mode.
 *    It corresponds to the index card Trigger in the popup menu Group
 *    Configuration.
 * ----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: Trigger Mode
/* Purpose : This function defines the trigger mode.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_TrigSour (ViSession instrSession,
                                          ViInt32   Source,
                                          ViChar    errorMessage[])
{
    ViString    list[] = {"IMM","RFP", "IFP","EXT",VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "TRIG:SOUR",
                         Source, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Threshold
/* Purpose : This function sets the threshold at which the trigger signal
/*           triggers the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_TrigThr (ViSession  instrSession,
                                         ViInt32    TriggerThreshold,
                                         ViChar     errorMessage[])
{
    ViString    list[] = {"LOW","MED","HIGH", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "TRIG:THR",
                         TriggerThreshold, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Trigger Slope
/* Purpose : This Function sets the trigger slope. It is enabled for
/*           trigger Source RFPower and IFPower only.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_TrigSlop (ViSession instrSession,
                                          ViInt32   slope,
                                          ViChar    errorMessage[])
{
    ViString    list[] = {"NEG","POS", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "TRIG:SLOP",
                         slope, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Trigger RF Power
/* Purpose : This Function sets the trigger RF power. It is valid for
/*           trigger Source RFPower. Possible values are Low, Medium and High.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_TrigThrRFPow (ViSession instrSession,
                                              ViInt32 RFPower,
                                              ViChar errorMessage[])
{

    ViString    list[] = {"LOW","MED", "HIGH", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(instrSession, "TRIG:THR:RFP",
                         RFPower, list,
                         RSCMU_ERROR_PARAMETER2));

}

/*===========================================================================*/
/* Function: Set Trigger IF Power
/* Purpose : This Function sets the trigger IF power. It is valid for
/*           trigger Source IFPower. Possible values between -47.0 and 0.0 dB.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_TrigThrIFPow (ViSession instrSession,
                                              ViReal64 IFPower,
                                              ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setViReal64(instrSession, "TRIG:THR:IFP",
                             IFPower, -47.0,0.0,
                             RSCMU_ERROR_PARAMETER2));

}

/*===========================================================================*/
/* Function: Set Trigger External Source
/* Purpose : This Function sets the trigger extrenal source.
/*           Possible values are Pin 6, Pin 7 and Pin 8
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_TrigSourExt (ViSession instrSession,
                                             ViInt32 extSource,
                                             ViChar errorMessage[])
{
    ViString    list[] = {"PIN6","PIN7", "PIN8", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(instrSession, "TRIG:SOUR:EXT",
                         extSource, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Minimum Trigger Gap
/* Purpose : This function defines a gap time (in us) before a trigger event
/*           can be generated. The setting of a numeric value is effective
/*           for Power or External trigger sources.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_TrigMtg (ViSession instrSession,
                                         ViInt32 minimumTriggerGap,
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    if (minimumTriggerGap != 0)
    {
        RETMSG(RSCMU_setViInt32(instrSession, "TRIG:MTG",
                                minimumTriggerGap, 1, 1000,
                                RSCMU_ERROR_PARAMETER2));
    }
    else
    {
        RETMSG(RSCMU_sendCmd(instrSession, "TRIG:MTG OFF"));
    }
    
}

/*
 *-----------------------------------------------------------------------------
 *    I/Q-IF Interface
 *    It corresponds to the index card IQ/IF in the Connection
 *    Control.
 * ----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: I/Q-IF
/* Purpose : This I/Q-IF parameter sets the both RX and TX path
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQIFRXTX (ViSession instrSession,
                                              ViInt32 iqif,
                                              ViChar errorMessage[])
{
    ViString    list[] = {"BYP","BYIQ","XOIO","IOIO","IOXO","FPAT","UDEF", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(instrSession, "CONF:IQIF:RXTX",
                         iqif, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: I/Q-IF RX Path
/* Purpose : This I/Q-IF parameter sets the RX path
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQIFRXPath (ViSession instrSession,
                                                ViInt32 RXPath,
                                                ViChar errorMessage[])
{
    ViString    list[] = {"BYP","BYIQ","XOIO","IOIO","IOXO", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(instrSession, "CONF:IQIF:RXP",
                         RXPath, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: I/Q-IF TX Path
/* Purpose : This I/Q-IF parameter sets the TX path
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQIFTXPath (ViSession instrSession,
                                                ViInt32 TXPath,
                                                ViChar errorMessage[])
{
    ViString    list[] = {"BYP","BYIQ","XOIO","IOIO","IOXO", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(instrSession, "CONF:IQIF:TXP",
                         TXPath, list,
                         RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 *      LEVEL
 *      This subsystem  contains the functions for determining the level
 *      of the generated RF signals.
 *      It corresponds to the input field RF Level of the panel RF
 *      Generator in the index card Signal in the popup menu Connect.
 *      Control.
 *-----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used as a setting command with the parameter ON this command
/*           sets all parameters of the subsystem to their default values
/*          (the setting OFF results in an error message).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DefRfgConf (ViSession instrSession,
                                             ViInt32 DefaultValue,
                                             ViChar errorMessage[])
{
    return RSCMU_RF_DefaultSettings(instrSession, "RFG", DefaultValue, RSCMU_ERROR_PARAMETER2, errorMessage);
}

/*===========================================================================*/
/* Function: Set RF level
/* Purpose : This function sets the RF generator level in the used
/*           timeslot. The permissible value range depends on the used
/*           RF output of the CMU and the external attenuation set.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgLev (ViSession  instrSession,
                                            ViReal64   RFLevel,
                                            ViChar     errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SOUR:RFG:LEV",
                             RFLevel, -137.0, 13.0,
                             RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 *      FREQUENCY
 *      The subsystem POWer:FREQuency contains the functions for setting
 *      the frequency.
 *      It corresponds to the hotkey assignment Frequency in the
 *      graphical measurement menu Power versus Time.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set RF Frequency
/* Purpose : This function defines the RF generator output frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgFreq (ViSession instrSession,
                                             ViReal64  RFFrequency,
                                             ViInt32   FreqUnit,
                                             ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64Unit(instrSession,"SOUR:RFG:FREQ",
                                 RFFrequency, freqMin, freqMax,
                                 FreqUnit,StrFreqUnit,RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 *      FREQUENCY MODULATION
 *      The subsystem RFGenerator:FM contains the functions for
 *      determination of the internal RF generator frequency modulation.
 *      It corresponds to the Mode and Offset settings in the Generator
 *      panel (in Connection Control, index card Signal and in the
 *      Analyzer/Generator menu).
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set FM state
/* Purpose : This function sets whether the output signal is modulated
/*           i.e. whether the modulation specified with CONF:RFG:FM:DEV
/*           has an effect.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgMod (ViSession instrSession,
                                            ViInt32   modulation,
                                            ViChar    errorMessage[])
{
    ViString    list[]= {"OFF", "SSB", "AM", "FM", "FMST", "DSB", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct RSCMU_Base_Globals     *p_globalvarBase;

  if ((RSCMU_Status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Status;
    if (p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version < RSCMU200_V3_50)
    {
      CHECK_VI_INT32_RANGE(instrSession, modulation, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    }
    else
        if (p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version < RSCMU200_V3_85)
        {
            CHECK_VI_INT32_RANGE(instrSession, modulation, 0, 3, RSCMU_ERROR_PARAMETER2, errorMessage);
        }

    RETMSG(RSCMU_setEnum(instrSession,"SOUR:RFG:MOD",
                         modulation, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set SSB frequency
/* Purpose : This function defines the (single side band) offset
/*           frequency for RF generator modulation.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModSsbFreq (ViSession instrSession,
                                                   ViReal64  FrequencyOffset,
                                                   ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SOUR:RFG:MOD:SSB:FREQ",
                         FrequencyOffset, RSCMU_FREQ_OFFSET_MIN, RSCMU_FREQ_OFFSET_MAX,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set AM modulation index
/* Purpose : This function defines the modulation index for AM modulation,
/*           i.e. the amplitude ration between the modulating AM signal to
/*           the RF carrier signal in percent.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModAmInd (ViSession instrSession,
                                                 ViReal64  ModulationIndex,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SOURce:RFGenerator:MODulation:AM:INDex",
                             ModulationIndex, 0.0, 100.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set FM frequency
/* Purpose : This function defines how fast the frequency of the FM modulated
/*           RF signal changes.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModFMFreq (ViSession instrSession,
                                                  ViReal64  FMFrequency,
                                                  ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SOURce:RFGenerator:MODulation:FM:FREQuency",
                             FMFrequency, 1.0, 50000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set FM deviation
/* Purpose : This function defines the frequency deviation of the FM modulated
/*           RF signal, i.e. the maximum amount by which the frequency of the
/*           modulated RF signal differs from the carrier frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModFMDev (ViSession instrSession,
                                                 ViReal64  FMDeviation,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SOURce:RFGenerator:MODulation:FM:DEViation",
                             FMDeviation, 0.0, 500000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: FM Stereo Audio Frequency
/* Purpose : This function specifies the frequency of the audio tone.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModFMSTAudFreq (ViSession instrSession,
                                                       ViReal64 FMStereoAudioFrequency,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_85,RSCMU_ERROR_FW_VERSION_3_85));

    RETMSG(RSCMU_setViReal64(instrSession, "SOUR:RFG:MOD:FMST:AUD:FREQ",
                             FMStereoAudioFrequency, 1000.0, 100000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: FM Stereo Audio Deviation
/* Purpose : This function specifies the frequency deviation of the audio tone.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModFMSTAudDev (ViSession instrSession,
                                                      ViReal64 FMStereoAudioDeviation,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_85,RSCMU_ERROR_FW_VERSION_3_85));

    RETMSG(RSCMU_setViReal64(instrSession, "SOUR:RFG:MOD:FMST:AUD:DEV",
                             FMStereoAudioDeviation, 0.0, 100000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: FM Stereo Pilot Devitaion
/* Purpose : This function specifies the frequency deviation of the pilot tone.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModFMSTPilDev (ViSession instrSession,
                                                      ViReal64 FMStereoPilotDeviation,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_85,RSCMU_ERROR_FW_VERSION_3_85));

    RETMSG(RSCMU_setViReal64(instrSession, "SOUR:RFG:MOD:FMST:PIL:DEV",
                             FMStereoPilotDeviation, 0.0, 100000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: FM Stereo Audio Mapping
/* Purpose : This function selects the transmission channel for the audio 
/*           signal.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgModFMSTAudMapp (ViSession instrSession,
                                                       ViInt32 FMStereoAudioMapping,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"MONO","LEFT","RIGH", VI_NULL};
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_85,RSCMU_ERROR_FW_VERSION_3_85));

    RETMSG(RSCMU_setEnum(instrSession, "SOUR:RFG:MOD:FMST:AUD:MAPP",
                         FMStereoAudioMapping, list, RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Number of Steps
/* Purpose : This function determines number of steps in Generator list mode
/*           to be processed. Up to 500 frequency/level combinations can be
/*           defined, the number of steps limits the actual list items to be
/*           processed.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgGlisStep (ViSession instrSession,
                                                 ViInt32 steps,
                                                 ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_setViInt32(instrSession, "SOUR:RFG:GLIS:STEP",
                             steps, 1, 50,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Dwell Time
/* Purpose : This function sets the dwell time as the duration for each step
/*           on the defined frequency/level combination list item.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgGlisDwel (ViSession instrSession,
                                                 ViReal64 dwellTime,
                                                 ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_setViReal64(instrSession, "SOUR:RFG:GLIS:DWEL",
                             dwellTime, 0.5, 1000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set List
/* Purpose : This function defines frequence and power list for generator
/*           list mode.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgGlis (ViSession instrSession,
                                             ViInt32 number_ofPoints,
                                             ViInt32 offset,
                                             ViReal64 frequencyList[],
                                             ViReal64 powerList[],
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "";
    ViInt32     i = 0;
    
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    CHECK_VI_INT32_RANGE (instrSession, number_ofPoints, 1, 500, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE (instrSession, offset, 0, 500, RSCMU_ERROR_PARAMETER3, errorMessage);
    
    
    for (i = 1; i < number_ofPoints; i++)
    {
        sprintf (wrtBuf, "SOUR:RFG:GLIS:FREQ%ld %f;:SOUR:RFG:GLIS:PLEV%ld %f", offset + i, frequencyList[i], offset + i, powerList[i]);
        CHECKRETMSG (RSCMU_sendCmd (instrSession, wrtBuf));
    }
    
    RETMSG (RSCMU_Status);
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : This Function sets all parameters of the subsystem to their 
/*           default values
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensAflDef (ViSession instrSession,
                                            ViInt32 defaultSettings,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_sendCmd(instrSession, "DEF:RFG:GLIS ON"));
}

/*===========================================================================*/
/* Function: Get EOL flag
/* Purpose : This function returns the "end of generator list" flag.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcRfgGlisEol (ViSession instrSession,
                                                ViBoolean *EOL,
                                                ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViInt32     i;
    
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    CHECKRETMSG(RSCMU_getEnum(instrSession, "FETC:RFG:GLIS:EOL?",
                         &i, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
    *EOL = (ViBoolean) (i == 1);
    RETMSG (RSCMU_Status);
}

/*===========================================================================*/
/* Function: Init
/* Purpose : This function starts generator list mode
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitRfgGlis (ViSession instrSession,
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_sendCmd(instrSession, "INIT:RFG:GLIS"));
}

/*===========================================================================*/
/* Function: Abort
/* Purpose : This function aborts generator list mode
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborRfgGlis (ViSession instrSession,
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_sendCmd(instrSession, "ABOR:RFG:GLIS"));
}

/*===========================================================================*/
/* Function: Fetch Status
/* Purpose : This function returns the current generator list mode status.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcRfgGlisStat (ViSession instrSession,
                                                 ViInt32 *generatorStatus,
                                                 ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    // Fetch the status of the RF Generator List Mode:
    CHECKRETMSG(RSCMU_getViString(instrSession, "FETC:RFG:GLIS:STAT?",rdBuf, RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(instrSession, str, measurementStatusArr, generatorStatus));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function:
/* Purpose : Sets ouput hopping frequency
/*
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgFhopStat (ViSession instrSession,
                                                 ViBoolean mode,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "SOUR:RFG:FHOP:STAT",
                         mode, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: sets hopping frequency
/* Purpose : This command defines the hopping output frequency of the RF
/*           generator
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgFhopFreq (ViSession instrSession,
                                                 ViReal64  frequencyValue,
                                                 ViInt32   unit,
                                                 ViChar    errorMessage[])
{
    ViReal64    min[] = {100.0E+3, 100.0, 0.1, 0.1E-3,RSCMU_NAN};

    RETMSG(RSCMU_setViReal64Unit(instrSession, "SOUR:RFG:FHOP:FREQ",
                         frequencyValue, min, freqMax,
                         unit, StrFreqUnit, RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Sets frequency hopping mode
/* Purpose : This function selects whether the frequency set is meant to be
/*           relative  or absolute
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgFhopFreqMode (ViSession instrSession,
                                                     ViBoolean mode,
                                                     ViChar    errorMessage[])
{
    ViString    list[]= {"REL", "ABS", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SOUR:RFG:FHOP:FREQ:MODE",
                         mode, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Sets ramping setting
/* Purpose : This function determines whether the RF generator generates a CW
/*           signal or a GSM-like burst signal
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgPulsStat (ViSession instrSession,
                                                 ViBoolean mode,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "SOUR:RFG:PULS:STAT",
                         mode, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 *      GENERATOR CONFIGURATION
 *      The subsystem RFGenerator:BANDwidth contains the function for
 *      determination of the modulation filterbandwidth.
 *      It corresponds to the Modulation Filter setting in the popup
 *      menu Generator Config. in the index card Modulation.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Modulation Filter Bandwidth
/* Purpose : This function defines the modulation filter value.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgBand (ViSession instrSession,
                                             ViInt32   modulationFilter,
                                             ViChar    errorMessage[])
{
    ViString    list[] = {"OFF","F30KHZ","F300KHZ", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SOUR:RFG:BAND",
                         modulationFilter, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT RF GENERATOR
 *    The subsystem RFGenerator contains the functions for
 *    determination of the parameters for internal RF generator
 *    control.
 *    It corresponds to the RF Generator panel in the index card
 *    Signal in the popup menu Connect. Control and in the measurement
 *    menu Analyzer/Generator.
 * ----------------------------------------------------------------------------
 *      GENERATOR CONTROL
 *      This class contains the functions for general control of the RF
 *      generator.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start RF Generator
/* Purpose : Starts RF Generator and reserves ressources
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitRfg (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "INIT:RFG"));
}

/*===========================================================================*/
/* Function: Switch Off RF Generator
/* Purpose : Switches off RF Generator and releases resources
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborRfg (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "ABOR:RFG"));
}

/*===========================================================================*/
/* Function: Generator Status
/* Purpose : This function returns the current generator status.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcRfgStat (ViSession  instrSession,
                                             ViPInt32   RFGeneratorStatus,
                                             ViChar     errorMessage[])
{
    ViString    list[] = {"OFF","RUN","ERR", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKERR(RSCMU_getEnum(instrSession, "FETC:RFG:STAT?",
                           RFGeneratorStatus, list,
                           RSCMU_ERROR_PARAMETER2));

    if (RSCMU_Status == RSCMU_ERROR_INTERPRETING_RESPONSE)
        RSCMU_Status = RSCMU_ERROR_GEN_STAT_UNKNOWN;

    return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
}

/*===========================================================================*/
/*
/*              Generator AuxTX (CMU-B95 or CMU-B96 is required!)
/*
/*===========================================================================*/

/*===========================================================================*/
/* AuxTX Path Select (only CMU-B96)
/* This function sets the AuxTX Path.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_RfgAuxTxPathSel (ViSession instrSession,
                                                 ViInt32 path,
                                                 ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"P1","P2", VI_NULL};

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "SOURce:RFGenerator:AUXTx:PATH", path, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* AuxTX Overange Level (only CMU-B96)
/* This function defines the Aux Tx overange level at Connector Out3
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgOverLevAuxTx (ViSession instrSession,
                                                     ViInt32 ovrlevel,
                                                     ViBoolean enable,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    // Check if function is correctly used:
    CHECK_VI_BOOLEAN_RANGE(instrSession, enable, RSCMU_ERROR_PARAMETER3, errorMessage);

    switch (enable)
    {
      case RSCMU_OFF:
        CHECKRETMSG (RSCMU_sendCmd (instrSession, "SOURce:RFGenerator:AUXTx:OLEVel OFF"));
        break;
      case RSCMU_ON:
        CHECKRETMSG(RSCMU_setViInt32(instrSession, "SOURce:RFGenerator:AUXTx:OLEVel",
                                 ovrlevel, -100, 13,
                                 RSCMU_ERROR_PARAMETER2));
        break;
    }

    return RSCMU_Status;

}

/*===========================================================================*/
/* Function: Set AuxTx level
/* Purpose : This function sets the AuxTx generator level
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgLevAuxTx (ViSession instrSession,
                                                 ViReal64  auxTxlevel,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "SOURce:RFGenerator:AUXTx:LEVel",
                             auxTxlevel, -122.0, -60.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set AuxTx Frequency
/* Purpose : This function defines the AuxTx generator output frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgFreqAuxTx (ViSession instrSession,
                                                  ViReal64  auxTxfrequency,
                                                  ViChar    errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

  if (   (auxTxfrequency <  350.0)
     || ((auxTxfrequency >  550.0)&&(auxTxfrequency <  700.0))
     || ((auxTxfrequency > 1100.0)&&(auxTxfrequency < 1400.0))
     ||  (auxTxfrequency > 2200.0)
     )
     return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_PARAMETER2, errorMessage);

  RETMSG(RSCMU_setViReal64(instrSession,"SOURce:RFGenerator:AUXTx:FREQuency",
                                 auxTxfrequency*1.0E+6,
                                 350.00E+6, 2200.00E+6,
                                 RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set AuxTx level at Path 2 (only CMU-B96)
/* Purpose : This function sets the AuxTx generator level
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgLevAuxTxPath2 (ViSession instrSession,
                                                      ViReal64 auxTxlevel,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViReal64(instrSession, "SOURce:RFGenerator:AUXTx:LEVel:P2",
                             auxTxlevel, -122.0, -60.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set AuxTx Frequency at Path 2 (only CMU-B96)
/* Purpose : This function defines the AuxTx generator output frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourRfgFreqAuxTxPath2 (ViSession instrSession,
                                                       ViReal64 auxTxfrequency,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

  if (   (auxTxfrequency <  700.0)
     || ((auxTxfrequency > 1100.0)&&(auxTxfrequency < 1400.0))
     ||  (auxTxfrequency > 2200.0)
     )
     return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_PARAMETER2, errorMessage);

  RETMSG(RSCMU_setViReal64(instrSession,"SOURce:RFGenerator:AUXTx:FREQuency:P2",
                                 auxTxfrequency*1.0E+6,
                                 700.00E+6, 2200.00E+6,
                                 RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Start AuxTx Generator
/* Purpose : Starts AuxTx Generator and reserves ressources
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitRfgAuxTx (ViSession instrSession,
                                              ViChar    errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

  RETMSG(RSCMU_sendCmd(instrSession, "INITiate:RFGenerator:AUXTx"));
}

/*===========================================================================*/
/* Function: Switch Off AuxTx Generator
/* Purpose : Switches off AuxTx Generator and releases resources
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborRfgAuxTx (ViSession instrSession,
                                              ViChar    errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

  RETMSG(RSCMU_sendCmd(instrSession, "ABORt:RFGenerator:AUXTx"));
}

/*===========================================================================*/
/* Function: AuxTx Generator Status
/* Purpose : This function returns the current AuxTx generator status.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcRfgStatAuxTx (ViSession instrSession,
                                                  ViInt32   *auxTxGenStatus,
                                                  ViChar    errorMessage[])
{
  ViString    list[] = {"OFF","RUN","ERR", VI_NULL};
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

  CHECKERR(RSCMU_getEnum(instrSession, "FETCh:RFGenerator:AUXTx:STATus?",
                         auxTxGenStatus, list,
                         RSCMU_ERROR_PARAMETER2));

  if (RSCMU_Status == RSCMU_ERROR_INTERPRETING_RESPONSE)
      RSCMU_Status = RSCMU_ERROR_GEN_STAT_UNKNOWN;

  return RSCMU_Base_errorMessage(instrSession, RSCMU_Status, errorMessage);
}

/*===========================================================================*/
/* Function: Selection of output connector for the generated AuxTx Signal
/* Purpose : This function determines the output connector to be used for the
/*           generated Aux Tx signal. The bidirectional connectors RF 1 and RF 2
/*           can be used both as input and output connectors in the same measurement.
/*           Only one input and one output may be active simultaneously, so the
/*           previous one is automatically deactivated on switch-over.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AuxTXOutp (ViSession  instrSession,
                                           ViInt32    State,
                                           ViChar     errorMessage[])
{
  ViString    StrState[] = {"RF1","RF2", VI_NULL};
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

  RETMSG(RSCMU_setEnum(instrSession, "OUTPut:AUXTx:STATe", State, StrState,
                         RSCMU_ERROR_PARAMETER2));
}
/*===========================================================================*/
/* Function: External attenuation at the output for the generated AuxTx Signal
/* Purpose : This command assigns an external attenuation value to the
/*           outputs of the instrument (RF 1, RF 2, RF 3 OUT).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AuxTXCorrOut (ViSession  instrSession,
                                              ViInt32    OutputNr,
                                              ViReal64   Absorption,
                                              ViChar     errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  GET_WRT_BUF(instrSession);

  // Version manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

  // Check if the function is correctly used:
  CHECK_VI_INT32_RANGE(instrSession, OutputNr, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);
  sprintf(wrtBuf, "SOURce:CORRection:LOSS:OUTPut%ld:AUXTx:MAGNitude", OutputNr+1);
  RETMSG(RSCMU_setViReal64(instrSession, wrtBuf,
                           Absorption, -50.0, 50.0,
                           RSCMU_ERROR_PARAMETER3));
}

/*
 *-----------------------------------------------------------------------------
 * CONNECTION CONTROL
 * ------------------
 * The functions presented in this section define the analyzer and
 * generator settings, the inputs and outputs used as well as the
 * reference frequency.
 * They correspond to the settings in the popup menu of the
 * Connect. Control softkey located to the right of the header of
 * each main menu.
 *-----------------------------------------------------------------------------
 *    ANALYSER SETTINGS
 *    The subsystem RFANalyzer contains the functions for
 *    determination of the parameters for analyzer control.
 *    It corresponds to the Analyzer Settings panel in the index card
 *    Signal in the popup menu Connect. Control.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set RF Frequency
/* Purpose : This function defines the analyzer input frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_RfanFreq (ViSession instrSession,
                                          ViReal64  RFFrequency,
                                          ViInt32   FreqUnit,
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64Unit(instrSession,"RFAN:FREQ",
                                 RFFrequency, freqMin, freqMax,
                                 FreqUnit,StrFreqUnit,RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Analyser Bandwidth
/* Purpose : This function defines the analyzer bandwidth. If WIDE is
/*           set no restriction is placed on the input frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_RfanBandRes (ViSession  instrSession,
                                             ViInt32    Bandwidth,
                                             ViChar     errorMessage[])
{
    ViString    list[] = {"10HZ","20HZ","30HZ","50HZ", "100HZ", "200HZ", "300HZ", "500HZ",
                          "1KHZ","2KHZ","3KHZ", "5KHZ", "10KHZ", "20KHZ", "30KHZ", "50KHZ",
                          "100KHZ", "200KHZ", "300KHZ", "500KHZ","1MHZ", "WIDE", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession,
                         "RFAN:BAND", Bandwidth, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set measurement Duration
/* Purpose : This function sets the time period during which the peak
/*           power is determined.
/*           With the time 0, the power is immediately measured on the
/*           measurement start,i.e. the instantaneous value is measured.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfRfanPowRtim (ViSession instrSession,
                                                 ViReal64  Duration,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession,
                             "CONF:RFAN:POW:RTIM", Duration,
                             0.0, 1.0,
                             RSCMU_ERROR_PARAMETER2));
}
/*===========================================================================*/
/* Function: Set Tests Repetition
/* Purpose : This function sets the number of statistics cycles,the stop
/*           condition and the stepping mode for the RF-Analysis.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfRfanContRep (ViSession  instrSession,
                                                 ViInt32    Repetition,
                                                 ViInt32    StopCondition,
                                                 ViInt32    StepMode,
                                                 ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Repetition, 0,1,RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    // Configure  tests cycles:
    sprintf (wrtBuf, "CONF:RFAN:CONT:REP %s,%s,%s", StrRepetition[Repetition], StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used as a setting command with the parameter ON this command
/*           sets all parameters of the subsystem to their default values
/*          (the setting OFF results in an error message).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DefRfanConf (ViSession instrSession,
                                             ViInt32 DefaultValue,
                                             ViChar errorMessage[])
{
    return RSCMU_RF_DefaultSettings(instrSession, "RFAN:PMWB:CONT", DefaultValue, RSCMU_ERROR_PARAMETER2, errorMessage);
}

/*
 *-----------------------------------------------------------------------------
 * STATISTICS
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Statistics Cycles
/* Purpose : This function sets the number of statistics cycles,the stop
/*           condition and the stepping mode for the RF Analysis.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfRfanContRepStat (ViSession  instrSession,
                                                               ViInt32    NbRepetition,
                                                               ViInt32    StopCondition,
                                                               ViInt32    StepMode,
                                                               ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, NbRepetition, RSCMU_MULT_MEAS_MIN, RSCMU_MULT_MEAS_MAX, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    // Configure tests cycles:
    sprintf (wrtBuf, "CONF:RFAN:CONT:REP %ld,%s,%s", NbRepetition, StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT RF ANALYSER
 *    The subsystem RFANalyzer contains the functions for
 *    determination of the parameters for analyzer control.
 *    It corresponds to the Analyzer Settings panel in the index card
 *    Signal in the popup menu Connect. Control.
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      The subsystem RFANalyzer contains the functions for general
 *      control of the RF analysis.
 *-----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: Start RF_analysis
/* Purpose : This function start a new RF_analysis measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitRfan (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "INIT:RFAN"));
}

/*===========================================================================*/
/* Function: Abort RF_analysis
/* Purpose : This function abort running RF-Analysis measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborRfan (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "ABOR:RFAN"));
}

/*===========================================================================*/
/* Function: Stop RF_analysis
/* Purpose : This function stops the RF Analysis after current statistic
/*           cycle.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_StopRfan (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "STOP:RFAN"));
}

/*===========================================================================*/
/* Function: Run next RF_analysis Step
/* Purpose : This function runs next RF-Analysis measurement step.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ContRfan (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "CONT:RFAN"));
}

/*===========================================================================*/
/* Function: RF Analyser Measurement Status
/* Purpose : This function returns the status of the current RF Analyser
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcRfanStat (ViSession  RFNSigSession,
                                              ViInt32   *MeasurementStatus,
                                              ViInt32   *CountingNo,
                                              ViChar     _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;

    // Fetch the status of the RF analyser measurement:
    CHECKRETMSG(RSCMU_getViString(RFNSigSession, "FETC:RFAN:STAT?",rdBuf, RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(RFNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRETMSG(RSCMU_getIndexFromString(RFNSigSession, str, measurementStatusArr, MeasurementStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(RFNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",CountingNo))
        *CountingNo = RSCMU_NONE;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the
/*           RF-Analysis is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfRfanErep (ViSession  instrSession,
                                              ViInt32    Mode,
                                              ViChar     errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "CONF:RFAN:EREP",
                         Mode, modeArr, RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *      RESULTS: MEASUREMENT VALUE
 *      The subsystem RFANalyzer:POWer[:RESULT] contains the function
 *      for reading the analyzer power.
 *      The subsystem corresponds to the Analyzer Power panel in the
 *      Analyzer / Generator menu.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Read scalar measured values
/* Purpose : This function  starts a measurement and output the RF power
/*           analysed by the CMU.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadRfanPow (ViSession  instrSession,
                                             ViReal64  *RFPower,
                                             ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViReal64(instrSession, "READ:RFAN:POW?", RFPower,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Fetch Measured Values
/* Purpose : This function  reads out unsynchronized measurement result:
/*           the RF power analysed by the CMU.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcRfanPow (ViSession  instrSession,
                                             ViReal64  *RFPower,
                                             ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViReal64(instrSession, "FETC:RFAN:POW?", RFPower,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Sample Measured Values
/* Purpose : This function  reads out synchronized measurement result:
/*           the RF power analysed by the CMU.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampRfanPow (ViSession  instrSession,
                                             ViReal64  *RFPower,
                                             ViChar     errorMessage[])
{
    RETMSG(RSCMU_getViReal64(instrSession, "SAMP:RFAN:POW?", RFPower,
                             RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 *      INPUT AND OUTPUT CONNECTORS
 *      The subsystem IOConnector contains the commands for
 *      configuration of the input and output connectors.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Selection of input connector
/* Purpose : This command determines the connector to be used for incoming
/*           RF signals. The bidirectional connectors RF 1 and RF 2 can be
/*           used both as input and output connectors in the very same
/*           measurement (see OUTPut[:STATe]).
/*           Only one input and one output may be active at a time, which
/*           is why the cur-rently active one is automatically deactivated
/*           on switchover.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_Inp (ViSession  instrSession,
                                     ViInt32    State,
                                     ViChar     errorMessage[])
{
    ViString    list[] = {"RF1","RF2","RF4", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "INP:STAT", State, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Selection of output connector
/* Purpose : This function determines the connector to be used for
/*           outgoing RF signals. The bidirectional connectors RF 1 and
/*           RF 2 can be used both as input and output connectors in the
/*           very same measurement.
/*           Only one input and one output may be active at a time, which
/*           is why the cur-rently active one is automatically deactivated
/*           on switchover.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_Outp (ViSession  instrSession,
                                      ViInt32    State,
                                      ViChar     errorMessage[])
{
    ViString    list[] = {"RF1","RF2","RF3",VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "OUTP:STAT", State, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: External attenuation at the input
/* Purpose : This command assigns an external attenuation value to the
/*           inputs of the instrument (RF 1, RF 2, RF 4 IN).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensCorrLossInp (ViSession  instrSession,
                                                 ViInt32    InputNr,
                                                 ViReal64   Absorption,
                                                 ViChar     errorMessage[])
{
    ViString    list[]={"INP1", "INP2", "INP4", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, InputNr, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(wrtBuf, "SENS:CORR:LOSS:%s", list[InputNr]);

    RETMSG(RSCMU_setViReal64(instrSession, wrtBuf,
                             Absorption, RSCMU_ATT_MIN, RSCMU_ATT_MAX,
                             RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: External attenuation at the output
/* Purpose : This command assigns an external attenuation value to the
/*           outputs of the instrument (RF 1, RF 2, RF 3 OUT).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourCorrLossOutp (ViSession  instrSession,
                                                  ViInt32    OutputNr,
                                                  ViReal64   Absorption,
                                                  ViChar     errorMessage[])
{
    ViString    list[]={"OUTP1", "OUTP2", "OUTP3", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, OutputNr, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(wrtBuf, "SENS:CORR:LOSS:%s", list[OutputNr]);

    RETMSG(RSCMU_setViReal64(instrSession, wrtBuf,
                             Absorption, RSCMU_ATT_MIN, RSCMU_ATT_MAX,
                             RSCMU_ERROR_PARAMETER3));
}


/*===========================================================================*/
/* Function: Selection of Aux TX Overange Level output connector
/* Purpose : This function determines the connector to be used for
/*           outgoing RF signals. The bidirectional connectors RF 1 and
/*           RF 2 can be used both as input and output connectors in the
/*           very same measurement.
/*           Only one input and one output may be active at a time, which
/*           is why the cur-rently active one is automatically deactivated
/*           on switchover.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AuxTXOvrLv (ViSession  instrSession,
                                            ViInt32    State,
                                            ViChar     errorMessage[])
{
    ViString    list[] = {"RF1","RF2","RF3",VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "OUTPut:AUXTx:OLEVel", State, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: External attenuation at the Aux TX Overange Level output connector
/* Purpose : This command assigns an external attenuation value to the
/*           outputs of the instrument (RF 1, RF 2, RF 4 IN).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AuxTXCorrOvrLv (ViSession  instrSession,
                                                 ViInt32    connector,
                                                 ViReal64   Absorption,
                                                 ViChar     errorMessage[])
{
    ViString    list[]={"OUTPut1", "OUTPut2", "OUTPut3", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, connector, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(wrtBuf, "SENSe:CORRection:LOSS:%s:AUXTx:OLEVel", list[connector]);

    RETMSG(RSCMU_setViReal64(instrSession, wrtBuf,
                             Absorption, RSCMU_ATT_MIN, RSCMU_ATT_MAX,
                             RSCMU_ERROR_PARAMETER3));
}

/*
 *-----------------------------------------------------------------------------
 *      SYNCHRONISATION
 *      Subsystem REFerence (Synchronization)
 *      The subsystem REFerence contains the commands for setting a system clock
 *      specific to the network. This frquency is set in the index card Synch. in
 *      the popup menu Connect. Control.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Switching on/off and setting the system clock
/* Purpose : This commands switches the system clock specific to the
/*           network at the REF OUT 2 connector on or off, and determines
/*           the system clock frequency applied to REF OUT 2.
/*           The frequency entered is internally rounded to one of a
/*           discrete values defined in the operating manual.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SourDmCloc (ViSession  instrSession,
                                            ViInt32    Mode,
                                            ViReal64   Frequency,
                                            ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Setting the system clock
    CHECKRETMSG(RSCMU_setViReal64(instrSession, "SOUR:DM:CLOC:FREQ",
                               Frequency, RSCMU_REF_FRQ_MIN, RSCMU_REF_FRQ_MAX,
                               RSCMU_ERROR_PARAMETER3));

    // Switching the system clock ON or OFF
    RETMSG(RSCMU_setEnum(instrSession, "SOUR:DM:CLOC:STAT",
                         Mode, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT WIDEBAND POWER
 *
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      This subsystem contains the functions for general control of the
 *      narrow-band power measurement.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start new measurement
/* Purpose : This function starts the wide-band power measurement, setting it
/*           to the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitWpow (ViSession instrSession,
                                          ViChar errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_INIT, RSCMU_WPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: Abort running measurement and switch off
/* Purpose : This function stops the wide-band power measurement, setting it
/*           to the status "OFF"
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_AborWpow (ViSession instrSession,
                                           ViChar errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_ABOR, RSCMU_WPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: Next Measurement Step
/* Purpose : This function continues the wide-band power measurement, setting
/*           it to the status "RUN"
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ContWpow (ViSession instrSession,
                                          ViChar errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_CONT, RSCMU_WPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: stop measurement after current statistic cycle
/* Purpose : This function stops the wide-band power measurement, setting it
/*           to the status "STOP"
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_StopWpow (ViSession instrSession,
                                          ViChar errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_STOP, RSCMU_WPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: Test Cycles
/* Purpose : This function sets the repetition mode, the stop condition and
/*           the stepping mode for the measurement.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ConfWpowContRep (ViSession instrSession,
                                                 ViInt32 Repetition,
                                                 ViInt32 StopCondition,
                                                 ViInt32 StepMode,
                                                 ViChar errorMessage[])
{
    return RSCMU_ConfControlRep (instrSession, RSCMU_WPOW, RSCMU_NONE, Repetition, StopCondition, StepMode, errorMessage);
}

/*===========================================================================*/
/* Function: Set Statistic Cycles
/* Purpose : This function sets the number of statistics cycles, the
/*           stop condition and the stepping mode for the measurement.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ConfWpowContRepStat (ViSession instrSession,
                                                     ViInt32 NbRepetition,
                                                     ViInt32 StopCondition,
                                                     ViInt32 StepMode,
                                                     ViChar errorMessage[])
{
    return RSCMU_ConfContRepStat (instrSession, RSCMU_WPOW, RSCMU_NONE, NbRepetition, StopCondition, StepMode, errorMessage);
}

/*===========================================================================*/
/* Function: Wide band Power Measurement Status
/* Purpose : This function returns the status of the current wide-band power
/*           measurement.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_FetcWpowStat (ViSession instrSession,
                                              ViInt32 *MeasurementStatus,
                                              ViInt32 *CountingNo,
                                              ViChar errorMessage[])
{
//  return RSCMU_FetcStatus (instrSession, RSCMU_WPOW, RSCMU_NONE, MeasurementStatus, CountingNo, StatisticNo, errorMessage);
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViChar      rdBuf[RD_BUFFER_SIZE] = "";
  ViChar      TableOfDataValues[100][100];

  // Fetch the status of the wideband power measurement:
  // Version Manager
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

  if ((RSCMU_Status = RSCMU_Query (instrSession, "FETC:WPOW:STAT?", rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
      return RSCMU_Status;

  // Split the multiple data values separated by a comma:
  RSCMU_SeparateDataValues (rdBuf, ",", TableOfDataValues);

  // Get the status of the measurement:
  if ((RSCMU_Status = RSCMU_GetMeasStatVal (MeasurementStatus, TableOfDataValues, 0)) < 0)
    return RSCMU_Status;

  // Get the current value of the statistics counter (the number of the current statistics cycle):
  if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, TableOfDataValues, 1)) < 0)
    return RSCMU_Status;

  return RSCMU_Status;

}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the measurement
/*           is terminated or stopped.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ConfWpowErep (ViSession instrSession,
                                              ViInt32 Report,
                                              ViChar errorMessage[])
{
    return RSCMU_ConfReporting (instrSession, RSCMU_WPOW, RSCMU_NONE, Report, errorMessage);
}

/*
 * ----------------------------------------------------------------------------
 *      RESULTS
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Read Scalar
/* Purpose : This function starts the WPOWer measurement and returns the
/*           result.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ReadWpow (ViSession instrSession,
                                          ViReal64 *Power, ViChar errorMessage[])
{

return RSCMU_RF_NSig_Wpow(instrSession, RSCMU_COMMAND_READ, Power, errorMessage);
}

/*===========================================================================*/
/* Function: Fetch Scalar
/* Purpose : This function reads out measurements (unsynchronized) and returns
/*           the result.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_FetcWpow (ViSession instrSession,
                                     ViReal64 *Power, ViChar errorMessage[])
{
return RSCMU_RF_NSig_Wpow(instrSession, RSCMU_COMMAND_FETC, Power, errorMessage);
}

/*===========================================================================*/
/* Function: Sample Scalar
/* Purpose : This function reads out measurements (synchronized) and returns
/*           the results.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_SampWpow (ViSession instrSession,
                                           ViReal64 *Power,
                                           ViChar errorMessage[])
{
return RSCMU_RF_NSig_Wpow(instrSession, RSCMU_COMMAND_SAMP, Power, errorMessage);
}

/*
 *-----------------------------------------------------------------------------
 *    INPUT LEVEL
 *    This subsystem contains the function for setting the input level
 *    range.
 *    It corresponds to the hotkey assignment Input Level in the
 *    graphical measurement menu Power versus Time.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Sense Input Level Range
/* Purpose : This function defines the level range of the Power vs Time
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensPowLevRang (ViSession instrSession,
                                                ViInt32   Range,
                                                ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession, "SENS:POW:LEV:RANG",
                            Range, RSCMU_LEV_RANGE_MIN, RSCMU_LEV_RANGE_MAX,
                            RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *    FREQUENCY
 *    This subsystem contains the functions for setting the frequency.
 *    It corresponds to the hotkey assignment Frequency in the
 *    graphical measurement menu Power versus Time.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Sense RF Center Frequency
/* Purpose : This function defines the input frequency of the analyzer.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensPowFreqCent (ViSession instrSession,
                                                 ViReal64  CenterFreq,
                                                 ViInt32   FreqUnit,
                                                 ViChar    errorMessage[])
{
    ViReal64    min[] = {10000005, 10000.005, 10.000005, 0.010000005,RSCMU_NAN};
    ViReal64    max[] = {2699999995.0, 2699999.995, 2699.999995, 2.699999995,RSCMU_NAN};

    RETMSG(RSCMU_setViReal64Unit(instrSession, "SENS:POW:FREQ:CENT",
                                 CenterFreq, min, max, FreqUnit, StrFreqUnit,
                                 RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set BRW
/* Purpose : This function defines the power measurement bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensPowFreqBand (ViSession instrSession,
                                                           ViInt32   Bandwidth,
                                                           ViChar    errorMessage[])
{
    ViString    list[] = {"10HZ","20HZ","30HZ","50HZ", "100HZ", "200HZ", "300HZ", "500HZ",
                          "1KHZ","2KHZ","3KHZ", "5KHZ", "10KHZ", "20KHZ", "30KHZ", "50KHZ",
                          "100KHZ", "200KHZ", "300KHZ", "500KHZ","1MHZ", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SENS:POW:FREQ:BAND",
                         Bandwidth, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 *-----------------------------------------------------------------------------
 *    TIME
 *    This subsystem contains the functions for setting the time
 *    modes.
 *    It corresponds to the hotkey assignment Time in the graphical
 *    measurement menu Power versus Time.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Sense Time Delay
/* Purpose : This function defines the time when the measurement is
/*           relative to the trigger time.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensPowTimeDel (ViSession instrSession,
                                                ViReal64  timeDelay,
                                                ViInt32   Unit,
                                                ViChar    errorMessage[])
{
    ViReal64    min[]={-15.8694E+6, - 15.8694E+3, - 15.8694, RSCMU_NAN};
    ViReal64    max[]={5.8694E+6, 5.8694E+3, 5.8694, RSCMU_NAN};

    RETMSG(RSCMU_setViReal64Unit(instrSession, "SENS:POW:TIME:DEL", timeDelay,
                                 min, max, Unit, StrTimeUnit,RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Sense Pow Time Span
/* Purpose : This function sets the span (i.e.the total evaluation time)
/*           of the Power vs Time measurement.
/*           The permissible spans depend on the selected bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensPowTimeSpan (ViSession instrSession,
                                                 ViReal64  Span,
                                                 ViInt32   Unit,
                                                 ViChar    errorMessage[])
{
    ViReal64    min[]={10.0, 10.0E-3, 10.0E-6, RSCMU_NAN};
    ViReal64    max[]={10.0E+6, 10.0E+3, 10.0, RSCMU_NAN};

    RETMSG(RSCMU_setViReal64Unit(instrSession, "SENS:POW:TIME:SPAN",
                                 Span, min, max, Unit, StrTimeUnit,
                                 RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Sense Triggered Meas. Timeout
/* Purpose : This function sets a timeout period after which the power
/*           measurement is aborted (e.g. if no trigger event could be
/*           detected).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensPowContTim (ViSession instrSession,
                                                ViReal64  timeOut,
                                                ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViReal64(instrSession, "CONF:POW:CONT:TIM",
                             timeOut, RSCMU_SENS_TRIG_MEAS_TMO_MIN,RSCMU_SENS_TRIG_MEAS_TMO_MAX,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Tests Repetition
/* Purpose : This function sets the number of statistics cycles, the
/*           stop condition and the stepping mode for the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfPowContRep (ViSession  instrSession,
                                                ViInt32    Repetition,
                                                ViInt32    StopCondition,
                                                ViInt32    StepMode,
                                                ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Repetition,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    sprintf (wrtBuf, "CONF:POW:CONT:REP %s,%s,%s", StrRepetition[Repetition], StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Set Grid (ON/OFF)
/* Purpose : This function switches the grid lines in the test diagrams
/*           on or off.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DispPowContGrid (ViSession instrSession,
                                                 ViInt32   gridLines,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "DISP:POW:CONT:GRID",
                         gridLines, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used as a setting command with the parameter ON this command
/*           sets all parameters of the subsystem to their default values
/*          (the setting OFF results in an error message).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DefPowConf (ViSession instrSession,
                                            ViInt32   DefaultValue,
                                            ViChar    errorMessage[])
{
    return RSCMU_RF_DefaultSettings(instrSession, "POW:CONT", DefaultValue, RSCMU_ERROR_PARAMETER2, errorMessage);
}

/*
 *-----------------------------------------------------------------------------
 * STATISTICS
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Statistic Cycles
/* Purpose : This function sets the number of statistics cycles, the
/*           stop condition and the stepping mode for the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfPowContRepStat (ViSession  instrSession,
                                                    ViInt32    NbRepetition,
                                                    ViInt32    StopCondition,
                                                    ViInt32    StepMode,
                                                    ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, NbRepetition, RSCMU_MULT_MEAS_MIN, RSCMU_MULT_MEAS_MAX, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    sprintf (wrtBuf, "CONF:POW:CONT:REP %ld,%s,%s", NbRepetition, StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

//*===========================================================================*/
/* Function: Set Measurement Range
/* Purpose : This function configures the 3 functions Read, Fetc, Samp
/*           _ArrPow by defining the test points on the time axis.
/*           the permissible range for the initial time and the test points
/*           number depends the span and the bandwidth set.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfArrPowRang (ViSession instrSession,
                                                ViReal64  Time,
                                                ViInt32   Unit,
                                                ViInt32   TestPtsNb,
                                                ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    CHECK_VI_INT32_RANGE (instrSession, Unit, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_REAL64_RANGE (instrSession, Time*unitExpLow[Unit], -15.0, 15.0, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "CONF:ARR:POW:RANG %.12f%s,", Time, StrTimeUnit[Unit]);

    RETMSG(RSCMU_setViInt32(instrSession, wrtBuf,
                            TestPtsNb, RSCMU_TEST_PTS_MIN, RSCMU_TEST_PTS_MAX,
                            RSCMU_ERROR_PARAMETER4));
}

//*===========================================================================*/
/* Function: Set Measurement SubArray Range
/* Purpose : This function configures the 3 functions Read, Fetc, Samp
/*           _SubArrPow by defining the test points on the time axis.
/*           the permissible range for the initial time and the test points
/*           number depends the span and the bandwidth set.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ConfSubArrPow (ViSession instrSession,
                                               ViInt32 Mode,
                                               ViReal64 StartTime,
                                               ViInt32 NbOfSamples,
                                               ViChar errorMessage[])
{
  // Check if the function is correctly used:
  CHECK_VI_REAL64_RANGE(instrSession, StartTime, -15.0, 15.0, RSCMU_ERROR_PARAMETER3, errorMessage);
  CHECK_VI_INT32_RANGE(instrSession, NbOfSamples, 1, 500, RSCMU_ERROR_PARAMETER4, errorMessage);

  return RSCMU_RF_ConfSubarrays (instrSession, RSCMU_POW, Mode, StartTime, NbOfSamples, errorMessage);
}

 /*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT NARROW-BAND POWER
 *
 * The subsystem NPOWer measures the power of the signal transmitted by the
 * mobile phone using the RF analyzer configuration of the POWer measurement.
 * Compared to WPOWer, the NPOWer measurement uses a narrow-band (500 kHz Gauss)
 * filter. The narrow-band NPOWer measurement yields the average, maximum and
 * minimum burst power of the current burst (display mode Current) and of the
 * averaged measurement curve (display mode Average).
 * The entire measurement curves (arrays) are not available, and no limit check
 * is performed. NPOWer is a quick and precise alternative to the WPOWer or
 * POWer measurements if only scalar results are needed.
 * Note: A Free Run trigger (TRIGger[:SEQuence]:SOURce FRUN) should be avoided
 * because it delays the NPOWer measurement.
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      This subsystem contains the functions for general control of the
 *      narrow-band power measurement.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start new measurement
/* Purpose : This function starts the narrow-band power measurement, setting it
/*           to the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitNpow (ViSession instrSession,
                                         ViChar    errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_INIT, RSCMU_RFNPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: Abort running measurement and switch off
/* Purpose : This function stops the narrow-band power measurement, setting it
/*           to the status "OFF"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborNpow (ViSession instrSession,
                                         ViChar    errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_ABOR, RSCMU_RFNPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: stop measurement after current statistic cycle
/* Purpose : This function stops the narrow-band power measurement, setting it
/*           to the status "STOP"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_StopNpow (ViSession instrSession,
                                                                 ViChar    errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_STOP, RSCMU_RFNPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: Next Measurement Step
/* Purpose : This function continues the narrow-band power measurement, setting
/*           it to the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ContNpow (ViSession instrSession,
                                         ViChar    errorMessage[])
{
    return RSCMU_SendControlCommand (instrSession, RSCMU_COMMAND_CONT, RSCMU_RFNPOW, RSCMU_NONE, errorMessage);
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the measurement
/*           is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfNpowErep (ViSession  instrSession,
                                             ViInt32    Report,
                                             ViChar     errorMessage[])
{
    return RSCMU_ConfReporting (instrSession, RSCMU_RFNPOW, RSCMU_NONE, Report, errorMessage);
}

/*===========================================================================*/
/* Function: Power/t Measurement Status
/* Purpose : This function returns the status of the current narrow-band power
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcNpowStat (ViSession  instrSession,
                                             ViInt32   *MeasurementStatus,
                                             ViInt32   *CountingNo,
                                             ViInt32   *StatisticNo,
                                             ViChar     errorMessage[])
{
    return RSCMU_FetcStatus (instrSession, RSCMU_RFNPOW, RSCMU_NONE, MeasurementStatus, CountingNo, StatisticNo, errorMessage);
}

/*
 * ----------------------------------------------------------------------------
 *      CONTROL
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set the filter bandwidth of the narrow-band power measurement
/* Purpose : This function defines the bandwidth of the power measurement.
/*           The TDMA filter is a matched (root-raised cosine) filter
/*           specified in standard TIA-EIA-136xxx for the test of modulation
/*           parameters. The CDMA filter is a 1.4 MHz bandpass filter
/*           specified for cdmaOne  and CDMA2000 tests.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensNpowBwid (ViSession instrSession,
                                              ViInt32   Bandwidth,
                                              ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"10HZ","20HZ","30HZ","50HZ", "100HZ", "200HZ", "300HZ", "500HZ",
                          "1KHZ","2KHZ","3KHZ", "5KHZ", "10KHZ", "20KHZ", "30KHZ", "50KHZ",
                          "100KHZ", "200KHZ", "300KHZ", "500KHZ","1MHZ", "TDMA", "CDMA", VI_NULL};

    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_07,RSCMU_ERROR_FW_VERSION_3_07));

    RETMSG(RSCMU_setEnum(instrSession,"NPOW:BWID",
                         Bandwidth, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Scope of Measurement
/* Purpose : This function defines the statistic count, repetition mode, stop
             condition, and stepping mode for measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfNpowCont (ViSession instrSession,
                                              ViInt32   NbBurst,
                                              ViInt32   repetition,
                                              ViInt32   NbRepetition,
                                              ViInt32   StopCondition,
                                              ViInt32   StepMode,
                                              ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    // Version Manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_07, RSCMU_ERROR_FW_VERSION_3_07));

    // Check if the calling function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, NbBurst, 1, 1000, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, repetition, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, NbRepetition, 1, 10000, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER6, errorMessage);

    if (repetition != RSCMU_MULTIPLE)
        sprintf(wrtBuf, "CONF:NPOW:CONT %ld,%s,%s,%s", NbBurst
                                                    , StrRepetition[repetition]
                                                    , StrStopCondition[StopCondition]
                                                    , StrStepMode[StepMode]);
    else
        sprintf(wrtBuf, "CONF:NPOW:CONT %ld,%ld,%s,%s", NbBurst
                                                    , NbRepetition
                                                    , StrStopCondition[StopCondition]
                                                    , StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Test Cycles
/* Purpose : This function sets the repetition mode, the stop condition and
/*           the stepping mode for the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfNpowContRep (ViSession  instrSession,
                                                 ViInt32    Repetition,
                                                 ViInt32    StopCondition,
                                                 ViInt32    StepMode,
                                                 ViChar     errorMessage[])
{
    return RSCMU_ConfControlRep (instrSession, RSCMU_NPOW, RSCMU_NONE, Repetition, StopCondition, StepMode, errorMessage);
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used as a setting command with the parameter ON this command
/*           sets all parameters of the subsystem to their default values
/*          (the setting OFF results in an error message).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DefNpowConf (ViSession instrSession,
                                             ViInt32 DefaultValue,
                                             ViChar errorMessage[])
{
    return RSCMU_RF_DefaultSettings(instrSession, "RFAN:PMFS:CONT", DefaultValue, RSCMU_ERROR_PARAMETER2, errorMessage);
}

/*
 * ----------------------------------------------------------------------------
 *      STATISTICS
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Statistic Count
/* Purpose : This function sets the number of evaluation periods that
/*           represent one statistics cycle.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfNpowContStat (ViSession instrSession,
                                                  ViInt32   NbBurst,
                                                  ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_07, RSCMU_ERROR_FW_VERSION_3_07));

    RETMSG(RSCMU_setViInt32(instrSession, "CONF:NPOW:CONT:STAT",
                            NbBurst, 1, 1000,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Statistic Cycles
/* Purpose : This function sets the number of statistics cycles, the
/*           stop condition and the stepping mode for the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfNpowContRepStat (ViSession instrSession,
                                                     ViInt32   NbRepetition,
                                                     ViInt32   StopCondition,
                                                     ViInt32   StepMode,
                                                     ViChar    errorMessage[])
{
    return RSCMU_ConfContRepStat (instrSession, RSCMU_NPOW, RSCMU_NONE, NbRepetition, StopCondition, StepMode, errorMessage);
}

/*
 * ----------------------------------------------------------------------------
 *      RESULTS
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Read Scalar
/* Purpose : This function starts the NPOWer measurement and returns the
/*           results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadNpow (ViSession  instrSession,
                                          ViReal64  *AvgPowCurr,
                                          ViReal64  *MinPowCurr,
                                          ViReal64  *MaxPowCurr,
                                          ViReal64  *AvgPowAvg,
                                          ViReal64  *MinPowAvg,
                                          ViReal64  *MaxPowAvg,
                                          ViReal64  *Frequency,
                                          ViChar     errorMessage[])
{
    return RSCMU_RF_NSig_Npow (instrSession, RSCMU_COMMAND_READ, AvgPowCurr, MinPowCurr, MaxPowCurr,
                               AvgPowAvg, MinPowAvg, MaxPowAvg, Frequency, errorMessage);
}

/*===========================================================================*/
/* Function: Fetch Scalar
/* Purpose : This function reads out measurements (unsynchronized) and returns
/*           the results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcNpow (ViSession  instrSession,
                                          ViReal64  *AvgPowCurr,
                                          ViReal64  *MinPowCurr,
                                          ViReal64  *MaxPowCurr,
                                          ViReal64  *AvgPowAvg,
                                          ViReal64  *MinPowAvg,
                                          ViReal64  *MaxPowAvg,
                                          ViReal64  *Frequency,
                                          ViChar     errorMessage[])
{
    return RSCMU_RF_NSig_Npow (instrSession, RSCMU_COMMAND_FETC, AvgPowCurr, MinPowCurr, MaxPowCurr,
                               AvgPowAvg, MinPowAvg, MaxPowAvg, Frequency, errorMessage);
}

/*===========================================================================*/
/* Function: Sample Scalar
/* Purpose : This function reads out measurements (synchronized) and returns
/*           the results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampNpow (ViSession  instrSession,
                                          ViReal64  *AvgPowCurr,
                                          ViReal64  *MinPowCurr,
                                          ViReal64  *MaxPowCurr,
                                          ViReal64  *AvgPowAvg,
                                          ViReal64  *MinPowAvg,
                                          ViReal64  *MaxPowAvg,
                                          ViReal64  *Frequency,
                                          ViChar     errorMessage[])
{
    return RSCMU_RF_NSig_Npow (instrSession, RSCMU_COMMAND_SAMP, AvgPowCurr, MinPowCurr, MaxPowCurr,
                               AvgPowAvg, MinPowAvg, MaxPowAvg, Frequency, errorMessage);
}

/*
/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT POWER/T
 *    This subsystem POWer contains the functions for measurement of
 *    the signal power.
 *    It corresponds to the measurement menu Power versus Time and the
 *    associated popup menu Power vs Time Configuration.
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      This subsystem contains the functions for general control of the
 *      power measurement.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start new measurement
/* Purpose : This function starts the power measurement, setting it to
/*           the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitPow (ViSession  instrSession,
                                         ViChar     errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "INIT:POW"));
}

/*===========================================================================*/
/* Function: Abort running measurement and switch off
/* Purpose : This function stops the power measurement, setting it to
/*           the status "OFF"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborPow (ViSession  instrSession,
                                         ViChar     errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "ABORT:POW"));
}

/*===========================================================================*/
/* Function: stop measurement after current statistic cycle
/* Purpose : This function stops the power measurement, setting it to
/*           the status "STOP"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_StopPow (ViSession  instrSession,
                                         ViChar     errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "STOP:POW"));
}

/*===========================================================================*/
/* Function: Next Measurement Step
/* Purpose : This function continue the power measurement, setting it to
/*           the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ContPow (ViSession  instrSession,
                                         ViChar     errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "CONT:POW"));
}

/*===========================================================================*/
/* Function: RF Power/t Measurement Status
/* Purpose : This function returns the status of the current RF Power/t
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcPowStat (ViSession  RFNSigSession,
                                             ViInt32   *MeasurementStatus,
                                             ViInt32   *CountingNo,
                                             ViChar     _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;

    // Fetch the status of the RF analyser measurement:
    CHECKRETMSG(RSCMU_getViString(RFNSigSession, "FETC:POW:STAT?",rdBuf, RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(RFNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(RFNSigSession, str, measurementStatusArr, MeasurementStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(RFNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",CountingNo))
        *CountingNo = RSCMU_NONE;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the
/*           measurement is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfPowErep (ViSession  instrSession,
                                             ViInt32    Mode,
                                             ViChar     errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "CONF:POW:EREP",
                         Mode, modeArr,
                         RSCMU_ERROR_PARAMETER2));
}

/*Subfunctions for read, fetch and sample measured values for POWer measurement*/

ViStatus _VI_FUNC RSCMU_RF_NSig_MeasArrPow (ViSession  instrSession,
                                            ViInt32    command,
                                            ViInt32    Mode,
                                            ViReal64  *initTime,
                                            ViInt32   *TestPtsNb,
                                            ViReal64   powerValues[],
                                            ViChar     errorMessage[])

{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    struct      RSCMU_SESSION_Globals  *p_globalvarBase;
    GET_BUFS( instrSession );

    // Check if the function is correctly used:
    // Notice: for vesion before V3.10 only current mode is possible
    CHECK_VI_INT32_RANGE(instrSession, Mode, 0, 3, RSCMU_ERROR_PARAMETER2, errorMessage);

    CHKNULLP(initTime, RSCMU_ERROR_PARAMETER2);
    CHKNULLP(TestPtsNb, RSCMU_ERROR_PARAMETER2);
    CHKNULLP(powerValues, RSCMU_ERROR_PARAMETER2);

    CHECKRETMSG(RSCMU_getViString(instrSession,"CONF:ARR:POW:RANG?",
                               rdBuf, RD_BUFFER_SIZE, 0));

    if (sscanf(rdBuf,"%le,%ld", initTime, TestPtsNb) != 2)
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

// Version manager
  if ((RSCMU_Status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Status;

   if (p_globalvarBase->RSCMU_FW_Version < RSCMU200_V2_15)
   return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_FW_VERSION_2_15, errorMessage);

      if (p_globalvarBase->RSCMU_FW_Version >= RSCMU200_V3_10)
    {
        sprintf(wrtBuf, "%s:ARR:POW:%s?",StrCommand[command],StrCAMMMode[Mode]);
    }
    else
    {
        sprintf(wrtBuf, "%s:ARR:POW?",StrCommand[command]);
    }

    CHECKERR (RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(instrSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,powerValues);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

ViStatus _VI_FUNC RSCMU_RF_NSig_MeasSubArrPow (ViSession  instrSession,
                                            ViInt32    command,
                                            ViInt32    Mode,
                                            ViReal64   powerValues[],
                                            ViChar     errorMessage[])

{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_BUFS( instrSession );

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Mode, 0, 3, RSCMU_ERROR_PARAMETER2, errorMessage);

    CHKNULLP(powerValues, RSCMU_ERROR_PARAMETER2);

    sprintf(wrtBuf, "%s:SUB:POW:%s?",StrCommand[command],StrCAMMMode[Mode]);

    CHECKERR (RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(instrSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,powerValues);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

ViStatus _VI_FUNC RSCMU_RF_NSig_MeasArrSpec (ViSession  instrSession,
                                             ViInt32    command,
                                             ViInt32    Mode,
                                             ViReal64  *initFreq,
                                             ViInt32   *TestPtsNb,
                                             ViReal64   powerValues[],
                                             ViChar     errorMessage[])
{

    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    struct      RSCMU_SESSION_Globals  *p_globalvarBase;
    GET_BUFS( instrSession );

    CHKNULLP(initFreq, RSCMU_ERROR_PARAMETER2);
    CHKNULLP(TestPtsNb, RSCMU_ERROR_PARAMETER2);
    CHKNULLP(powerValues, RSCMU_ERROR_PARAMETER2);

    CHECKRETMSG(RSCMU_getViString(instrSession,"CONF:ARR:SPEC:RANG?",
                               rdBuf, RD_BUFFER_SIZE, 0));

    if (sscanf(rdBuf,"%le,%ld", initFreq, TestPtsNb) != 2)
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

// Version manager
  if ((RSCMU_Status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Status;

  if (p_globalvarBase->RSCMU_FW_Version < RSCMU200_V2_15)
   return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_FW_VERSION_2_15, errorMessage);

      if (p_globalvarBase->RSCMU_FW_Version >= RSCMU200_V3_10)
    {
        sprintf(wrtBuf, "%s:ARR:SPEC:%s?",StrCommand[command],StrCAMMMode[Mode]);
    }
    else
    {
        sprintf(wrtBuf, "%s:ARR:SPEC?",StrCommand[command]);
    }

    CHECKERR (RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(instrSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,powerValues);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

ViStatus _VI_FUNC RSCMU_RF_NSig_MeasSubArrSpec (ViSession  instrSession,
                                             ViInt32    command,
                                             ViInt32    Mode,
                                             ViReal64   powerValues[],
                                             ViChar     errorMessage[])
{

    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    ViInt32     NbOfMeasValues;
    ViChar      BurstPowerTmp[669][20];

//  GET_BUFS( instrSession );

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Mode, 0, 3, RSCMU_ERROR_PARAMETER2, errorMessage);

    CHKNULLP(powerValues, RSCMU_ERROR_PARAMETER2);

    sprintf(wrtBuf, "%s:SUB:SPEC:%s?",StrCommand[command],StrCAMMMode[Mode]);

//  CHECKERR (RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));

//  CHECKERR(RSCMU_ReadDataUnknownLength(instrSession, &hugeBuf, VI_NULL));
//  if (RSCMU_Status < VI_SUCCESS)
//      if (hugeBuf) free(hugeBuf);
//
//  RSCMU_atofArray (hugeBuf,powerValues);
//
//  if (hugeBuf) free(hugeBuf);
  if ((RSCMU_Status = RSCMU_Query_Array (instrSession, wrtBuf, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
    return RSCMU_Status;

  // Find all burst power values by breaking the read buffer:
  NbOfMeasValues = RSCMU_scanStringBurst (rdBuf, BurstPowerTmp);

  RSCMU_Status = RSCMU_ReturnCMUResponse (NbOfMeasValues, BurstPowerTmp, powerValues);

  return RSCMU_Status;
}

/*
 *-----------------------------------------------------------------------------
 *    MEASURED VALUES
 *    This subsystem contains the functions for measurement and output
 *    of the measured power versus the time.
 *    It corresponds to the graphical measurement menu Power versus
 *    Time.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Read Measured Values
/* Purpose : This function returns the power values versus the time.
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrPower (ViSession  instrSession,
                                               ViInt32    Mode,
                                               ViReal64  *initTime,
                                               ViInt32   *TestPtsNb,
                                               ViReal64   powerValues[],
                                               ViChar     errorMessage[])
{

    RETMSG(RSCMU_RF_NSig_MeasArrPow(instrSession, RSCMU_COMMAND_READ, Mode, initTime, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Fetch Measured Values
/* Purpose : This function reads out (unsynch.) the power values versus
/*           the time. Measurement starting point and number of
/*           measurement results are defined in the function
/*           "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcArrPower (ViSession  instrSession,
                                               ViInt32    Mode,
                                               ViReal64  *initTime,
                                               ViInt32   *TestPtsNb,
                                               ViReal64   powerValues[],
                                               ViChar     errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasArrPow(instrSession, RSCMU_COMMAND_FETC, Mode, initTime, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Sample Measured Values
/* Purpose : This function reads out (synch.) the power values versus
/*           the time. Measurement starting point and number of
/*           measurement results are defined in the function
/*           "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampArrPower (ViSession  instrSession,
                                               ViInt32    Mode,
                                               ViReal64   *initTime,
                                               ViInt32   *TestPtsNb,
                                               ViReal64   powerValues[],
                                               ViChar     errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasArrPow(instrSession, RSCMU_COMMAND_SAMP, Mode, initTime, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* for compatibility only!!!
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrPow (ViSession  instrSession,
                                            ViReal64  *initTime,
                                            ViInt32   *TestPtsNb,
                                            ViReal64   powerValues[],
                                            ViChar     errorMessage[])
{

    RETMSG(RSCMU_RF_NSig_MeasArrPow(instrSession, RSCMU_COMMAND_READ, RSCMU_CURRENT, initTime, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* for compatibility only!!!
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcArrPow (ViSession  instrSession,
                                            ViReal64  *initTime,
                                            ViInt32   *TestPtsNb,
                                            ViReal64   powerValues[],
                                            ViChar     errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasArrPow(instrSession, RSCMU_COMMAND_FETC, RSCMU_CURRENT, initTime, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* for compatibility only!!!
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampArrPow (ViSession  instrSession,
                                            ViReal64   *initTime,
                                            ViInt32   *TestPtsNb,
                                            ViReal64   powerValues[],
                                            ViChar     errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasArrPow(instrSession, RSCMU_COMMAND_SAMP, RSCMU_CURRENT, initTime, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Read Measured Values
/* Purpose : This function returns the power values versus the time.
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrPow (ViSession instrSession,
                                               ViInt32 Mode,
                                               ViReal64 powerValues[],
                                               ViChar errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasSubArrPow(instrSession, RSCMU_COMMAND_READ, Mode, powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Fetch Measured Values
/* Purpose : This function reads out (unsynch.) the power values versus
/*           the time. Measurement starting point and number of
/*           measurement results are defined in the function
/*           "Set Measurement Range".
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_FetcSubArrPow (ViSession instrSession,
                                               ViInt32 Mode,
                                               ViReal64 powerValues[],
                                               ViChar errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasSubArrPow(instrSession, RSCMU_COMMAND_FETC, Mode, powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Sample Measured Values
/* Purpose : This function reads out (synch.) the power values versus
/*           the time. Measurement starting point and number of
/*           measurement results are defined in the function
/*           "Set Measurement Range".
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_SampSubArrPow (ViSession instrSession,
                                               ViInt32 Mode,
                                               ViReal64 powerValues[],
                                               ViChar errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasSubArrPow(instrSession, RSCMU_COMMAND_SAMP, Mode, powerValues,errorMessage));
}

/*
 * ----------------------------------------------------------------------------
 *      FREQUENCY / RBW
 *      This class contains the functions for setting the frequency.
 *      It corresponds to the hotkey assignment Frequency/RBW in the
 *      graphical measurement menu Spectrum.
 * ----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: Sense Frequency
/* Purpose : This function sets the center frequency and span or the
/*           start and spectrum analyzer stop frequency.
/*           Start and stop frequency are used for calculation of the
/*           other two frequencies.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensSpecFreqStartStop (ViSession instrSession,
                                                       ViReal64  StartFreq,
                                                       ViInt32   Unit1,
                                                       ViReal64  StopFreq,
                                                       ViInt32   Unit2,
                                                       ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViReal64    min_start[] = {0.0,0.0,10.00001, 10.00001E-3, RSCMU_NAN};
    ViReal64    max_start[] = {0.0,0.0,2699.99999, 2.69999999, RSCMU_NAN};
    ViReal64    min_stop[] = {0.0,0.0,10.0, 10.0E-3, RSCMU_NAN};
    ViReal64    max_stop[] = {0.0,0.0,2700.0, 2.7, RSCMU_NAN};

    CHECK_VI_INT32_RANGE(instrSession, Unit1, RSCMU_MHZ, RSCMU_GHZ, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, Unit2, RSCMU_MHZ, RSCMU_GHZ, RSCMU_ERROR_PARAMETER5, errorMessage);

    CHECKRET(RSCMU_setViReal64Unit(instrSession, "SPEC:FREQ:STAR",
                                   StartFreq, min_start, max_start,
                                   Unit1, StrFreqUnit,RSCMU_ERROR_PARAMETER2));

    RETMSG(RSCMU_setViReal64Unit(instrSession, "SPEC:FREQ:STOP",
                                 StopFreq, min_stop, max_stop,
                                 Unit2, StrFreqUnit,RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Sense Frequency
/* Purpose : This function sets the center frequency and span or the
/*           start and spectrum analyzer stop frequency.
/*           Start and stop frequency are used for calculation of the
/*           other two frequencies.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensSpecFreqCentSpan (ViSession instrSession,
                                                      ViReal64  CenterFreq,
                                                      ViInt32   Unit1,
                                                      ViReal64  Span,
                                                      ViInt32   Unit2,
                                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViReal64    min_center[] = {0.0,0.0,10.000005, 10.000005E-3, RSCMU_NAN};
    ViReal64    max_center[] = {0.0,0.0,2699.999995, 2.699999995, RSCMU_NAN};
    ViReal64    min_span[] = {0.0,0.0,1.0E-5, 1.0E-8, RSCMU_NAN};
    ViReal64    max_span[] = {0.0,0.0,2699.9, 2.6999, RSCMU_NAN};

    CHECK_VI_INT32_RANGE(instrSession, Unit1, RSCMU_MHZ, RSCMU_GHZ, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, Unit2, RSCMU_MHZ, RSCMU_GHZ, RSCMU_ERROR_PARAMETER5, errorMessage);

    CHECKRET(RSCMU_setViReal64Unit(instrSession, "SPEC:FREQ:CENT",
                                   CenterFreq, min_center, max_center,
                                   Unit1, StrFreqUnit,RSCMU_ERROR_PARAMETER2));

    RETMSG(RSCMU_setViReal64Unit(instrSession, "SPEC:FREQ:SPAN",
                                 Span, min_span, max_span,
                                 Unit2, StrFreqUnit,RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Sense RBW
/* Purpose : This function defines the spectrum measurement bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensSpecFreqBand (ViSession instrSession,
                                                  ViInt32   RBW,
                                                  ViChar    errorMessage[])
{
    ViString    list[] = {"10HZ","20HZ","30HZ","50HZ", "100HZ", "200HZ", "300HZ", "500HZ",
                          "1KHZ","2KHZ","3KHZ", "5KHZ", "10KHZ", "20KHZ", "30KHZ", "50KHZ",
                          "100KHZ", "200KHZ", "300KHZ", "500KHZ","1MHZ", "AUTO", VI_NULL};

    RETMSG(RSCMU_setEnum(instrSession, "SPEC:FREQ:BAND",
                         RBW, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 * ----------------------------------------------------------------------------
 *      INPUT LEVEL
 *      This class contains the functions for setting the level.
 *      It corresponds to the hotkey assignment Input Level in the
 *      graphical measurement menu Spectrum.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Sense Level Range
/* Purpose : This function defines the level range for the spectrum
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SensSpecLevRang (ViSession instrSession,
                                                 ViInt32   Range,
                                                 ViChar    errorMessage[])
{
    RETMSG(RSCMU_setViInt32(instrSession,"SENS:SPEC:LEV:RANG",
                            Range, RSCMU_LEV_RANGE_MIN,RSCMU_LEV_RANGE_MAX,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Set Tests Repetition
/* Purpose : This function sets the number of statistics cycles, the
/*           stop condition and the stepping mode for the spectrum
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfSpecContRep (ViSession  instrSession,
                                                 ViInt32    Repetition,
                                                 ViInt32    StopCondition,
                                                 ViInt32    StepMode,
                                                 ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Repetition,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    sprintf (wrtBuf, "CONF:SPEC:CONT:REP %s,%s,%s", StrRepetition[Repetition], StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Set Grid (ON/OFF)
/* Purpose : This function switches the grid lines in the test diagrams
/*           on or off.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DispSpecContGrid (ViSession instrSession,
                                                  ViInt32   gridLines,
                                                  ViChar    errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "DISP:SPEC:CONT:GRID",
                         gridLines, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used as a setting command with the parameter ON this command
/*           sets all parameters of the subsystem to their default values
/*          (the setting OFF results in an error message).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_DefSpecConf (ViSession instrSession,
                                             ViInt32 DefaultValue,
                                             ViChar errorMessage[])
{
    return RSCMU_RF_DefaultSettings(instrSession, "SPEC:CONT", DefaultValue, RSCMU_ERROR_PARAMETER2, errorMessage);
}

/*===========================================================================*/
/* Function: Detector
/* Purpose : If defines how the measurement curve is calculated from the entire
/*           set of measurement points.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfSpecDetector (ViSession instrSession,
                                                  ViInt32 detector,
                                                  ViChar errorMessage[])
{
    ViString    list[] = {"PEAK","RMS", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_05, RSCMU_ERROR_FW_VERSION_3_05));

    RETMSG(RSCMU_setEnum(instrSession, "SENSe:SPECtrum:DETector",
                         detector, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 * ----------------------------------------------------------------------------
 *      STATISTICS
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Set Statistic Cycles
/* Purpose : This function sets the number of statistics cycles, the
/*           stop condition and the stepping mode for the spectrum
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfSpecContRepStat (ViSession  instrSession,
                                                     ViInt32    NbRepetition,
                                                     ViInt32    StopCondition,
                                                     ViInt32    StepMode,
                                                     ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, NbRepetition, RSCMU_MULT_MEAS_MIN, RSCMU_MULT_MEAS_MAX, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    sprintf (wrtBuf, "CONF:SPEC:CONT:REP %ld,%s,%s", NbRepetition, StrStopCondition[StopCondition], StrStepMode[StepMode]);
    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Set Measurement Range
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfArrSpecRang (ViSession instrSession,
                                                 ViReal64  initialFreq,
                                                 ViInt32   Unit,
                                                 ViInt32   TestPtsNb,
                                                 ViChar    errorMessage[])
{
    ViReal64    order;
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(instrSession);

    CHECK_VI_INT32_RANGE(instrSession, Unit, RSCMU_MHZ, RSCMU_GHZ, RSCMU_ERROR_PARAMETER3, errorMessage);

    order = (Unit == RSCMU_GHZ) ? 1.0E+3 : 1.0;

    CHECK_VI_REAL64_RANGE(instrSession, initialFreq, 1/order, 2690.0/order, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Set Frequency Bandwidth for power measurement
    sprintf (wrtBuf, "CONF:ARR:SPEC:RANG %.12f %s,", initialFreq, StrFreqUnit[Unit]);
    RETMSG(RSCMU_setViInt32(instrSession, wrtBuf,
                            TestPtsNb,RSCMU_TEST_PTS_MIN,RSCMU_TEST_PTS_MAX,
                            RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Set SubArray Range
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfSubArrSpec (ViSession instrSession,
                                                ViInt32 Mode,
                                                ViReal64 FirstMeasurementFreq,
                                                ViInt32 MeasureResultsNumber,
                                                ViChar errorMessage[])
{

      // Check if the function is correctly used:
  CHECK_VI_INT32_RANGE(instrSession, Mode, 0, 4, RSCMU_ERROR_PARAMETER2, errorMessage);
  CHECK_VI_REAL64_RANGE(instrSession, FirstMeasurementFreq, 10.00, 2699.99, RSCMU_ERROR_PARAMETER3, errorMessage);
  CHECK_VI_INT32_RANGE(instrSession, MeasureResultsNumber, 1, 560, RSCMU_ERROR_PARAMETER4, errorMessage);

  return RSCMU_RF_ConfSubarrays (instrSession, RSCMU_SPEC, Mode, FirstMeasurementFreq, MeasureResultsNumber, errorMessage);

}

/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT SPECTRUM
 *    This subsystem contains the functions for frequency spectrum
 *    measurement .
 *    The subsystem corresponds to the measurement menu Spectrum and
 *    the associated popup menu Spectrum Config.
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      This subsystem  contains the functions for general control of
 *      the spectrums.
 *      It corresponds to the Spectrum. softkey in the measurement menu
 *      Spectrum.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start new measurement
/* Purpose : This function start a new Spectrum measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitSpec (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "INIT:SPEC"));
}

/*===========================================================================*/
/* Function: Abort running measurement and switch off
/* Purpose : this function abort running Spectrum measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborSpec (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "ABORT:SPEC"));
}

/*===========================================================================*/
/* Function: stop measurement
/* Purpose : This function stops Spectrum measurement after current
/*           statistic cycle.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_StopSpec (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "STOP:SPEC"));
}

/*===========================================================================*/
/* Function: Next Measurement Step
/* Purpose : This function runs next Spectrum measurement step.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ContSpec (ViSession instrSession, ViChar errorMessage[])
{
    RETMSG(RSCMU_sendCmd(instrSession, "CONT:SPEC"));
}

/*===========================================================================*/
/* Function: RF Spectrum Measurement Status
/* Purpose : This function returns the status of the current RF spectrum
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcSpecStat (ViSession  RFNSigSession,
                                              ViInt32   *MeasurementStatus,
                                              ViInt32   *CountingNo,
                                              ViChar     _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;

    // Fetch the status of the RF analyser measurement:
    CHECKRETMSG(RSCMU_getViString(RFNSigSession, "FETC:SPEC:STAT?",rdBuf, RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(RFNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(RFNSigSession, str, measurementStatusArr, MeasurementStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(RFNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",CountingNo))
        *CountingNo = RSCMU_NONE;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the
/*           spectrum measurement is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfSpecErep (ViSession  instrSession,
                                              ViInt32    Mode,
                                              ViChar     errorMessage[])
{
    RETMSG(RSCMU_setEnum(instrSession, "CONF:SPEC:EREP",
                         Mode, modeArr,
                         RSCMU_ERROR_PARAMETER2));
}

/*
 * ----------------------------------------------------------------------------
 *      MEASURED VALUES
 *      This class contains the functions for measurement and output of
 *      the spectrum measurement.
 *      It corresponds to the various output elements in the graphical
 *      measurement menu Spectrum.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Read Measurement Values
/* Purpose : This function returns the  Spectrum measurement results.
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrSpectrum (ViSession  instrSession,
                                                 ViInt32    Mode,
                                                 ViReal64  *initFreq,
                                                 ViInt32   *TestPtsNb,
                                                 ViReal64   powerValues[],
                                                 ViChar     errorMessage[])
{

    RETMSG(RSCMU_RF_NSig_MeasArrSpec(instrSession, RSCMU_COMMAND_READ, Mode, initFreq, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Fetch Measurement Values
/* Purpose : This function reads out the  Spectrum measurement results. (unsynch)
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcArrSpectrum (ViSession  instrSession,
                                                 ViInt32    Mode,
                                                 ViReal64   *initFreq,
                                                 ViInt32   *TestPtsNb,
                                                 ViReal64   powerValues[],
                                                 ViChar     errorMessage[])
{

    RETMSG(RSCMU_RF_NSig_MeasArrSpec(instrSession, RSCMU_COMMAND_FETC, Mode, initFreq, TestPtsNb,powerValues,errorMessage));

}

/*===========================================================================*/
/* Function: Sample Measurement Values
/* Purpose : This function reads out the  Spectrum measurement results. (synch)
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampArrSpectrum (ViSession  instrSession,
                                                 ViInt32    Mode,
                                                 ViReal64   *initFreq,
                                                 ViInt32   *TestPtsNb,
                                                 ViReal64   powerValues[],
                                                 ViChar     errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasArrSpec(instrSession, RSCMU_COMMAND_SAMP, Mode, initFreq, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* for compatibility only!!!
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrSpec (ViSession  instrSession,
                                             ViReal64  *initFreq,
                                             ViInt32   *TestPtsNb,
                                             ViReal64   powerValues[],
                                             ViChar     errorMessage[])
{

    RETMSG(RSCMU_RF_NSig_MeasArrSpec(instrSession, RSCMU_COMMAND_READ, RSCMU_CURRENT, initFreq, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* for compatibility only!!!
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcArrSpec (ViSession  instrSession,
                                             ViReal64   *initFreq,
                                             ViInt32   *TestPtsNb,
                                             ViReal64   powerValues[],
                                             ViChar     errorMessage[])
{

    RETMSG(RSCMU_RF_NSig_MeasArrSpec(instrSession, RSCMU_COMMAND_FETC, RSCMU_CURRENT, initFreq, TestPtsNb,powerValues,errorMessage));

}

/*===========================================================================*/
/* for compatibility only!!!
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampArrSpec (ViSession  instrSession,
                                             ViReal64   *initFreq,
                                             ViInt32   *TestPtsNb,
                                             ViReal64   powerValues[],
                                             ViChar     errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasArrSpec(instrSession, RSCMU_COMMAND_SAMP, RSCMU_CURRENT, initFreq, TestPtsNb,powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Read Subarray of Measurement Values
/* Purpose : This function returns the  Spectrum measurement results.
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrSpec (ViSession instrSession,
                                                ViInt32   Mode,
                                                ViReal64  powerValues[],
                                                ViChar    errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasSubArrSpec(instrSession, RSCMU_COMMAND_READ, Mode, powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Fetch Subarray of Measurement Values
/* Purpose : This function reads out the  Spectrum measurement results. (unsynch)
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcSubArrSpec (ViSession instrSession,
                                                ViInt32 Mode,
                                                ViReal64 powerValues[],
                                                ViChar errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasSubArrSpec(instrSession, RSCMU_COMMAND_FETC, Mode, powerValues,errorMessage));
}

/*===========================================================================*/
/* Function: Sample Subarray of Measurement Values
/* Purpose : This function reads out the  Spectrum measurement results. (synch)
/*           Starting point of the measurement and number of measurement
/*           results are defined in the function "Set Measurement Range".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampSubArrSpec (ViSession instrSession,
                                                ViInt32  Mode,
                                                ViReal64 powerValues[],
                                                ViChar errorMessage[])
{
    RETMSG(RSCMU_RF_NSig_MeasSubArrSpec(instrSession, RSCMU_COMMAND_SAMP, Mode, powerValues,errorMessage));
}



/*
 *-----------------------------------------------------------------------------
 *    SPECtrum Markers
 *    The functions in the SPECtrum:MARKer subsystem query
 *    trace values and perform a peak search.
 * ----------------------------------------------------------------------------
 */

/*===========================================================================*/
/* Function: Fetch Marker absolute
/* Purpose : This function sets a virtual reference marker to the specified
/*           <x position> and returns the measured RF level at this position.
/*           The command doesnt affect the reference marker in manual control.
/*           The frequency range for the Spectrum measurement is set via
/*           [SENSe:]SPECtrum:FREQuency:STARt and [SENSe:]SPECtrum:FREQuency:STOP..
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcSpecMarkerAbsolute (ViSession instrSession,
                                                        ViReal64 x_position,
                                                        ViReal64 *level,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_BUFS(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_60,RSCMU_ERROR_FW_VERSION_3_60));

    sprintf(wrtBuf, "FETCh:SPECtrum:MARKer:ABSolute? %lf MHZ", x_position);

    RETMSG(RSCMU_getViReal64(instrSession, wrtBuf, level,
                             RSCMU_ERROR_PARAMETER3));
}



/*===========================================================================*/
/* Function: Fetch Marker relative
/* Purpose : This function sets virtual delta markers 1 and 2 to the specified
/*           x-positions and returns the difference of the two marker levels
/*           level(marker 1) - level(marker 2). The command doesnt affect
/*           the relative (delta) markers in manual control.
/*           The frequency range for the Spectrum measurement is set via
/*           [SENSe:]SPECtrum:FREQuency:STARt and [SENSe:]SPECtrum:FREQuency:STOP..
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcSpecMarkerRelative (ViSession instrSession,
                                                        ViReal64 x1_position,
                                                        ViReal64 x2_position,
                                                        ViReal64 *level,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_BUFS(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_60,RSCMU_ERROR_FW_VERSION_3_60));

    sprintf(wrtBuf, "FETCh:SPECtrum:MARKer:RELative? %lf MHZ, %lf MHZ", x1_position, x2_position);

    RETMSG(RSCMU_getViReal64(instrSession, wrtBuf, level,
                             RSCMU_ERROR_PARAMETER4));
}


/*===========================================================================*/
/* Function: Fetch Peak Marker
/* Purpose : This function places the reference marker to the maximum of the
/*           trace and returns the marker frequency and the peak level.
/*           NAN, NAN is returned if no Spectrum measurement is running.
/*           The command also affects the reference marker in manual control..
/*           The frequency range for the Spectrum measurement is set via
/*           [SENSe:]SPECtrum:FREQuency:STARt and [SENSe:]SPECtrum:FREQuency:STOP..
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcSpecMarkerPeak (ViSession instrSession,
                                                    ViReal64 *frequency,
                                                    ViReal64 *level,
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      Tab[100][100];
    GET_RD_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_60,RSCMU_ERROR_FW_VERSION_3_60));

    CHECKRETMSG(RSCMU_getViString(instrSession,"FETCh:SPECtrum:MARKer:PEAK?",rdBuf,RD_BUFFER_SIZE,0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", Tab);

    // Get the frequency
    if (strcmp (Tab[0], "NAN") != 0)
      sscanf (Tab[0], "%lf", frequency);
    else *frequency = RSCMU_NAN;

    // Get the level
    if (strcmp (Tab[1], "NAN") != 0)
      sscanf (Tab[1], "%lf", level);
    else *level = RSCMU_NAN;

    return RSCMU_Status;
}




/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT IQ/Slot
 *    The subsystem IQSLot measures the I and Q amplitudes in a sequence of
 *    consecutive measurement steps. The subsystem corresponds to the measurement
 *    menu I/Q vs. Slot and the associated popup menu I/Q vs. Slot Configuration.
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      This subsystem contains the functions for general control of the
 *      IQ/Slot measurement.
 *-----------------------------------------------------------------------------
 */


/*===========================================================================*/
/* IQ/Slot Config Default Setting
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotDefSet (ViSession instrSession,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG (RSCMU_sendCmd (instrSession, "DEFault:IQSLot:CONTrol ON"));
}




/*===========================================================================*/
/* Repetition Mode
/* This function defines the number of test cycles and, if
/* required, an abort condition for the measurement.
/* Note: In the case of READ commands (READ:...), the <Repetition>
/* parameter has no effect; the measurement is always aborted after
/* a single shot.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotRep (ViSession instrSession,
                                               ViInt32 repetition,
                                               ViInt32 stopCondition,
                                               ViInt32 stepMode,
                                               ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, repetition,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, stopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, stepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    sprintf (wrtBuf, "CONFigure:IQSLot:CONTrol:REPetition %s,%s,%s", StrRepetition[repetition], StrStopCondition[stopCondition], StrStepMode[stepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}



/*===========================================================================*/
/* Nr of Steps
/* This function defines the number of steps per measurement cycle:
/* a singleshot measurement is complete after the specified number
/* of steps.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotNrSteps (ViSession instrSession,
                                                   ViInt32 steps,
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQSLot:CONTrol:NOSTeps",
                            steps, 1, 1000,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Number of Subsweeps
/* Purpose : This function defines the number of subsweeps per measurement 
/*           cycle: a single shot measurement is complete after the specified 
/*           number of subsweeps.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotNrSubsweeps (ViSession instrSession,
                                                       ViInt32 subsweeps,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_setViInt32(instrSession, "CONF:IQSL:CONT:NOS",
                             subsweeps, 1, 20,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Trigger Offset
/* This function defines an offset between the trigger event (power
/* or external trigger:
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotTrgOffset (ViSession instrSession,
                                                     ViInt32 triggerOffset,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQSLot:CONTrol:TOFFset",
                            triggerOffset, 0, 32768,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Step Length
/* This function defines the length of each step. To measure a series of power
/* steps the sum of the measurement length (CONFigure:IQSLot:CONTrol:MLENgth)
/* plus the trigger offset (CONFigure:IQSLot:CONTrol :TOFFset)
/*  should not exceed the step length.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotStepLength (ViSession instrSession,
                                                      ViInt32 step_length,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQSLot:CONTrol:SLENgth",
                            step_length, 128, 8192,
                            RSCMU_ERROR_PARAMETER2));
}



/*===========================================================================*/
/* Measurement Length
/* This function defines the measurement length within each step. All I/Q
/* results are averaged over the measurement length so that a single complex
/* result is obtained for each step.:
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotMesLength (ViSession instrSession,
                                                     ViInt32 measurementLength,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQSLot:CONTrol:MLENgth",
                            measurementLength, 128, 16384,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Frequency Estimation Limit
/* This function defines a threshold for the average levels in the slots that
/* is is relevant for the frequency offset estimate Slots below the threshold
/* are not considered for the estimate.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotFreqEstLimit (ViSession instrSession,
                                                        ViInt32 frequEstimLimit,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQSLot:CONTrol:FELimit",
                            frequEstimLimit, -200, -30,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Capture Timeout
/* Purpose : This function specifies a maximum time between the start of the 
/*           measurement and the trigger time in tenths of a second 
/*           (numeric entry: 1 to 600). The I/Q Slot measurement must be 
/*           triggered (no free run mode).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotContCtim (ViSession instrSession,
                                                    ViReal64 captureTimeout,
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V4_20,RSCMU_ERROR_FW_VERSION_4_20));

    RETMSG(RSCMU_setViReal64(instrSession, "CONF:IQSL:CONT:CTIM",
                             captureTimeout, 0.1, 60.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Display Format
/* This function selects the display format for the complex I/Q vs.Slot results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotDisplFormat (ViSession instrSession,
                                                       ViInt32 displayFormat,
                                                       ViChar errorMessage[])
{
    ViString    list[] = {"IQ","LP", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQSLot:SELection",
                         displayFormat, list,
                         RSCMU_ERROR_PARAMETER2));
}


/*===========================================================================*/
/* Configure Subarray
/* This function configures the read, fetch and sample commands.
/* It restricts the measurement to up to 32 subranges where either
/* all measurement results or a single statistical value is returned
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotSubarrayRange (ViSession instrSession,
                                                         ViInt32 statisticsMode,
                                                         ViUInt32 start,
                                                         ViInt32 number_of_steps,
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, statisticsMode, 0, 4, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, start, 1, 1000, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, number_of_steps, 1, 1000, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Configures subarrays
    sprintf (wrtBuf, "CONFigure:SUBarrays:IQSLot %s,%ld,%ld", StrStat[statisticsMode]
                                                            , start
                                                            , number_of_steps);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Frequency Offset
/* Get Frequency Offset
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcIQSlotFreqOffs (ViSession instrSession,
                                                    ViReal64 *frequencyOffset,
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_getViReal64(instrSession, "FETCh:SCALar:IQSLot:FREQuency:OFFSet?", frequencyOffset,
                             RSCMU_ERROR_PARAMETER2));

}

/*===========================================================================*/
/* Init
/* Start measurement without waiting => RUN
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitIQSlot (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "INITiate:IQSLot"));
}

/*===========================================================================*/
/* Abort
/* Abort measurement and switch off => OFF
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborIQSlot (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "ABORt:IQSLot"));
}

/*===========================================================================*/
/* Stop
/* Stop measurement => STOP
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_StopIQSlot (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "STOP:IQSLot"));
}

/*===========================================================================*/
/* Continue
/* Next measurement step (only counting mode) => RUN
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ContIQSlot (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "CONTinue:IQSLot"));
}


/*===========================================================================*/
/* IQ/Slot status
/* This function returns status of the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcIQSlotStat (ViSession instrSession,
                                                ViInt32 *measurementStatus,
                                                ViInt32 *counter_for_current_stat_cycle,
                                                ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_RD_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    // Fetch the current status:
    CHECKRETMSG(RSCMU_getViString(instrSession, "FETCh:IQSLot:STATus?", rdBuf,
                                  RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(instrSession, str, measurementStatusArr, measurementStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",counter_for_current_stat_cycle))
        *counter_for_current_stat_cycle = RSCMU_NONE;

    return RSCMU_Status;
}



/*===========================================================================*/
/* Event reporting of the measurement
/* This function defines the events generated when the
/* Measurement is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQSlotErep (ViSession instrSession,
                                                ViInt32 measurementEventReporting,
                                                ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQSLot:EREPorting",
                         measurementEventReporting, modeArr, RSCMU_ERROR_PARAMETER2));
}



/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQSlotIAmpl (ViSession instrSession,
                                                    ViReal64 iAmplitudes[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:ARRay:IQSLot:I"
                           ,iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQSlotIAmpl (ViSession instrSession,
                                                     ViReal64 iAmplitudes[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:ARRay:IQSLot:I"
                           ,iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQSlotIAmpl (ViSession instrSession,
                                                      ViReal64 iAmplitudes[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:ARRay:IQSLot:I"
                           ,iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQSlotIAmpl (ViSession instrSession,
                                                       ViReal64 iAmplitudes[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:SUBarrays:IQSLot:I"
                           ,iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQSlotIAmpl (ViSession instrSession,
                                                        ViReal64 iAmplitudes[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:SUBarrays:IQSLot:I"
                           ,iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQSlotIAmpl (ViSession instrSession,
                                                         ViReal64 iAmplitudes[],
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:SUBarrays:IQSLot:I"
                           ,iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQSlotQAmpl (ViSession instrSession,
                                                    ViReal64 qAmplitudes[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:ARRay:IQSLot:Q"
                           ,qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQSlotQAmpl (ViSession instrSession,
                                                     ViReal64 qAmplitudes[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:ARRay:IQSLot:Q"
                           ,qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQSlotQAmpl (ViSession instrSession,
                                                      ViReal64 qAmplitudes[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:ARRay:IQSLot:Q"
                           ,qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQSlotQAmpl (ViSession instrSession,
                                                       ViReal64 qAmplitudes[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:SUBarrays:IQSLot:Q"
                           ,qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQSlotQAmpl (ViSession instrSession,
                                                        ViReal64 qAmplitudes[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:SUBarrays:IQSLot:Q"
                           ,qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQSlotQAmpl (ViSession instrSession,
                                                         ViReal64 qAmplitudes[],
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:SUBarrays:IQSLot:Q"
                           ,qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQSlotLevel (ViSession instrSession,
                                                    ViReal64 level[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:ARRay:IQSLot:LEVel"
                           ,level, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQSlotLevel (ViSession instrSession,
                                                     ViReal64 level[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:ARRay:IQSLot:LEVel"
                           ,level, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQSlotLevel (ViSession instrSession,
                                                      ViReal64 level[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:ARRay:IQSLot:LEVel"
                           ,level, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQSlotLevel (ViSession instrSession,
                                                       ViReal64 level[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:SUBarrays:IQSLot:LEVel"
                           ,level, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQSlotLevel (ViSession instrSession,
                                                        ViReal64 level[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:SUBarrays:IQSLot:LEVel"
                           ,level, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQSlotLevel (ViSession instrSession,
                                                         ViReal64 level[],
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:SUBarrays:IQSLot:LEVel"
                           ,level, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQSlotPhase (ViSession instrSession,
                                                    ViReal64 phase[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:ARRay:IQSLot:PHASe"
                           ,phase, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQSlotPhase (ViSession instrSession,
                                                     ViReal64 phase[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:ARRay:IQSLot:PHASe"
                           ,phase, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQSlotPhase (ViSession instrSession,
                                                      ViReal64 phase[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:ARRay:IQSLot:PHASe"
                           ,phase, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQSlotPhase (ViSession instrSession,
                                                       ViReal64 phase[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "READ:SUBarrays:IQSLot:PHASe"
                           ,phase, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQSlotPhase (ViSession instrSession,
                                                        ViReal64 phase[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "FETCh:SUBarrays:IQSLot:PHASe"
                           ,phase, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQSlotPhase (ViSession instrSession,
                                                         ViReal64 phase[],
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasArray( instrSession, "SAMPle:SUBarrays:IQSLot:PHASe"
                           ,phase, errorMessage));
}



/*
 *-----------------------------------------------------------------------------
 *    MEASUREMENT OBJECT IQ Recorder
 *    The subsystem IQ Recorder....
 * ----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *      This subsystem contains the functions for general control of the
 *      IQ Recorder measurement.
 *-----------------------------------------------------------------------------
 */


/*===========================================================================*/
/* IQ Recorder Config Default Setting
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecDefSet (ViSession instrSession,
                                              ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG (RSCMU_sendCmd (instrSession, "DEFault:IQRecorder:CONTrol ON"));

}


/*===========================================================================*/
/* Measurement Length
/* This Function sets the measurement length. The number of measured IQ's are
/* dependent from HW option:
/* The range with HW option CMU-U65V02 and CMU-U65V04 is: 1 ... 65536.
/* The range with Default option is: 1 ... 32768.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecMesLength (ViSession instrSession,
                                                      ViInt32 measurementLength,
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQRecorder:CONTrol:CLENgth",
                            measurementLength, 1, 65536,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Time Delay
/* This Function sets the a delay between triggering event and first sample.
/* The unit is in samples.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecTDelay (ViSession instrSession,
                                                   ViInt32 sampleDelay,
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setViInt32(instrSession, "CONFigure:IQRecorder:CONTrol:TDELay",
                            sampleDelay, -32000, 32000,
                            RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Capture Timeout
/* This function defines the capture timeout is in multiples of
/* 100ms. If this time has elapsed the measurement delivers "capture_timeout";
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecCTimeout (ViSession instrSession,
                                                     ViInt32 capture_timeout,
                                                     //ViInt32 enable,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, capture_timeout, 1, 600, RSCMU_ERROR_PARAMETER2, errorMessage);
//    CHECK_VI_INT32_RANGE(instrSession, enable, RSCMU_OFF, RSCMU_ON, RSCMU_ERROR_PARAMETER3, errorMessage);

//    if (enable == RSCMU_OFF)
//      sprintf (wrtBuf, "CONFigure:IQRecorder:CONTrol:CLENgth OFF");
//    else
      sprintf (wrtBuf, "CONFigure:IQRecorder:CONTrol:CLENgth %d", capture_timeout);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}


/*===========================================================================*/
/* Filter
/* This function selects the filter type: It is possible to use the
/* Gaussian or the Nyquist filter.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecFilter (ViSession instrSession,
                                                 ViInt32 filter,
                                                 ViChar errorMessage[])
{
    ViString    list[] = {"GAUS","NYQ", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQRecorder:CONTrol:FILTer",
                         filter, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Gaussian Filter
/* This function configure the Gaussian filter .
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecGFilter (ViSession instrSession,
                                                 ViInt32 filter,
                                                 ViChar errorMessage[])
{
    ViString    list[] = {"F50K","F100K","F200K","F300K","F500K","F1M", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQRecorder:CONTrol:GFILter",
                         filter, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Nyquist Filter
/* This function configure the Nyquist filter .
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecNFilter (ViSession instrSession,
                                                 ViInt32 filter,
                                                 ViChar errorMessage[])
{
    ViString    list[] = {"F50K","F100K","F200K","F300K","F500K","F1M","F2M","F3M","F5M", VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQRecorder:CONTrol:NFILter",
                         filter, list,
                         RSCMU_ERROR_PARAMETER2));
}



/*===========================================================================*/
/* Output Format
/* This function selects the measurement output format: IQ-format (rectangular)
/* or magnitude and wraped phase (polar wrap) or magnitude and unwraped phase (polar unwrap)
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecRMode (ViSession instrSession,
                                                  ViInt32 mode,
                                                   ViChar errorMessage[])
{
    ViString    list[] = {"IQ","PLW","PLUW",VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQRecorder:CONTrol:RMODe",
                         mode, list,
                         RSCMU_ERROR_PARAMETER2));
}


/*===========================================================================*/
/* Output Format
/* This function configure of  the magnitude value of the polar format
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecLFormat (ViSession instrSession,
                                                    ViInt32 unit,
                                                     ViChar errorMessage[])
{
    ViString    list[] = {"VOLT","DBM",VI_NULL};
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQRecorder:CONTrol:LFORmat",
                         unit, list,
                         RSCMU_ERROR_PARAMETER2));
}


/*===========================================================================*/
/* Configure Subarray
/* This function configures the read, fetch and sample commands.
/* It restricts the measurement to up to 32 subranges where either
/* all measurement results or a single statistical value is returned
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecSubarrayRange (ViSession instrSession,
                                                        ViInt32 statisticsMode,
                                                        ViUInt32 start,
                                                        ViInt32 number_of_steps,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_80, RSCMU_ERROR_FW_VERSION_3_80));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, statisticsMode, 0, 4, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, start, 1, 1000, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, number_of_steps, 1, 1000, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Configures subarrays
    sprintf (wrtBuf, "CONFigure:SUBarrays:IQRecorder %s,%ld,%ld", StrStat[statisticsMode]
                                                            , start
                                                            , number_of_steps);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Get configured Filter setting
/* This function returns the configured filter type (Nyquist, Gauss),
/* the current samplingrate and actual filter bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcIQRecFSBW (ViSession instrSession,
                                               ViInt32 *filterType,
                                               ViInt32 *samplingRatekHz,
                                               ViInt32 *bandwidthkHz,
                                               ViChar errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViChar      rdBuf[RD_BUFFER_SIZE]= "";
  ViChar      Tab[100][100];

  CHECKRETMSG(RSCMU_getViString(instrSession, "FETCh:SCALar:IQRecorder:FSBW?", rdBuf,
                                  RD_BUFFER_SIZE, 0));

  // Split the multiple data values separated by a comma:
  RSCMU_SeparateDataValues (rdBuf, ",", Tab);

  // Get the Filtertype:
  if (strcmp (Tab[0], "NAN") != 0)
    sscanf (Tab[0], "%d", filterType);
  else *filterType = RSCMU_NAN_VI_INT32;

  // Get the Sampling Rate:
  if (strcmp (Tab[1], "NAN") != 0)
    sscanf (Tab[1], "%d", samplingRatekHz);
  else *samplingRatekHz = RSCMU_NAN_VI_INT32;

  // Get the Bandwidth:
  if (strcmp (Tab[2], "NAN") != 0)
    sscanf (Tab[2], "%d", bandwidthkHz);
  else *bandwidthkHz = RSCMU_NAN_VI_INT32;


  return RSCMU_Status;

}

/*===========================================================================*/
/* Init
/* Start measurement without waiting => RUN
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_InitIQRec (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "INITiate:IQRecorder"));
}

/*===========================================================================*/
/* Abort
/* Abort measurement and switch off => OFF
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_AborIQRec (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "ABORt:IQRecorder"));
}

/*===========================================================================*/
/* Stop
/* Stop measurement => STOP
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_StopIQRec (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "STOP:IQRecorder"));
}

/*===========================================================================*/
/* Continue
/* Next measurement step (only counting mode) => RUN
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ContIQRec (ViSession instrSession,
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_sendCmd(instrSession, "CONTinue:IQRecorder"));
}


/*===========================================================================*/
/* IQ/Slot status
/* This function returns status of the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetcIQRecStat (ViSession instrSession,
                                                ViInt32 *measurementStatus,
                                                ViInt32 *counter_for_current_stat_cycle,
                                                ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_RD_BUF(instrSession);

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    // Fetch the current status:
    CHECKRETMSG(RSCMU_getViString(instrSession, "FETCh:IQRecorder:STATus?", rdBuf,
                                  RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(instrSession, str, measurementStatusArr, measurementStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",counter_for_current_stat_cycle))
        *counter_for_current_stat_cycle = RSCMU_NONE;

    return RSCMU_Status;
}


/*===========================================================================*/
/* Event reporting of the measurement
/* This function defines the events generated when the
/* Measurement is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ConfIQRecErep (ViSession instrSession,
                                                ViInt32 measurementEventReporting,
                                                ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_setEnum(instrSession, "CONFigure:IQRecorder:EREPorting",
                         measurementEventReporting, modeArr, RSCMU_ERROR_PARAMETER2));
}



/* ----------------------------------------------------------------------------
 * lokal functions for IQ Recorder:
 * ----------------------------------------------------------------------------*/
ViStatus RSCMU_MeasIQRecArray (ViSession  instrSession,
                               ViChar    *StrCommand,
                               ViInt32   *measStatus,
                               ViInt32   *nbValues,
                               ViReal64   values[],
                               ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_IQREC_SIZE_ARRAY] = "";
    ViChar     *StrValue;
    ViInt16     Index = 0;
    ViUInt16    stb;

    sprintf (wrtBuf, "%s?", StrCommand);
//    if ((RSCMU_Status = RSCMU_Query_Array (instrSession, wrtBuf, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
//        return RSCMU_Status;
    CHECKRETMSG(RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));
    if (strstr(wrtBuf,"?"))
    {
        CHECKRETMSG(RSCMU_viRead(instrSession,rdBuf,RD_IQREC_SIZE_ARRAY));
    }
    RSCMU_Status = RSCMU_sysStatus(instrSession, &stb);

  // Find all values by breaking the read buffer:
  StrValue = strtok(rdBuf, ",");

  while (StrValue)
  {
    if ( Index==0)
      {
      // get the measurement status indicator on Index 0
      if (strcmp (StrValue, "OK") == 0)
        *measStatus = RSCMU_IQREC_OK;
      else if (strcmp (StrValue, "NTR") == 0)
        *measStatus = RSCMU_IQREC_NTR;
      else if (strcmp (StrValue, "OFLW") == 0)
        *measStatus = RSCMU_IQREC_OFLW;
      else if (strcmp (StrValue, "INV") == 0)
        *measStatus = RSCMU_IQREC_INV;
      else *measStatus = RSCMU_NAN_VI_INT32;
      }
    if ( Index==1)
      {
      // get Nr of Values on Index 1
      if (strcmp (StrValue, "NAN") != 0)
        sscanf (StrValue, "%d", nbValues);
      else *nbValues = RSCMU_NAN_VI_INT32;
      }
    if ( Index>=2)
      {
      // get all Values Index >1
      if (strcmp (StrValue, "NAN") != 0)
        sscanf (StrValue, "%lf", &values[Index-2]);
      else values[Index-2] = RSCMU_NAN;
      }

    StrValue = strtok (NULL, ",");
    Index++;
  }
    return RSCMU_Status;
}

ViStatus RSCMU_MeasIQRecSubArray (ViSession  instrSession,
                                  ViChar    *StrCommand,
                                  ViInt32   *measStatus,
                                  ViReal64   values[],
                                  ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_IQREC_SIZE_ARRAY] = "";
    ViChar     *StrValue;
    ViInt16     Index = 0;
    ViUInt16    stb;

    sprintf (wrtBuf, "%s?", StrCommand);
    CHECKRETMSG(RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));
    if (strstr(wrtBuf,"?"))
    {
        CHECKRETMSG(RSCMU_viRead(instrSession,rdBuf,RD_IQREC_SIZE_ARRAY));
    }
    RSCMU_Status = RSCMU_sysStatus(instrSession, &stb);

  // Find all values by breaking the read buffer:
  StrValue = strtok(rdBuf, ",");

  while (StrValue)
  {
    if ( Index==0)
      {
      // get the measurement status indicator on Index 0
      if (strcmp (StrValue, "OK") == 0)
        *measStatus = RSCMU_IQREC_OK;
      else if (strcmp (StrValue, "NTR") == 0)
        *measStatus = RSCMU_IQREC_NTR;
      else if (strcmp (StrValue, "OFLW") == 0)
        *measStatus = RSCMU_IQREC_OFLW;
      else if (strcmp (StrValue, "INV") == 0)
        *measStatus = RSCMU_IQREC_INV;
      else *measStatus = RSCMU_NAN_VI_INT32;
      }
    if ( Index>=1)
      {
      // get all Values Index >0
      if (strcmp (StrValue, "NAN") != 0)
        sscanf (StrValue, "%lf", &values[Index-1]);
      else values[Index-1] = RSCMU_NAN;
      }

    StrValue = strtok (NULL, ",");
    Index++;
  }
    return RSCMU_Status;
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQRecIAmpl (ViSession instrSession,
                                                   ViInt32 *measStatusIndicator,
                                                   ViInt32 *nr_ofValues,
                                                   ViReal64 iAmplitudes[],
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "READ:ARRay:IQRecorder:I"
                                 ,measStatusIndicator, nr_ofValues, iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQRecIAmpl (ViSession instrSession,
                                                    ViInt32 *measStatusIndicator,
                                                    ViInt32 *nr_ofValues,
                                                    ViReal64 iAmplitudes[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "FETCh:ARRay:IQRecorder:I"
                                 ,measStatusIndicator, nr_ofValues, iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQRecIAmpl (ViSession instrSession,
                                                     ViInt32 *measStatusIndicator,
                                                     ViInt32 *nr_ofValues,
                                                     ViReal64 iAmplitudes[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "SAMPle:ARRay:IQRecorder:I"
                                 ,measStatusIndicator, nr_ofValues, iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQRecIAmpl (ViSession instrSession,
                                                      ViInt32 *measStatusIndicator,
                                                      ViReal64 iAmplitudes[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "READ:SUBarrays:IQRecorder:I"
                                    ,measStatusIndicator, iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQRecIAmpl (ViSession instrSession,
                                                       ViInt32 *measStatusIndicator,
                                                       ViReal64 iAmplitudes[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "FETCh:SUBarrays:IQRecorder:I"
                                    ,measStatusIndicator, iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQRecIAmpl (ViSession instrSession,
                                                        ViInt32 *measStatusIndicator,
                                                        ViReal64 iAmplitudes[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "SAMPle:SUBarrays:IQRecorder:I"
                                    ,measStatusIndicator, iAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQRecQAmpl (ViSession instrSession,
                                                   ViInt32 *measStatusIndicator,
                                                   ViInt32 *nr_ofValues,
                                                   ViReal64 qAmplitudes[],
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "READ:ARRay:IQRecorder:Q"
                                 ,measStatusIndicator, nr_ofValues, qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQRecQAmpl (ViSession instrSession,
                                                    ViInt32 *measStatusIndicator,
                                                    ViInt32 *nr_ofValues,
                                                    ViReal64 qAmplitudes[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "FETCh:ARRay:IQRecorder:Q"
                                 ,measStatusIndicator, nr_ofValues, qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQRecQAmpl (ViSession instrSession,
                                                     ViInt32 *measStatusIndicator,
                                                     ViInt32 *nr_ofValues,
                                                     ViReal64 qAmplitudes[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "SAMPle:ARRay:IQRecorder:Q"
                                 ,measStatusIndicator, nr_ofValues, qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQRecQAmpl (ViSession instrSession,
                                                      ViInt32 *measStatusIndicator,
                                                      ViReal64 qAmplitudes[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "READ:SUBARRays:IQRecorder:Q"
                                    ,measStatusIndicator, qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQRecQAmpl (ViSession instrSession,
                                                       ViInt32 *measStatusIndicator,
                                                       ViReal64 qAmplitudes[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "FETCh:SUBARRays:IQRecorder:Q"
                                    ,measStatusIndicator, qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQRecQAmpl (ViSession instrSession,
                                                        ViInt32 *measStatusIndicator,
                                                        ViReal64 qAmplitudes[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "SAMPle:SUBARRays:IQRecorder:Q"
                                    ,measStatusIndicator, qAmplitudes, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQRecLevel (ViSession instrSession,
                                                   ViInt32 *measStatusIndicator,
                                                   ViInt32 *nr_ofValues,
                                                   ViReal64 level[],
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "READ:ARRay:IQRecorder:LEVel"
                                 ,measStatusIndicator, nr_ofValues, level, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQRecLevel (ViSession instrSession,
                                                    ViInt32 *measStatusIndicator,
                                                    ViInt32 *nr_ofValues,
                                                    ViReal64 level[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "FETCh:ARRay:IQRecorder:LEVel"
                                 ,measStatusIndicator, nr_ofValues, level, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQRecLevel (ViSession instrSession,
                                                     ViInt32 *measStatusIndicator,
                                                     ViInt32 *nr_ofValues,
                                                     ViReal64 level[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "SAMPle:ARRay:IQRecorder:LEVel"
                                 ,measStatusIndicator, nr_ofValues, level, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQRecLevel (ViSession instrSession,
                                                      ViInt32 *measStatusIndicator,
                                                      ViReal64 level[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "READ:SUBarrays:IQRecorder:LEVel"
                                    ,measStatusIndicator, level, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQRecLevel (ViSession instrSession,
                                                       ViInt32 *measStatusIndicator,
                                                       ViReal64 level[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "FETCh:SUBarrays:IQRecorder:LEVel"
                                    ,measStatusIndicator, level, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQRecLevel (ViSession instrSession,
                                                        ViInt32 *measStatusIndicator,
                                                        ViReal64 level[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "SAMPle:SUBarrays:IQRecorder:LEVel"
                                    ,measStatusIndicator, level, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQRecPhase (ViSession instrSession,
                                                   ViInt32 *measStatusIndicator,
                                                   ViInt32 *nr_ofValues,
                                                   ViReal64 phase[],
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "READ:ARRay:IQRecorder:PHASe"
                                 ,measStatusIndicator, nr_ofValues, phase, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQRecPhase (ViSession instrSession,
                                                    ViInt32 *measStatusIndicator,
                                                    ViInt32 *nr_ofValues,
                                                    ViReal64 phase[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "FETCh:ARRay:IQRecorder:PHASe"
                                 ,measStatusIndicator, nr_ofValues, phase, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQRecPhase (ViSession instrSession,
                                                     ViInt32 *measStatusIndicator,
                                                     ViInt32 *nr_ofValues,
                                                     ViReal64 phase[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "SAMPle:ARRay:IQRecorder:PHASe"
                                 ,measStatusIndicator, nr_ofValues, phase, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQRecPhase (ViSession instrSession,
                                                      ViInt32 *measStatusIndicator,
                                                      ViReal64 phase[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "READ:SUBarrays:IQRecorder:PHASe"
                                    ,measStatusIndicator, phase, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQRecPhase (ViSession instrSession,
                                                       ViInt32 *measStatusIndicator,
                                                       ViReal64 phase[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "FETCh:SUBarrays:IQRecorder:PHASe"
                                    ,measStatusIndicator, phase, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQRecPhase (ViSession instrSession,
                                                        ViInt32 *measStatusIndicator,
                                                        ViReal64 phase[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "SAMPle:SUBarrays:IQRecorder:PHASe"
                                    ,measStatusIndicator, phase, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQRecIQ (ViSession instrSession,
                                                   ViInt32 *measStatusIndicator,
                                                   ViInt32 *nr_ofValues,
                                                   ViReal64 *iq,
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "READ:ARRay:IQRecorder:IQ"
                                 ,measStatusIndicator, nr_ofValues, iq, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQRecIQ (ViSession instrSession,
                                                    ViInt32 *measStatusIndicator,
                                                    ViInt32 *nr_ofValues,
                                                    ViReal64 iq[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "FETCh:ARRay:IQRecorder:IQ"
                                 ,measStatusIndicator, nr_ofValues, iq, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQRecIQ (ViSession instrSession,
                                                     ViInt32 *measStatusIndicator,
                                                     ViInt32 *nr_ofValues,
                                                     ViReal64 iq[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "SAMPle:ARRay:IQRecorder:IQ"
                                 ,measStatusIndicator, nr_ofValues, iq, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQRecIQ (ViSession instrSession,
                                                      ViInt32 *measStatusIndicator,
                                                      ViReal64 iq[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "READ:SUBarrays:IQRecorder:IQ"
                                    ,measStatusIndicator, iq, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQRecIQ (ViSession instrSession,
                                                       ViInt32 *measStatusIndicator,
                                                       ViReal64 iq[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "FETCh:SUBarrays:IQRecorder:IQ"
                                    ,measStatusIndicator, iq, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQRecIQ (ViSession instrSession,
                                                        ViInt32 *measStatusIndicator,
                                                        ViReal64 iq[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "SAMPle:SUBarrays:IQRecorder:IQ"
                                    ,measStatusIndicator, iq, errorMessage));
}

/*===========================================================================*/
/* Read Measurement Array
/* This function  starts a single Shot measurement and return results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadArrIQRecPL (ViSession instrSession,
                                                   ViInt32 *measStatusIndicator,
                                                   ViInt32 *nr_ofValues,
                                                   ViReal64 pl[],
                                                   ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "READ:ARRay:IQRecorder:PL"
                                 ,measStatusIndicator, nr_ofValues, pl, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchArrIQRecPL (ViSession instrSession,
                                                    ViInt32 *measStatusIndicator,
                                                    ViInt32 *nr_ofValues,
                                                    ViReal64 pl[],
                                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "FETCh:ARRay:IQRecorder:PL"
                                 ,measStatusIndicator, nr_ofValues, pl, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement Array
/* This function returns the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleArrIQRecPL (ViSession instrSession,
                                                     ViInt32 *measStatusIndicator,
                                                     ViInt32 *nr_ofValues,
                                                     ViReal64 pl[],
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecArray( instrSession, "SAMPle:ARRay:IQRecorder:PL"
                                 ,measStatusIndicator, nr_ofValues, pl, errorMessage));
}

/*===========================================================================*/
/* Read Measurement SubArray
/* This function  starts a single Shot measurement and return results in a
/* defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_ReadSubArrIQRecPL (ViSession instrSession,
                                                      ViInt32 *measStatusIndicator,
                                                      ViReal64 pl[],
                                                      ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "READ:SUBarrays:IQRecorder:PL"
                                    ,measStatusIndicator, pl, errorMessage));
}

/*===========================================================================*/
/* Fetch Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_FetchSubArrIQRecPL (ViSession instrSession,
                                                       ViInt32 *measStatusIndicator,
                                                       ViReal64 pl[],
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "FETCh:SUBarrays:IQRecorder:PL"
                                    ,measStatusIndicator, pl, errorMessage));
}

/*===========================================================================*/
/* Sample Measurement SubArray
/* This function returns the measurement results in a defined Subarray.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_SampleSubArrIQRecPL (ViSession instrSession,
                                                        ViInt32 *measStatusIndicator,
                                                        ViReal64 pl[],
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    // Version manager
    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_80,RSCMU_ERROR_FW_VERSION_3_80));

    RETMSG(RSCMU_MeasIQRecSubArray( instrSession, "SAMPle:SUBarrays:IQRecorder:PL"
                                    ,measStatusIndicator, pl, errorMessage));
}

/*
 * ----------------------------------------------------------------------------
 * CLOSE
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Close
/* Purpose : This function close the Basic Unit session and release the
/*           resources
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Close_RF_NSig (ViSession RFNSigSession, ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    struct RSCMU_RF_NSig_Globals *p_globalvarRF;

    if ((RSCMU_Status = viGetAttribute (RFNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarRF)) < 0)
        return RSCMU_Base_errorMessage (RFNSigSession, RSCMU_Status, errorMessage);

    // Deallocate the space pointed by p_globalvarRF:
    if (p_globalvarRF)
        free (p_globalvarRF);

    // Deallocate all system resources allocated to the RFNSig Session:
    CHECKRET(viClose (RFNSigSession));

    return RSCMU_Status;
}

/*****************************************************************************/
/*****************************************************************************/
/*========== Class: Audio_NonSignalling =====================================*/
/*****************************************************************************/
/*****************************************************************************/
/*
 * ----------------------------------------------------------------------------
 * Initialize
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Initialize
/* Purpose : This function is used to establish communication with the Audio
/*           Function Group. Based on the Instrument Descriptor, this
/*           operation establishes communication session with the CMU and
/*           creates an Instrument Handle that is used by all other Audio
/*           operations to perform operations on that session.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Init_Audio_NSig (ViSession       BaseSession,
                                                   ViPSession      AudioNSigSession,
                                                   ViChar _VI_FAR  errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViSession   rmSession = 0; // Resource Manager Session
  ViInt32     secondAddr = RSCMU_SECONDARY_ADDRESS_INVALID;
  ViInt32     firstFreeSecAddr = RSCMU_SECONDARY_ADDRESS_INVALID;
  ViInt32     comp = -1;
  ViChar      RsrcName[25]="";
  ViBoolean   Flag = RSCMU_OFF;

  struct RSCMU_Base_Globals        *p_globalvarBase;
  struct RSCMU_Audio_NSig_Globals  *p_globalvarAudioNSig;

  /* --- Set standard "C" locale --- */
  {
    char *locale;

    locale = setlocale(LC_ALL, "C");
  }

  // Reset the Error Message
  strcpy (errorMessage, "No Error");

  // Queries the specified attribute for the Base object:
  if ((RSCMU_Status = viGetAttribute (BaseSession, VI_ATTR_USER_DATA,(ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Base_errorMessage (BaseSession, RSCMU_Status, errorMessage);

  // Find the secondary address for the Audio Function Group:
  while ((comp != 0) && (secondAddr <= RSCMU_SECONDARY_ADDRESS_MAX))
  {
    secondAddr++;
    comp = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp, "\"AUDIO_NSig\"", 12);
    if (comp == 0)
    {
      Flag = RSCMU_ON;
    }
    else
    {
      ViInt32 comp1;
      comp1 = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp, "NONE", 4);
      if (comp1 == 0 && firstFreeSecAddr == RSCMU_SECONDARY_ADDRESS_INVALID)
      {
        firstFreeSecAddr = secondAddr;
      }
    }
  }

  // If not found try to assign or error
  if (Flag == RSCMU_OFF)
  {
    if ((RSCMU_Status = RSCMU_Base_SystRemAddrSec (BaseSession, firstFreeSecAddr,
                        RSCMU_AUDIO_NSIG, errorMessage)) < 0)
      return RSCMU_Base_errorMessage (BaseSession, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);
    Flag = RSCMU_ON;
    secondAddr = firstFreeSecAddr;
  }

  // Specify the resource with which to establish a communication session:
  if (strncmp ("GPIB",p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, 4) == 0)
    sprintf (RsrcName, "%s::%ld::%ld::INSTR",p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address, secondAddr);
  else
    sprintf (RsrcName, "%s::INSTR",p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);

  // take default instrument session
  rmSession = p_globalvarBase->RSCMU_tGlobal.RSCMU_RmSessionDefault;

  if ((RSCMU_Status = viOpen (rmSession, RsrcName, VI_NULL, 3000, AudioNSigSession)) != 0)
    return RSCMU_Base_errorMessage (rmSession, RSCMU_Status, errorMessage);

  // Allocate space for the RSCMU_Audio_NSig_Globals object:
  if ((p_globalvarAudioNSig = (struct RSCMU_Audio_NSig_Globals *)malloc(sizeof(struct RSCMU_Audio_NSig_Globals))) == 0)
    return RSCMU_initCleanUp (rmSession, *AudioNSigSession, RSCMU_Status);

  memset ( p_globalvarAudioNSig, 0, sizeof ( struct RSCMU_Audio_NSig_Globals ) );
  if ((RSCMU_Status = viSetAttribute(*AudioNSigSession, VI_ATTR_USER_DATA,(ViAttrState)p_globalvarAudioNSig)) < 0)
  {
    free( p_globalvarAudioNSig );
    return RSCMU_initCleanUp (rmSession, *AudioNSigSession, RSCMU_Status);
  }

  // Set values of variables for the Audio Function Group:
  strcpy (p_globalvarAudioNSig->RSCMU_tGlobal.FGrp_Name, "AUDIO_NSig");
  strcpy (p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_SCPIConnection, p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_primary_address = p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address;
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_secondary_address = secondAddr;
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_TRACE = p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE;
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_SessionNb = *AudioNSigSession;
  if (p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_TRACE == RSCMU_ON)
    p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_AddrTargetFile = p_globalvarBase->RSCMU_tGlobal.RSCMU_AddrTargetFile;

  // Base CMU FW available
  strcpy (p_globalvarAudioNSig->RSCMU_tGlobal.CmuVersion, p_globalvarBase->RSCMU_tGlobal.CmuVersion);
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_FW_Version = p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version;

  // Record the open session:
  p_globalvarBase->RSCMU_tSecAddr[secondAddr].SessionNb = *AudioNSigSession;

  // Timeout set to 15 seconds:
  CHECKRET(viSetAttribute (*AudioNSigSession, VI_ATTR_TMO_VALUE, RSCMU_TMO_VALUE));

  CHECKRET(RSCMU_setCheckStatus(*AudioNSigSession, VI_TRUE));
  return RSCMU_Status;
}

/*
 *-----------------------------------------------------------------------------
 * CONNECTION CONTROL
 * ------------------
 * The functions presented in this section determine the signals analyzed and
 * generated by the CMU. It corresponds to the settings in the popup menu ofthe
 * softkey Connect. Control, located to the right of the headline of each main
 * menu.
 * ----------------------------------------------------------------------------
 * Subsystem for AF Input/Output
 *
 * This subsystem contains the functions to determine the input source of the
 * CMU speech encoder and the output destination of its speech decoder. It
 * corresponds to the panel AF Connector Setup in the index card AF/RF in the
 * popup menu Connect. Control.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Speech Encoder
/* Purpose : This command determines the input source that feeds the CMU
/*           speech encoder (option CMU-B52).
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_Audio_NSig_RoutSpen (ViSession  AudioNSigSession,
                                             ViInt32    Source,
                                             ViChar     _VI_FAR  errorMessage[])
{
    ViString    list[] = {"HAND","GEN","AUX1", VI_NULL};

    RETMSG(RSCMU_setEnum(AudioNSigSession, "ROUT:SPEN:INP",
                         Source, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Speech Decoder
/* Purpose : This command routes the CMU speech decoder output
/*           (option CMU-B52).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_RoutSpd (ViSession       AudioNSigSession,
                                            ViInt32         Destination,
                                            ViChar _VI_FAR  errorMessage[])
{
    ViString    list[] = {"HAND","ANAL","AUX2", VI_NULL};

    RETMSG(RSCMU_setEnum(AudioNSigSession, "ROUT:SPD:OUTP",
                         Destination, list,
                         RSCMU_ERROR_PARAMETER2));
}

/* ----------------------------------------------------------------------------
 * Subsystem AF Level
 *
 * The subsystem AFLevel determines the input level for the currently used input.
 * It corresponds to the Mode and Manual Level section in the Control tab of the
 * Analyzer Configuration menu and to the Analyzer Level softkey with the hotkeys
 * Mode and AF Max. Level in the Audio Multitone measurement menu.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used with ON, this function sets all the parameters of the
/*           subsystem to their default values. (OFF has no effect).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SensAflDef (ViSession      AudioNSigSession,
                                               ViInt32        defaultSettings,
                                               ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    RETMSG(RSCMU_setEnum(AudioNSigSession, "AFL:DEF",
                         defaultSettings, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Input level - Mode
/* Purpose : This function defines the mode for setting the maximum input level.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SensAflMode (ViSession      AudioNSigSession,
                                                ViInt32        mode,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"MAN","AUT", VI_NULL};

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    RETMSG(RSCMU_setEnum(AudioNSigSession, "AFL:MODE",
                         mode, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: ConfLevelMode
/* Purpose : This function defines the mode for setting the maximum input
/*           level.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SensAflModeExt (ViSession instrumentHandle,
                                                   ViInt32 channel, ViInt32 mode,
                                                   ViChar errorMessage[])
{
  ViChar    buffer[RD_BUFFER_SIZE] = "";
  ViString    StrMode[]= {"MAN","AUT", VI_NULL};

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, RSCMU_AF_CHAN_1, RSCMU_AF_CHAN_2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "SENS:AFL:%s:MODE", audioAnalyzerChanArr[channel]);
               
    RETMSG(RSCMU_setEnum(instrumentHandle,
                       buffer,
                       mode,
                       StrMode,
                       RSCMU_ERROR_PARAMETER3));
  
}

/*===========================================================================*/
/* Function: AF Generator Level
/* Purpose : This function defines the maximum expected AF input level.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SensAflMax (ViSession      AudioNSigSession,
                                               ViReal64       AFMaxLevelmV,
                                               ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    RETMSG(RSCMU_setViReal64(AudioNSigSession,"AFL:MAX",
                             AFMaxLevelmV/1000.0, 0.0, 30.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Max Level                                                       */
/* Purpose : This function defines the maximum expected AF input level.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SensAflMaxExt (ViSession instrumentHandle,
                                                  ViInt32 channel,
                                                  ViReal64 maxLevel,
                                                  ViChar errorMessage[])
{
    ViChar    buffer[RD_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, RSCMU_AF_CHAN_1, RSCMU_AF_CHAN_2, RSCMU_ERROR_PARAMETER2, errorMessage);
  
    sprintf (buffer, "SENS:AFL:%s:MAX", audioAnalyzerChanArr[channel]);
    RETMSG (RSCMU_setViReal64 (instrumentHandle,
                               buffer,
                               maxLevel,
                               0.0, 30.0,
                               RSCMU_ERROR_PARAMETER3)); 
}

/*
 *-----------------------------------------------------------------------------
 * Generator Object AFGenerator
 * ----------------------------
 * The subsystem AFGenerator contains the commands for controlling and
 * configuring the internal AF generator. It corresponds to the measurement
 * softkey Generator in the measurement menu Audio Analyzer/Generator and the
 * associated input fields.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: AF Generator Level
/* Purpose : This function determines the AF generator level on AF OUT.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SourAfgLev (ViSession  AudioNSigSession,
                                               ViReal64   AFLevelmV,
                                               ViChar     _VI_FAR  errorMessage[])
{
    RETMSG(RSCMU_setViReal64(AudioNSigSession, "SOUR:AFG:LEV",
                             AFLevelmV/1000.0, 0.0, 5.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: AF auxiliary Generator Level
/* Purpose : This function determines the AF generator level on auxiliary
/*           output AUX2.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SourAfg2Lev (ViSession  AudioNSigSession,
                                                ViReal64   AFLevelmV,
                                                ViChar     _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    RETMSG(RSCMU_setViReal64(AudioNSigSession,"SOUR:AFG:SEC:LEV",
                             AFLevelmV/1000.0, 0.0, 5.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: AF Generator Frequency
/* Purpose : This function determines the AF generator frequency.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SourAfgFreq (ViSession       AudioNSigSession,
                                                ViReal64        AFFrequencyHz,
                                                ViChar _VI_FAR  errorMessage[])
{
    RETMSG(RSCMU_setViReal64(AudioNSigSession, "SOUR:AFG:FREQ",
                             AFFrequencyHz, 20.0, 20000.0,
                             RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: AF auxiliary Generator Frequency
/* Purpose : This function determines the AF generator frequency on auxiliary
/*           output AUX2.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SourAfg2Freq (ViSession       AudioNSigSession,
                                                 ViReal64        AFFrequencyHz,
                                                 ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    RETMSG(RSCMU_setViReal64(AudioNSigSession,"SOUR:AFG:SEC:FREQ",
                              AFFrequencyHz, 20.0, 20000.0,
                              RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: Start AF generator
/* Purpose : This function starts the AF generator, setting it to the status
/*           "RUN".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_InitAfg (ViSession       AudioNSigSession,
                                            ViChar _VI_FAR  errorMessage[])
{
    RETMSG(RSCMU_sendCmd (AudioNSigSession, "INIT:AFG"));
}

/*===========================================================================*/
/* Function: Start AF auxiliary generator
/* Purpose : This function starts the AF auxiliary generator, setting it to
/*           the status "RUN".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_InitAfg2 (ViSession       AudioNSigSession,
                                             ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Start the AF generator:
    RETMSG(RSCMU_sendCmd (AudioNSigSession, "INIT:AFG:SEC"));
}

/*===========================================================================*/
/* Function: Abort and switch off AF generator
/* Purpose : This function abort and switch off the AF generator, setting it to
/*           the status "OFF".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_AborAfg (ViSession       AudioNSigSession,
                                            ViChar _VI_FAR  errorMessage[])
{
    RETMSG(RSCMU_sendCmd (AudioNSigSession, "ABOR:AFG"));
}

/*===========================================================================*/
/* Function: Abort and switch off AF auxiliary generator
/* Purpose : This function abort and switch off the AF auxiliary generator,
/*           setting it to the status "OFF".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_AborAfg2 (ViSession       AudioNSigSession,
                                             ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Switch off the AF generator:
    RETMSG(RSCMU_sendCmd (AudioNSigSession, "ABOR:AFG:SEC"));
}

/*===========================================================================*/
/* Function: AF generator status
/* Purpose : This function returns the current AF generator status.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcAfgStat (ViSession       AudioNSigSession,
                                                ViPInt32        AFGeneratorStatus,
                                                ViChar _VI_FAR  errorMessage[])
{
    RETMSG(RSCMU_getEnum(AudioNSigSession, "FETC:AFG:STAT?",
                        AFGeneratorStatus, genStatArr,
                        RSCMU_ERROR_GEN_STAT_UNKNOWN));
}

/*===========================================================================*/
/* Function: AF auxiliary generator status
/* Purpose : This function returns the current AF auxiliary generator status.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcAfg2Stat (ViSession       AudioNSigSession,
                                                 ViPInt32        AFGeneratorStatus,
                                                 ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    RETMSG(RSCMU_getEnum(AudioNSigSession,"FETC:AFG:SEC:STAT?",
                         AFGeneratorStatus, genStatArr,
                         RSCMU_ERROR_GEN_STAT_UNKNOWN));
}

/*===========================================================================*/
/* Function: AF generator - Signal Mode
/* Purpose : This function defines the mode for setting AC or DC Signal.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SetAfglSignMode (ViSession      AudioNSigSession,
                                                ViInt32        SignalMode,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"AC","DC", VI_NULL};

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(AudioNSigSession, "SOUR:AFG:SMOD",
                         SignalMode, list,
                         RSCMU_ERROR_PARAMETER2));
}

ViStatus _VI_FUNC RSCMU_Audio_NSig_SetAfg2SignMode (ViSession      AudioNSigSession,
                                                ViInt32        SignalMode,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"AC","DC", VI_NULL};

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_10));

    RETMSG(RSCMU_setEnum(AudioNSigSession, "SOUR:AFG:SEC:SMOD",
                         SignalMode, list,
                         RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Function: ConfAnalyzerFiltACCoup                                                
/* Purpose : This function selects the first band pass in the AF analyzer to
/*           be used if the AF path coupling is set to AC.
/*           Note(s):
/*           (1) This function is available only with option R&S CBT-B41.
/* Firmware: V4.25
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAnFiltACCoup (ViSession instrumentHandle,
                                                     ViInt32 channel,
                                                     ViInt32 bandwidthACCoupling,
                                                     ViChar errorMessage[])
{
  ViChar    buffer[RD_BUFFER_SIZE] = "";
                                    
  // Check if the function is correctly used:       
  CHECK_VI_INT32_RANGE(instrumentHandle, channel, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);
  
  sprintf (buffer, "CONF:AFAN:%s:FILT:BPAS:ACC", audioAnalyzerChanArr[channel]);
   
  RETMSG (RSCMU_setEnum (instrumentHandle,
                             buffer,
                             bandwidthACCoupling,
                             bandPassDCCouplingArr,
                             RSCMU_ERROR_PARAMETER3));
}                                    

/*===========================================================================*/
/* Function: ConfAnalyzerFiltDCCoup                                                
/* Purpose : This function selects the first band pass in the AF analyzer to
/*           be used if the AF path coupling is set to DC.
/*           Note(s):
/*           (1) This function is available only with option R&S CBT-B41.
/* Firmware: V4.25
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAnFiltDCCoup (ViSession instrumentHandle,
                                                     ViInt32 channel,
                                                     ViInt32 bandwidthDCCoupling,
                                                     ViChar errorMessage[])
{
  ViChar    buffer[RD_BUFFER_SIZE] = "";

  // Check if the function is correctly used:       
  CHECK_VI_INT32_RANGE(instrumentHandle, channel, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

  sprintf (buffer, "CONF:AFAN:%s:FILT:BPAS:DCC", audioAnalyzerChanArr[channel]);
  if ((bandwidthDCCoupling == RSCMU_BP01) ||
      (bandwidthDCCoupling == RSCMU_BP04) ||
      (bandwidthDCCoupling == RSCMU_BP08) ||
      (bandwidthDCCoupling == RSCMU_BP12) ||
      (bandwidthDCCoupling == RSCMU_BP16))
  {
      RETMSG (RSCMU_ERROR_PARAMETER3);
  }
      
      
  RETMSG (RSCMU_setEnum (instrumentHandle,
                             buffer,
                             bandwidthDCCoupling,
                             bandPassDCCouplingArr,
                             RSCMU_ERROR_PARAMETER3)); 
}

/*
 *-----------------------------------------------------------------------------
 * Measurement Object AFANalyser
 *
 * The measurement object AFANalyzer contains the commands for measuring the
 * audio signal. It corresponds to the Analyzer softkey in the main menu Audio
 * Analyzer/Generator and the associated output fields.
 * ----------------------------------------------------------------------------
 * Subsystem AFANalyzer (Measurement Control)
 *
 * The subsystem AFAnalyzer contains the commands for general control of the
 * audio analysis.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start new AF measurement
/* Purpose : This function starts the AF analyzer, setting it to the status
/*           "RUN".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_InitAfan (ViSession       AudioNSigSession,
                                             ViInt32         AFInputConnector,
                                             ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start new AF measurement:
    sprintf (wrtBuf, "INIT:AFAN:%s", AFInputArr[AFInputConnector]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Abort and switch off AF analyser
/* Purpose : This function abort and switch off the AF analyser, setting it to
/*           the status "OFF".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_AborAfan (ViSession       AudioNSigSession,
                                             ViInt32         AFInputConnector,
                                             ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start new AF measurement:
    sprintf (wrtBuf, "ABOR:AFAN:%s", AFInputArr[AFInputConnector]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Stop AF analyser
/* Purpose : This function abort and switch off the AF analyser, setting it to
/*           the status "STOP".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_StopAfan (ViSession AudioNSigSession,
                                             ViInt32   AFInputConnector,
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start new AF measurement:
    sprintf (wrtBuf, "STOP:AFAN:%s", AFInputArr[AFInputConnector]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Restart a new measurement
/* Purpose : This function abort and switch off the AF analyser, setting it to
/*           the status "OFF".
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ContAfan (ViSession AudioNSigSession,
                                             ViInt32   AFInputConnector,
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start new AF measurement:
    sprintf (wrtBuf, "CONT:AFAN:%s", AFInputArr[AFInputConnector]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*
 * ----------------------------------------------------------------------------
 * Subsystem AFANalyzer:CONTrol (Control)
 *
 * The subsystem AFAnalyzer:CONTrol contains the commands for the configuration
 * of the audio analysis.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Test cycles
/* Purpose : This function determines the number of statistics cycles. A stop
/*           condition is not available.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanContRep (ViSession       AudioNSigSession,
                                                    ViInt32         AFInputConnector,
                                                    ViInt32         Repetition,
                                                    ViInt32         StopCondition,
                                                    ViInt32         StepMode,
                                                    ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, Repetition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, StepMode,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Configure tests cycles:
    sprintf (wrtBuf, "CONF:AFAN:%s:CONT:REP %s,%s,%s", AFInputArr[AFInputConnector], StrRepetition[Repetition], StrStopCondition[StopCondition], StrStepMode[StepMode]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: AF Path Coupling
/* Purpose : This function determines the AF path coupling.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanContCoup (ViSession       AudioNSigSession,
                                                     ViInt32         AFInputConnector,
                                                     ViInt32         AFPathCoupling,
                                                     ViChar _VI_FAR  errorMessage[])
{
    ViString    StrAFPathCoupling[] = {"AC","DC"};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector, 0,1,RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFPathCoupling,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Configure AF path coupling:
    sprintf (wrtBuf, "CONF:AFAN:%s:CONT:COUP %s", AFInputArr[AFInputConnector], StrAFPathCoupling[AFPathCoupling]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Reduce Measurement Time
/* Purpose : This command is to restrict the frequency range the audio analyzer
/*           uses to calculate the measurement results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanMtr (ViSession      AudioNSigSession,
                                                ViInt32        AFInputConnector,
                                                ViInt32        Mode,
                                                ViInt32        Frequency,
                                                ViChar _VI_FAR errorMessage[])
{
    ViString    StrAFFrequencyMode[]  = {"LOWF","EXPF"};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, Mode, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, Frequency, 10, 20000, RSCMU_ERROR_PARAMETER4, errorMessage);

    sprintf (wrtBuf, "CONF:AFAN:%s:MTR %s,%ld", AFInputArr[AFInputConnector]
                                           , StrAFFrequencyMode[Mode]
                                           , Frequency);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Distorsion / Frequency
/* Purpose : This function determines the AF frequency distorsion.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanContDistFreq (ViSession      AudioNSigSession,
                                                         ViInt32        AFInputConnector,
                                                         ViReal64       DistorsionFrequencyHz,
                                                         ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, DistorsionFrequencyHz, 20.0, 20000.0, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Set the distorsion frequency:
    sprintf (wrtBuf, "CONF:AFAN:%s:CONT:DIST:FREQ %.12f HZ", AFInputArr[AFInputConnector], DistorsionFrequencyHz);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*
 * ----------------------------------------------------------------------------
 * Subsystem AFANalyzer...:FILTer (Filter)
 *
 * The subsystem AFANalyzer:...FILTer contains the commands for the
 * configuration of the audio analysis filter.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: VBP frequency
/* Purpose : This function determines the center frequency of the variable
/*           band pass.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanFiltVBPFreq (ViSession AudioNSigSession,
                                                        ViInt32   AFInputConnector,
                                                        ViReal64  FrequencyHz,
                                                        ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

      // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, FrequencyHz, 20.0, 20000.0, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Set the VBP center frequency:
    sprintf (wrtBuf, "CONF:AFAN:%s:FILT:VBP:CFR %.12f HZ", AFInputArr[AFInputConnector], FrequencyHz);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: VBP Bandwidth
/* Purpose : This function determines the bandwidth of the variable band pass.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanFiltVBPBW (ViSession AudioNSigSession,
                                                      ViInt32   AFInputConnector,
                                                      ViReal64  bandwidthHz,
                                                      ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, bandwidthHz, 10.0, 10000.0, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Set the VBP bandwidth:
    sprintf (wrtBuf, "CONF:AFAN:%s:FILT:VBP:BWID %.12f HZ", AFInputArr[AFInputConnector], bandwidthHz);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Weighing Filter
/* Purpose : This function selects the weighting filter after the fixed band
/*           pass.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanFiltWeight (ViSession AudioNSigSession,
                                                       ViInt32   AFInputConnector,
                                                       ViInt32   Filter,
                                                       ViChar    errorMessage[])
{
    ViString    StrWeightingFilter[] =  {"OFF","CME","CCI","A", VI_NULL};
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, Filter, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Select the weighting filter:
    sprintf (wrtBuf, "CONF:AFAN:%s:FILT:WEIG %s", AFInputArr[AFInputConnector], StrWeightingFilter[Filter]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Fixed Band Pass Selection
/* Purpose : This function selects the first band pass in the AF Analyser.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanFiltBPass (ViSession AudioNSigSession,
                                                      ViInt32   AFInputConnector,
                                                      ViInt32   BandPass,
                                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, BandPass, 1, 19, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Select the first band pass:
    sprintf (wrtBuf, "CONF:AFAN:%s:FILT:BPAS BP%02ld", AFInputArr[AFInputConnector], BandPass);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the
/*           spectrum measurement is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfAfanErep (ViSession AudioNSigSession,
                                                 ViInt32   AFInputConnector,
                                                 ViInt32   mode,
                                                 ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status;
    GET_WRT_BUF(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, mode, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (wrtBuf, "CONF:AFAN:%s:EREP %s", AFInputArr[AFInputConnector], modeArr[mode]);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*
 *-----------------------------------------------------------------------------
 * Subsystem AFANalyzer[:RESult] (Measured Values)
 *
 * The subsystem AFAnalyzer[:RESult] contains the commands for reading the
 * results of the audio analysis.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start single shot measurement and return results
/* Purpose : This function start a measurement and output all scalar
/*           measurement results. These are:
/*                     Peak voltage
/*                     RMS voltage
/*                     DC Voltage
/*                     Distorsion
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadAfan (ViSession       AudioNSigSession,
                                             ViInt32         AFInputConnector,
                                             ViPReal64       AFPeakLevelmv,
                                             ViPReal64       AFRMSLevelmv,
                                             ViPReal64       AFDCVoltage,
                                             ViPReal64       AFDistorsion,
                                             ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS(AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "READ:AFAN:%s?", AFInputArr[AFInputConnector]);

    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf,
                               rdBuf, RD_BUFFER_SIZE,0));

    GET_VI_REAL64(*AFPeakLevelmv,rdBuf);
    GET_VI_REAL64(*AFRMSLevelmv,NULL);
    GET_VI_REAL64(*AFDCVoltage,NULL);
    GET_VI_REAL64(*AFDistorsion,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Start single shot measurement and return results
/* Purpose : This function starts a measurement and outputs all scalar
/*           measurement results. See also the input path configuration in
/*           the class "filter" help. These are:
/*
/*           - Peak voltage 1 & RMS voltage 1: values of AC voltage after
/*             first band pass and the weighting filter
/*           - DC voltage
/*           - Distorsion
/*           - Peak voltage 2 & RMS voltage 2: values of AC voltage after
/*             first band pass and second band pass (variable band pass)
/*           - Frequency counter
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadAfanWF_VBP (ViSession       AudioNSigSession,
                                                   ViInt32         AFInputConnector,
                                                   ViPReal64       AFPeakLevelWFmv,
                                                   ViPReal64       AFRMSLevelWFmv,
                                                   ViPReal64       AFDCVoltage,
                                                   ViPReal64       AFDistorsion,
                                                   ViPReal64       AFPeakLevelVBPmv,
                                                   ViPReal64       AFRMSLevelVBPmv,
                                                   ViPReal64       AFFrequency,
                                                   ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "READ:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*AFPeakLevelWFmv,rdBuf);
    GET_VI_REAL64(*AFRMSLevelWFmv,NULL);
    GET_VI_REAL64(*AFDCVoltage,NULL);
    GET_VI_REAL64(*AFDistorsion,NULL);
    GET_VI_REAL64(*AFPeakLevelVBPmv,NULL);
    GET_VI_REAL64(*AFRMSLevelVBPmv,NULL);
    GET_VI_REAL64(*AFFrequency,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Read out measurement results (unsynchronized)
/* Purpose : This function read out a measurement and output all scalar
/*           measurement results. These are:
/*                     Peak voltage
/*                     RMS voltage
/*                     DC Voltage
/*                     Distorsion
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcAfan (ViSession       AudioNSigSession,
                                             ViInt32         AFInputConnector,
                                             ViPReal64       AFPeakLevelmv,
                                             ViPReal64       AFRMSLevelmv,
                                             ViPReal64       AFDCVoltage,
                                             ViPReal64       AFDistorsion,
                                             ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "FETC:AFAN:%s?", AFInputArr[AFInputConnector]);

    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf,
                               rdBuf, RD_BUFFER_SIZE,
                               0));

    GET_VI_REAL64(*AFPeakLevelmv,rdBuf);
    GET_VI_REAL64(*AFRMSLevelmv,NULL);
    GET_VI_REAL64(*AFDCVoltage,NULL);
    GET_VI_REAL64(*AFDistorsion,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Read out measurement results (unsynchronized)
/* Purpose : This function starts a measurement and outputs all scalar
/*           measurement results. See also the input path configuration in
/*           the class "filter" help. These are:
/*
/*           - Peak voltage 1 & RMS voltage 1: values of AC voltage after
/*             first band pass and the weighting filter
/*           - DC voltage
/*           - Distorsion
/*           - Peak voltage 2 & RMS voltage 2: values of AC voltage after
/*             first band pass and second band pass (variable band pass)
/*           - Frequency counter
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcAfanWF_VBP (ViSession       AudioNSigSession,
                                                   ViInt32         AFInputConnector,
                                                   ViPReal64       AFPeakLevelWFmv,
                                                   ViPReal64       AFRMSLevelWFmv,
                                                   ViPReal64       AFDCVoltage,
                                                   ViPReal64       AFDistorsion,
                                                   ViPReal64       AFPeakLevelVBPmv,
                                                   ViPReal64       AFRMSLevelVBPmv,
                                                   ViPReal64       AFFrequency,
                                                   ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS(AudioNSigSession);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "FETC:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*AFPeakLevelWFmv,rdBuf);
    GET_VI_REAL64(*AFRMSLevelWFmv,NULL);
    GET_VI_REAL64(*AFDCVoltage,NULL);
    GET_VI_REAL64(*AFDistorsion,NULL);
    GET_VI_REAL64(*AFPeakLevelVBPmv,NULL);
    GET_VI_REAL64(*AFRMSLevelVBPmv,NULL);
    GET_VI_REAL64(*AFFrequency,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Read out measurement results (synchronized)
/* Purpose : This function read out a measurement and output all scalar
/*           measurement results. These are:
/*                     Peak voltage
/*                     RMS voltage
/*                     DC Voltage
/*                     Distorsion
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SampAfan (ViSession AudioNSigSession,
                                             ViInt32   AFInputConnector,
                                             ViPReal64 AFPeakLevelmv,
                                             ViPReal64 AFRMSLevelmv,
                                             ViPReal64 AFDCVoltage,
                                             ViPReal64 AFDistorsion,
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession );

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "SAMP:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*AFPeakLevelmv,rdBuf);
    GET_VI_REAL64(*AFRMSLevelmv,NULL);
    GET_VI_REAL64(*AFDCVoltage,NULL);
    GET_VI_REAL64(*AFDistorsion,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Read out measurement results (synchronized)
/* Purpose : This function starts a measurement and outputs all scalar
/*           measurement results. See also the input path configuration in
/*           the class "filter" help. These are:
/*
/*           - Peak voltage 1 & RMS voltage 1: values of AC voltage after
/*             first band pass and the weighting filter
/*           - DC voltage
/*           - Distorsion
/*           - Peak voltage 2 & RMS voltage 2: values of AC voltage after
/*             first band pass and second band pass (variable band pass)
/*           - Frequency counter
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SampAfanWF_VBP (ViSession       AudioNSigSession,
                                                   ViInt32         AFInputConnector,
                                                   ViPReal64       AFPeakLevelWFmv,
                                                   ViPReal64       AFRMSLevelWFmv,
                                                   ViPReal64       AFDCVoltage,
                                                   ViPReal64       AFDistorsion,
                                                   ViPReal64       AFPeakLevelVBPmv,
                                                   ViPReal64       AFRMSLevelVBPmv,
                                                   ViPReal64       AFFrequency,
                                                   ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "SAMP:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*AFPeakLevelWFmv,rdBuf);
    GET_VI_REAL64(*AFRMSLevelWFmv,NULL);
    GET_VI_REAL64(*AFDCVoltage,NULL);
    GET_VI_REAL64(*AFDistorsion,NULL);
    GET_VI_REAL64(*AFPeakLevelVBPmv,NULL);
    GET_VI_REAL64(*AFRMSLevelVBPmv,NULL);
    GET_VI_REAL64(*AFFrequency,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: AF analyzer status
/* Purpose : This function returns the current AF generator status.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcAfanStat (ViSession AudioNSigSession,
                                                 ViInt32   AFInputConnector,
                                                 ViInt32   *AFAnalyzerStatus,
                                                 ViInt32   *NbCount,
                                                 ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);

    // Fetch the current AF generator status:
    sprintf(wrtBuf,"FETC:AFAN:%s:STAT?",AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf,rdBuf, RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(AudioNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(AudioNSigSession, str, measurementStatusArr, AFAnalyzerStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(AudioNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",NbCount))
        *NbCount = RSCMU_NONE;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Start single shot measurement and return results
/* Purpose : This function starts a measurement and outputs all scalar
/*           measurement results. See also the input path configuration in
/*           the class "filter" help. These are:
/*
/*           - Peak voltage 1 & RMS voltage 1: values of AC voltage after
/*             first band pass and the weighting filter
/*           - DC voltage
/*           - Distorsion
/*           - Peak voltage 2 & RMS voltage 2: values of AC voltage after
/*             first band pass and second band pass (variable band pass)
/*           - Frequency counter
/*           - SINAD
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadAfanWF_SINAD (ViSession AudioNSigSession,
                                                     ViInt32 AFInputConnector,
                                                     ViReal64 *ACVoltage1_Peak,
                                                     ViReal64 *ACVoltage1_RMS,
                                                     ViReal64 *DCVoltage,
                                                     ViReal64 *distorsion,
                                                     ViReal64 *ACVoltage2_Peak,
                                                     ViReal64 *ACVoltage2_RMS,
                                                     ViReal64 *frequency_counter,
                                                     ViReal64 *SINAD,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "READ:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*ACVoltage1_Peak,rdBuf);
    GET_VI_REAL64(*ACVoltage1_RMS,NULL);
    GET_VI_REAL64(*DCVoltage,NULL);
    GET_VI_REAL64(*distorsion,NULL);
    GET_VI_REAL64(*ACVoltage2_Peak,NULL);
    GET_VI_REAL64(*ACVoltage2_RMS,NULL);
    GET_VI_REAL64(*frequency_counter,NULL);
    GET_VI_REAL64(*SINAD,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Returns results of single measurement (unsynchronized)
/* Purpose : This function starts a measurement and outputs all scalar
/*           measurement results. See also the input path configuration in
/*           the class "filter" help. These are:
/*
/*           - Peak voltage 1 & RMS voltage 1: values of AC voltage after
/*             first band pass and the weighting filter
/*           - DC voltage
/*           - Distorsion
/*           - Peak voltage 2 & RMS voltage 2: values of AC voltage after
/*             first band pass and second band pass (variable band pass)
/*           - Frequency counter
/*           - SINAD
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcAfanWF_SINAD (ViSession AudioNSigSession,
                                                     ViInt32 AFInputConnector,
                                                     ViReal64 *ACVoltage1_Peak,
                                                     ViReal64 *ACVoltage1_RMS,
                                                     ViReal64 *DCVoltage,
                                                     ViReal64 *distorsion,
                                                     ViReal64 *ACVoltage2_Peak,
                                                     ViReal64 *ACVoltage2_RMS,
                                                     ViReal64 *frequency_counter,
                                                     ViReal64 *SINAD,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "FETC:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*ACVoltage1_Peak,rdBuf);
    GET_VI_REAL64(*ACVoltage1_RMS,NULL);
    GET_VI_REAL64(*DCVoltage,NULL);
    GET_VI_REAL64(*distorsion,NULL);
    GET_VI_REAL64(*ACVoltage2_Peak,NULL);
    GET_VI_REAL64(*ACVoltage2_RMS,NULL);
    GET_VI_REAL64(*frequency_counter,NULL);
    GET_VI_REAL64(*SINAD,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Returns results of single measurement (synchronized)
/* Purpose : This function starts a measurement and outputs all scalar
/*           measurement results. See also the input path configuration in
/*           the class "filter" help. These are:
/*
/*           - Peak voltage 1 & RMS voltage 1: values of AC voltage after
/*             first band pass and the weighting filter
/*           - DC voltage
/*           - Distorsion
/*           - Peak voltage 2 & RMS voltage 2: values of AC voltage after
/*             first band pass and second band pass (variable band pass)
/*           - Frequency counter
/*           - SINAD
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SampAfanWF_SINAD (ViSession AudioNSigSession,
                                                     ViInt32 AFInputConnector,
                                                     ViReal64 *ACVoltage1_Peak,
                                                     ViReal64 *ACVoltage1_RMS,
                                                     ViReal64 *DCVoltage,
                                                     ViReal64 *distorsion,
                                                     ViReal64 *ACVoltage2_Peak,
                                                     ViReal64 *ACVoltage2_RMS,
                                                     ViReal64 *frequency_counter,
                                                     ViReal64 *SINAD,
                                                     ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFInputConnector,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Start single shot measurement and return results:
    sprintf (wrtBuf, "SAMP:AFAN:%s?", AFInputArr[AFInputConnector]);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    GET_VI_REAL64(*ACVoltage1_Peak,rdBuf);
    GET_VI_REAL64(*ACVoltage1_RMS,NULL);
    GET_VI_REAL64(*DCVoltage,NULL);
    GET_VI_REAL64(*distorsion,NULL);
    GET_VI_REAL64(*ACVoltage2_Peak,NULL);
    GET_VI_REAL64(*ACVoltage2_RMS,NULL);
    GET_VI_REAL64(*frequency_counter,NULL);
    GET_VI_REAL64(*SINAD,NULL);

    return RSCMU_Status;
}

/*
 *-----------------------------------------------------------------------------
 * Test Object MULTitone
 *
 * The subsystem MULTitone measures the level of an audio test signal comprising
 * up to 20 test tones. The subsystem corresponds to the measurement menu Audio
 * Multitone and the associated popup menu Audio Multitone Configuration. In
 * analogy to the AFGenerator and AFANalyzer subsystems reported above, the Audio
 * Multitone measurement provides two independent circuits:
 *
 * - In the first audio channel (subsystem MULTitone:AF1Channel), the audio
 *   signals are applied to the connectors AF OUT (output, AF generator signal)
 *   and AF IN (input) on the CMU front panel. The first audio channel corresponds
 *   to the Audio Multitone menu and the associated configuration menu.
 *
 * - In the second audio channel (subsystem MULTitone:AF2Channel), the audio
 *   signals are applied to the connectors AUX 2 (output, AF generator signal) and
 *   AUX 1 (input) on the CMU front panel. The secondary audio circuit can not be
 *   controlled manually. With the exception of the input and output connectors,
 *   the two audio circuits are identical. All remote control commands are analogous.
 * ----------------------------------------------------------------------------
 * Subsystem MULTitone:CONTrol (Control)
 *
 * The subsystem MULTitone controls the measurement. It corresponds to the softkey
 * AF Chan. One in the measurement menu Audio Multitone and some of the associated
 * hotkeys.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Grid ON/OFF
/* Purpose : This function switches the grid lines in the test diagram on or off
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_DispMultGrid (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32        enable,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "DISP:MULT:AF%ldC:GRID", AFChannel);
    RETMSG(RSCMU_setEnum(AudioNSigSession,
                         wrtBuf, enable, mode_On_Off,
                         RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: AF reference channel
/* Purpose : This function defines the AF reference level, i.e. the 0-dB line
/*           in the test diagram.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultRlev (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViReal64       RefLevelmV,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, RefLevelmV, 0.0, 5000.0, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Set the AF reference level:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:RLEV %.12f MV", AFChannel, RefLevelmV);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: AF reference channel
/* Purpose : This function defines the AF reference level, i.e. the 0-dB line
/*           in the test diagram.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultRmod (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32        RelativeTo,
                                                 ViInt32        ToneNumber,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, RelativeTo,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 20, RSCMU_ERROR_PARAMETER4, errorMessage);

    if(RelativeTo == RSCMU_RLEV)
        sprintf (wrtBuf, "CONF:MULT:AF%ldC:RMOD RLEV", AFChannel);
    else
        sprintf (wrtBuf, "CONF:MULT:AF%ldC:RMOD TON%ld", AFChannel
                                                     , ToneNumber);

    RETMSG(RSCMU_sendCmd(AudioNSigSession,wrtBuf));
}

/*===========================================================================*/
/* Function: AF Generator Lead
/* Purpose : This function defines a holdoff time for the AF generator.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultAfgl (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViReal64       Timems,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Set the AF reference level:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:AFGL", AFChannel);
    RETMSG(RSCMU_setViReal64(AudioNSigSession, wrtBuf,
                             Timems/1000, 0.0, 1.0,
                             RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: ConfMultPathCoupling                                                
/* Purpose : This function determines the AF path coupling for multitone
/*           measurements.
/*           Note(s):
/*           (1) This function is available only with option R&S CBT-B41.
/* Firmware: V4.25
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultContCoup (ViSession instrumentHandle,
                                                     ViInt32 channel,
                                                     ViInt32 AFPathCoupling,
                                                     ViChar errorMessage[])
{
  ViChar    buffer[RD_BUFFER_SIZE] = "";                
  ViString    StrAFPathCoupling[] = {"AC","DC",VI_NULL};

  // Check if the function is correctly used:       
  CHECK_VI_INT32_RANGE(instrumentHandle, channel, RSCMU_AF1, RSCMU_AF2, RSCMU_ERROR_PARAMETER2, errorMessage);
   
  sprintf (buffer, "CONF:MULT:AF%ldC:COUP", channel);

  RETMSG (RSCMU_setEnum (instrumentHandle,
                             buffer,
                             AFPathCoupling,
                             StrAFPathCoupling,
                             RSCMU_ERROR_PARAMETER3));      
}

/*===========================================================================*/
/* Function: Test cycles
/* Purpose : This function determines the number of statistics cycles. A stop
/*           condition is not available.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultContRep (ViSession      AudioNSigSession,
                                                    ViInt32        AFChannel,
                                                    ViInt32        Repetition,
                                                    ViInt32        StopCondition,
                                                    ViInt32        StepMode,
                                                    ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, Repetition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, StepMode,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Configure tests cycles:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:CONT:REP %s,%s,%s", AFChannel, StrRepetition[Repetition], StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: ConfMultContRep                                                
/* Purpose : This function defines the repetition mode, the stop condition
/*           and the stepping mode for the spectrum measurement.
/*           Note(s):
/*           (1) This function is available only with option R&S CBT-B41.
/* Firmware: V4.25
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultContRepExt (ViSession instrumentHandle,
                                                       ViInt32 channel,
                                                       ViInt32 repetition,
                                                       ViInt32 multipleMeasNbRepetition,
                                                       ViInt32 stopCondition,
                                                       ViInt32 stepMode,
                                                       ViChar errorMessage[])
{
  ViChar    wrtBuf[WR_BUFFER_SIZE] = "";
      
  // Check if the function is correctly used:
  CHECK_VI_INT32_RANGE(instrumentHandle, channel, RSCMU_AF1, RSCMU_AF2, RSCMU_ERROR_PARAMETER2, errorMessage);
  CHECK_VI_INT32_RANGE(instrumentHandle, repetition, RSCMU_CONTINUOUS, RSCMU_MULTIPLE, RSCMU_ERROR_PARAMETER3, errorMessage);
  CHECK_VI_INT32_RANGE(instrumentHandle, multipleMeasNbRepetition, 1, 1000, RSCMU_ERROR_PARAMETER4, errorMessage);
  CHECK_VI_INT32_RANGE(instrumentHandle, stopCondition, RSCMU_NONE, RSCMU_STOP_ON, RSCMU_ERROR_PARAMETER5, errorMessage);
  CHECK_VI_INT32_RANGE(instrumentHandle, stepMode, RSCMU_NONE, RSCMU_STEP, RSCMU_ERROR_PARAMETER6, errorMessage);
  
  if (repetition == RSCMU_MULTIPLE)        
    sprintf (wrtBuf, "CONF:MULT:AF%dC:CONT:REP %ld,%s,%s", channel, multipleMeasNbRepetition, StrStopCondition[stopCondition], StrStepMode[stepMode]);
  else                
    sprintf (wrtBuf, "CONF:MULT:AF%dC:CONT:REP %s,%s,%s", channel, StrRepetition[repetition], StrStopCondition[stopCondition], StrStepMode[stepMode]);
  
  RETMSG (RSCMU_sendCmd (instrumentHandle, wrtBuf));
}

/* ----------------------------------------------------------------------------
 * Subsystem MULTitone:TDEFinition
 *
 * The subsystem MULTitone:...TDEFinition configures the audio test signal used
 * for the Audio Multitone measurement. The subsystem corresponds to the Tone
 * Def. tab of the popup menu Audio Multitone Configuration.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Configuration of the test tones frequencies
/* Purpose : This function configures the 20 test tones frequencies for the
/*           function RSCMU_Audio_NSig_ConfMultTdef(). The minimum frequency
/*           spacing between two tones is 1 Hz.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdefFreq (ViSession AudioNSigSession,
                                                     ViInt32   freq1_Hz,
                                                     ViInt32   freq2_Hz,
                                                     ViInt32   freq3_Hz,
                                                     ViInt32   freq4_Hz,
                                                     ViInt32   freq5_Hz,
                                                     ViInt32   freq6_Hz,
                                                     ViInt32   freq7_Hz,
                                                     ViInt32   freq8_Hz,
                                                     ViInt32   freq9_Hz,
                                                     ViInt32   freq10_Hz,
                                                     ViInt32   freq11_Hz,
                                                     ViInt32   freq12_Hz,
                                                     ViInt32   freq13_Hz,
                                                     ViInt32   freq14_Hz,
                                                     ViInt32   freq15_Hz,
                                                     ViInt32   freq16_Hz,
                                                     ViInt32   freq17_Hz,
                                                     ViInt32   freq18_Hz,
                                                     ViInt32   freq19_Hz,
                                                     ViInt32   freq20_Hz,
                                                     ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq1_Hz, 10, 15999, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq2_Hz, 10, 15999, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq3_Hz, 10, 15999, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq4_Hz, 10, 15999, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq5_Hz, 10, 15999, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq6_Hz, 10, 15999, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq7_Hz, 10, 15999, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq8_Hz, 10, 15999, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq9_Hz, 10, 15999, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq10_Hz, 10, 15999, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq11_Hz, 10, 15999, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq12_Hz, 10, 15999, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq13_Hz, 10, 15999, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq14_Hz, 10, 15999, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq15_Hz, 10, 15999, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq16_Hz, 10, 15999, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq17_Hz, 10, 15999, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq18_Hz, 10, 15999, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq19_Hz, 10, 15999, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, freq20_Hz, 10, 15999, RSCMU_ERROR_PARAMETER21, errorMessage);

    //Fill the test tones structure with the frequencies
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[0] = freq1_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[1] = freq2_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[2] = freq3_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[3] = freq4_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[4] = freq5_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[5] = freq6_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[6] = freq7_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[7] = freq8_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[8] = freq9_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[9] = freq10_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[10] = freq11_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[11] = freq12_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[12] = freq13_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[13] = freq14_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[14] = freq15_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[15] = freq16_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[16] = freq17_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[17] = freq18_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[18] = freq19_Hz;
    p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[19] = freq20_Hz;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Configuration of the test tones levels
/* Purpose : This function configures the 20 test tones levels for the function
/*           RSCMU_Audio_NSig_ConfMultTdef(). The sum of all test tones must
/*           not exceed the maximum AF generator level quoted in the data sheet
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdefLev (ViSession AudioNSigSession,
                                                    ViInt32   lev1_mV,
                                                    ViInt32   lev2_mV,
                                                    ViInt32   lev3_mV,
                                                    ViInt32   lev4_mV,
                                                    ViInt32   lev5_mV,
                                                    ViInt32   lev6_mV,
                                                    ViInt32   lev7_mV,
                                                    ViInt32   lev8_mV,
                                                    ViInt32   lev9_mV,
                                                    ViInt32   lev10_mV,
                                                    ViInt32   lev11_mV,
                                                    ViInt32   lev12_mV,
                                                    ViInt32   lev13_mV,
                                                    ViInt32   lev14_mV,
                                                    ViInt32   lev15_mV,
                                                    ViInt32   lev16_mV,
                                                    ViInt32   lev17_mV,
                                                    ViInt32   lev18_mV,
                                                    ViInt32   lev19_mV,
                                                    ViInt32   lev20_mV,
                                                    ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev1_mV, 0, 5000, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev2_mV, 0, 5000, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev3_mV, 0, 5000, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev4_mV, 0, 5000, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev5_mV, 0, 5000, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev6_mV, 0, 5000, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev7_mV, 0, 5000, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev8_mV, 0, 5000, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev9_mV, 0, 5000, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev10_mV, 0, 5000, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev11_mV, 0, 5000, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev12_mV, 0, 5000, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev13_mV, 0, 5000, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev14_mV, 0, 5000, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev15_mV, 0, 5000, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev16_mV, 0, 5000, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev17_mV, 0, 5000, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev18_mV, 0, 5000, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev19_mV, 0, 5000, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, lev20_mV, 0, 5000, RSCMU_ERROR_PARAMETER21, errorMessage);

    //Fill the test tones structure with the levels
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[0] = lev1_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[1] = lev2_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[2] = lev3_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[3] = lev4_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[4] = lev5_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[5] = lev6_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[6] = lev7_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[7] = lev8_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[8] = lev9_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[9] = lev10_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[10] = lev11_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[11] = lev12_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[12] = lev13_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[13] = lev14_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[14] = lev15_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[15] = lev16_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[16] = lev17_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[17] = lev18_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[18] = lev19_mV;
    p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[19] = lev20_mV;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Configuration of the test tones switches
/* Purpose : This function configures the 20 test tones switches for the function
/*           RSCMU_Audio_NSig_ConfMultTdef().
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdefSwitch (ViSession AudioNSigSession,
                                                       ViInt32   enable1,
                                                       ViInt32   enable2,
                                                       ViInt32   enable3,
                                                       ViInt32   enable4,
                                                       ViInt32   enable5,
                                                       ViInt32   enable6,
                                                       ViInt32   enable7,
                                                       ViInt32   enable8,
                                                       ViInt32   enable9,
                                                       ViInt32   enable10,
                                                       ViInt32   enable11,
                                                       ViInt32   enable12,
                                                       ViInt32   enable13,
                                                       ViInt32   enable14,
                                                       ViInt32   enable15,
                                                       ViInt32   enable16,
                                                       ViInt32   enable17,
                                                       ViInt32   enable18,
                                                       ViInt32   enable19,
                                                       ViInt32   enable20,
                                                       ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable1,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable2,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable3,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable4,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable5,0,1, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable6,0,1, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable7,0,1, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable8,0,1, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable9,0,1, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable10,0,1, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable11,0,1, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable12,0,1, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable13,0,1, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable14,0,1, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable15,0,1, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable16,0,1, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable17,0,1, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable18,0,1, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable19,0,1, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable20,0,1, RSCMU_ERROR_PARAMETER21, errorMessage);

    //Fill the test tones structure with the switches
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[0] = enable1;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[1] = enable2;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[2] = enable3;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[3] = enable4;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[4] = enable5;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[5] = enable6;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[6] = enable7;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[7] = enable8;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[8] = enable9;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[9] = enable10;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[10] = enable11;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[11] = enable12;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[12] = enable13;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[13] = enable14;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[14] = enable15;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[15] = enable16;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[16] = enable17;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[17] = enable18;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[18] = enable19;
    p_globalvarAudioNSig->RSCMU_tTestTones.enable[19] = enable20;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Test tones definition
/* Purpose : This function enables and configure up to 20 test tones. Before
/*           launching it, frequencies, levels and switches have to be
/*           configured by using these 3 functions:
/*
/*           - Audio_NSig_ConfMultTdefFreq
/*           - Audio_NSig_ConfMultTdefLev
/*           - Audio_NSig_ConfMultTdefSwitch
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdef (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     p2buf;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;
    ViInt32     i;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Configures the test tones with the structure:
    p2buf = wrtBuf;
    p2buf += sprintf (wrtBuf, "CONF:MULT:AF%ldC:TDEF ", AFChannel);

    for(i = 0; i < 20; i++)
    {
        p2buf += sprintf(p2buf,"%ld,%ld MV,%s,", p_globalvarAudioNSig->RSCMU_tTestTones.frequency_Hz[i]
                                             , p_globalvarAudioNSig->RSCMU_tTestTones.level_mV[i]
                                             , mode_On_Off[p_globalvarAudioNSig->RSCMU_tTestTones.enable[i]]);
    }

    /* remove trailing comma */
    *--p2buf = '\0';

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Single test tone definition
/* Purpose : This function enables and configures one of up to 20 test tones
/*           (<nr> = 1 to 20).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdefTone (ViSession      AudioNSigSession,
                                                     ViInt32        AFChannel,
                                                     ViInt32        ToneNbr,
                                                     ViInt32        frequency_Hz,
                                                     ViInt32        level_mV,
                                                     ViInt32        enable,
                                                     ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNbr, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, frequency_Hz, 10, 15999, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, level_mV, 0, 5000, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable,0,1, RSCMU_ERROR_PARAMETER6, errorMessage);

    // Configures the test tone:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:TDEF:TONE%ld %ld,%ld MV,%s", AFChannel
                                                              , ToneNbr
                                                              , frequency_Hz
                                                              , level_mV
                                                              , mode_On_Off[enable]);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Level selection
/* Purpose : This function defines how the voltage of each of the test tones
/*           is determined.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdefMode (ViSession      AudioNSigSession,
                                                     ViInt32        AFChannel,
                                                     ViInt32        mode,
                                                     ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    list[] = {"SEP","TLEV", VI_NULL};
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, mode,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Sets the level mode
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:TDEF:MODE", AFChannel);

    RETMSG(RSCMU_setEnum(AudioNSigSession, wrtBuf,
                         mode, list,
                         RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Level selection
/* Purpose : This function defines how the voltage of each of the test tones
/*           is determined.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultTdefTlev (ViSession      AudioNSigSession,
                                                     ViInt32        AFChannel,
                                                     ViInt32        level_mV,
                                                     ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Sets the total level
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:TDEF:TLEV", AFChannel);
    RETMSG(RSCMU_setViReal64(AudioNSigSession, wrtBuf,
                             level_mV/1000.0, 0.0, 5.0,
                             RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Default settings
/* Purpose : This function enables default setting for all parameters of the
/*           subsystem for AF1 or AF2 or both.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_DefMultTdef (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViInt32        enable,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    if(AFChannel == RSCMU_ALL_AF)
        strcpy(wrtBuf, "DEF:MULT:TDEF");
    else
        sprintf(wrtBuf, "DEF:MULT:AF%ldC:TDEF", AFChannel);

    RETMSG(RSCMU_setEnum(AudioNSigSession, wrtBuf,
                         enable, mode_On_Off,
                         RSCMU_ERROR_PARAMETER3));
}

/* ----------------------------------------------------------------------------
 * Subsystem MULTitone:AF1Channel:FILTer
 *
 * The subsystem MULTitone:AF1Channel:FILTer configures the voiceprocessing
 * equipment used for the Audio Multitone measurement. The subsystem corresponds
 * to the Filters tab of the popup menu Audio Multitone Configuration.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Bandpass, DC coupling
/* Purpose : This function selects the bandpass filter to be used if the AF
/*           path coupling is set to DC.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultFiltBpasDcc (ViSession      AudioNSigSession,
                                                        ViInt32        AFChannel,
                                                        ViInt32        BandPass,
                                                        ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, BandPass, 1, 19, RSCMU_ERROR_PARAMETER3, errorMessage);
    if((BandPass == 1) || (BandPass == 4) || (BandPass == 8) || (BandPass == 12) || (BandPass == 16))
        return RSCMU_Base_errorMessage(AudioNSigSession, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Select the band pass:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:FILT:BPAS:DCC BP%02ld", AFChannel
                                                           , BandPass);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Bandpass, DC coupling
/* Purpose : This function selects the bandpass filter to be used if the AF
/*           path coupling is set to DC.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultFiltBpasAcc (ViSession      AudioNSigSession,
                                                        ViInt32        AFChannel,
                                                        ViInt32        BandPass,
                                                        ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, BandPass, 1, 19, RSCMU_ERROR_PARAMETER3, errorMessage);
//    if((BandPass == 1) || (BandPass == 4) || (BandPass == 8) || (BandPass == 12) || (BandPass == 16))
//        return RSCMU_Base_errorMessage(AudioNSigSession, RSCMU_ERROR_NO_DC_ALLOWED, errorMessage);

    // Select the band pass:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:FILT:BPAS:ACC BP%02ld", AFChannel
                                                           , BandPass);
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Weighing Filter
/* Purpose : This function selects the weighting filter after the fixed band
/*           pass.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultFiltWeight (ViSession      AudioNSigSession,
                                                       ViInt32        AFChannel,
                                                       ViInt32        Filter,
                                                       ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    ViString    list[] =  {"OFF","CME","CCI","A",VI_NULL};
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 3, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Select the weighting filter:
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:FILT:WEIG", AFChannel);

    RETMSG(RSCMU_setEnum(AudioNSigSession,
                         wrtBuf, Filter, list,
                         RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Default settings
/* Purpose : This function enables default setting for all parameters of the
/*           subsystem for AF1 or AF2 or both.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_DefMultFilt (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViInt32        enable,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Set default settings
    if(AFChannel == RSCMU_ALL_AF)
        sprintf(wrtBuf, "DEF:MULT:FILT %s", mode_On_Off[enable]);
    else
        sprintf(wrtBuf, "DEF:MULT:AF%ldC:FILT %s", AFChannel
                                                , mode_On_Off[enable]);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/* ----------------------------------------------------------------------------
 * Subsystem MULTitone:...LIMit
 *
 * The subsystem MULTitone:...LIMit defines tolerance values for the Audio
 * Multitone measurement. The subsystem corresponds to the Limits tab of the
 * popup menu Audio Multitone Configuration.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Upper limit, all definitions
/* Purpose : This function configures the upper limit lines for the function
/*           Audio_NSig_ConfMultLimUpp().
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultLimUppDef (ViSession AudioNSigSession,
                                                      ViReal64  limit1,
                                                      ViReal64  limit2,
                                                      ViReal64  limit3,
                                                      ViReal64  limit4,
                                                      ViReal64  limit5,
                                                      ViReal64  limit6,
                                                      ViReal64  limit7,
                                                      ViReal64  limit8,
                                                      ViReal64  limit9,
                                                      ViReal64  limit10,
                                                      ViReal64  limit11,
                                                      ViReal64  limit12,
                                                      ViReal64  limit13,
                                                      ViReal64  limit14,
                                                      ViReal64  limit15,
                                                      ViReal64  limit16,
                                                      ViReal64  limit17,
                                                      ViReal64  limit18,
                                                      ViReal64  limit19,
                                                      ViReal64  limit20,
                                                      ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit1, -80, 80, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit2, -80, 80, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit3, -80, 80, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit4, -80, 80, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit5, -80, 80, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit6, -80, 80, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit7, -80, 80, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit8, -80, 80, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit9, -80, 80, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit10, -80, 80, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit11, -80, 80, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit12, -80, 80, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit13, -80, 80, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit14, -80, 80, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit15, -80, 80, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit16, -80, 80, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit17, -80, 80, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit18, -80, 80, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit19, -80, 80, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit20, -80, 80, RSCMU_ERROR_PARAMETER21, errorMessage);

    // Fills the limit lines structure
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[0]  = limit1;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[1]  = limit2;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[2]  = limit3;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[3]  = limit4;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[4]  = limit5;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[5]  = limit6;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[6]  = limit7;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[7]  = limit8;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[8]  = limit9;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[9]  = limit10;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[10] = limit11;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[11] = limit12;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[12] = limit13;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[13] = limit14;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[14] = limit15;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[15] = limit16;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[16] = limit17;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[17] = limit18;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[18] = limit19;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[19] = limit20;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Upper limit, all switches
/* Purpose : This function configures the switches for the function
/*           Audio_NSig_ConfMultLimUpp().
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultLimUppSwitch (ViSession AudioNSigSession,
                                                         ViInt32   enable1,
                                                         ViInt32   enable2,
                                                         ViInt32   enable3,
                                                         ViInt32   enable4,
                                                         ViInt32   enable5,
                                                         ViInt32   enable6,
                                                         ViInt32   enable7,
                                                         ViInt32   enable8,
                                                         ViInt32   enable9,
                                                         ViInt32   enable10,
                                                         ViInt32   enable11,
                                                         ViInt32   enable12,
                                                         ViInt32   enable13,
                                                         ViInt32   enable14,
                                                         ViInt32   enable15,
                                                         ViInt32   enable16,
                                                         ViInt32   enable17,
                                                         ViInt32   enable18,
                                                         ViInt32   enable19,
                                                         ViInt32   enable20,
                                                         ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable1,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable2,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable3,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable4,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable5,0,1, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable6,0,1, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable7,0,1, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable8,0,1, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable9,0,1, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable10,0,1, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable11,0,1, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable12,0,1, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable13,0,1, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable14,0,1, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable15,0,1, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable16,0,1, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable17,0,1, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable18,0,1, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable19,0,1, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable20,0,1, RSCMU_ERROR_PARAMETER21, errorMessage);

    // Fills the limit lines structure
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[0]   = enable1;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[1]   = enable2;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[2]   = enable3;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[3]   = enable4;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[4]   = enable5;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[5]   = enable6;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[6]   = enable7;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[7]   = enable8;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[8]   = enable9;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[9]   = enable10;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[10]  = enable11;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[11]  = enable12;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[12]  = enable13;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[13]  = enable14;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[14]  = enable15;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[15]  = enable16;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[16]  = enable17;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[17]  = enable18;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[18]  = enable19;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[19]  = enable20;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Upper limit, Overall
/* Purpose : This function configures the upper limit lines and enables the
/*           limit check at the 20 test tones that can be defined via
/*           CONFigure:MULTitone:AF1Channel:TONE<nr>.
/*
/*           Before launching it, limits and switches have to be defined by
/*           these two functions:
/*
/*           - Audio_NSig_ConfMultLimUppDef
/*           - Audio_NSig_ConfMultLimUppSwitch
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultLimUpp (ViSession AudioNSigSession,
                                                   ViInt32   AFChannel,
                                                   ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     p2buf;
    ViInt32     i;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Sets the upper limit lines for all test tones
    p2buf = wrtBuf;
    p2buf += sprintf(wrtBuf, "CONF:MULT:AF%ldC:LIM:LINE:ASYM:UPP ", AFChannel);
    for(i=0;i<20;i++)
        p2buf += sprintf(p2buf, "%.12f,%s,",p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_limits_dB[i],
                         mode_On_Off[p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Upper_enable[i]]);

    *--p2buf = '\0';
    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Upper limit, Single Point
/* Purpose : This function configures the upper limit and enables the limit
/*           check at one of 20 test tones that can be defined via
/*           CONFigure:MULTitone:AF1Channel:TONE<nr>. The test tones are
/*           numbered by <nr> = 1 to 20.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultToneLimUpp (ViSession      AudioNSigSession,
                                                       ViInt32        AFChannel,
                                                       ViInt32        ToneNumber,
                                                       ViReal64       limit_dB,
                                                       ViInt32        enable,
                                                       ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNumber, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit_dB, -80, 80, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Sets the upper limit lines for one test tone
    sprintf(wrtBuf, "CONF:MULT:AF%ldC:TONE%ld:LIM:LINE:ASYM:UPP %.12f,%s", AFChannel
                                                                     , ToneNumber
                                                                     , limit_dB
                                                                     , mode_On_Off[enable]);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Lower limit, all definitions
/* Purpose : This function configures the upper limit lines for the function
/*           Audio_NSig_ConfMultLimLow().
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultLimLowDef (ViSession AudioNSigSession,
                                                      ViReal64  limit1,
                                                      ViReal64  limit2,
                                                      ViReal64  limit3,
                                                      ViReal64  limit4,
                                                      ViReal64  limit5,
                                                      ViReal64  limit6,
                                                      ViReal64  limit7,
                                                      ViReal64  limit8,
                                                      ViReal64  limit9,
                                                      ViReal64  limit10,
                                                      ViReal64  limit11,
                                                      ViReal64  limit12,
                                                      ViReal64  limit13,
                                                      ViReal64  limit14,
                                                      ViReal64  limit15,
                                                      ViReal64  limit16,
                                                      ViReal64  limit17,
                                                      ViReal64  limit18,
                                                      ViReal64  limit19,
                                                      ViReal64  limit20,
                                                      ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit1, -80, 80, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit2, -80, 80, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit3, -80, 80, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit4, -80, 80, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit5, -80, 80, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit6, -80, 80, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit7, -80, 80, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit8, -80, 80, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit9, -80, 80, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit10, -80, 80, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit11, -80, 80, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit12, -80, 80, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit13, -80, 80, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit14, -80, 80, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit15, -80, 80, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit16, -80, 80, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit17, -80, 80, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit18, -80, 80, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit19, -80, 80, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit20, -80, 80, RSCMU_ERROR_PARAMETER21, errorMessage);

    // Fills the limit lines structure
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[0]  = limit1;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[1]  = limit2;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[2]  = limit3;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[3]  = limit4;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[4]  = limit5;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[5]  = limit6;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[6]  = limit7;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[7]  = limit8;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[8]  = limit9;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[9]  = limit10;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[10] = limit11;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[11] = limit12;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[12] = limit13;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[13] = limit14;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[14] = limit15;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[15] = limit16;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[16] = limit17;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[17] = limit18;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[18] = limit19;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[19] = limit20;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Lower limit, all switches
/* Purpose : This function configures the switches for the function
/*           Audio_NSig_ConfMultLimLow().
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultLimLowSwitch (ViSession AudioNSigSession,
                                                         ViInt32   enable1,
                                                         ViInt32   enable2,
                                                         ViInt32   enable3,
                                                         ViInt32   enable4,
                                                         ViInt32   enable5,
                                                         ViInt32   enable6,
                                                         ViInt32   enable7,
                                                         ViInt32   enable8,
                                                         ViInt32   enable9,
                                                         ViInt32   enable10,
                                                         ViInt32   enable11,
                                                         ViInt32   enable12,
                                                         ViInt32   enable13,
                                                         ViInt32   enable14,
                                                         ViInt32   enable15,
                                                         ViInt32   enable16,
                                                         ViInt32   enable17,
                                                         ViInt32   enable18,
                                                         ViInt32   enable19,
                                                         ViInt32   enable20,
                                                         ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRET(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable1,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable2,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable3,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable4,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable5,0,1, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable6,0,1, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable7,0,1, RSCMU_ERROR_PARAMETER8, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable8,0,1, RSCMU_ERROR_PARAMETER9, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable9,0,1, RSCMU_ERROR_PARAMETER10, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable10,0,1, RSCMU_ERROR_PARAMETER11, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable11,0,1, RSCMU_ERROR_PARAMETER12, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable12,0,1, RSCMU_ERROR_PARAMETER13, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable13,0,1, RSCMU_ERROR_PARAMETER14, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable14,0,1, RSCMU_ERROR_PARAMETER15, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable15,0,1, RSCMU_ERROR_PARAMETER16, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable16,0,1, RSCMU_ERROR_PARAMETER17, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable17,0,1, RSCMU_ERROR_PARAMETER18, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable18,0,1, RSCMU_ERROR_PARAMETER19, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable19,0,1, RSCMU_ERROR_PARAMETER20, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable20,0,1, RSCMU_ERROR_PARAMETER21, errorMessage);

    // Fills the limit lines structure
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[0]   = enable1;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[1]   = enable2;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[2]   = enable3;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[3]   = enable4;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[4]   = enable5;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[5]   = enable6;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[6]   = enable7;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[7]   = enable8;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[8]   = enable9;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[9]   = enable10;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[10]  = enable11;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[11]  = enable12;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[12]  = enable13;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[13]  = enable14;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[14]  = enable15;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[15]  = enable16;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[16]  = enable17;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[17]  = enable18;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[18]  = enable19;
    p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[19]  = enable20;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Lower limit, Overall
/* Purpose : This function configures the upper limit lines and enables the
/*           limit check at the 20 test tones that can be defined via
/*           CONFigure:MULTitone:AF1Channel:TONE<nr>.
/*
/*           Before launching it, limits and switches have to be defined by
/*           these two functions:
/*
/*           - Audio_NSig_ConfMultLimLowDef
/*           - Audio_NSig_ConfMultLimLowSwitch
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultLimLow (ViSession AudioNSigSession,
                                                   ViInt32   AFChannel,
                                                   ViChar    _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     p2buf;
    ViInt32     i;
    struct      RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Queries the specified attribute for the Audio Non-Signalling object:
    CHECKRETMSG(viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Sets the upper limit lines for all test tones
    p2buf = wrtBuf;
    p2buf += sprintf(wrtBuf, "CONF:MULT:AF%ldC:LIM:LINE:ASYM:LOW ", AFChannel);

    for(i=0;i<20;i++)
        p2buf += sprintf(p2buf, "%.12f,%s,", p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_limits_dB[i]
                          , mode_On_Off[p_globalvarAudioNSig->RSCMU_tMultitoneLimits.Lower_enable[i]]);

    *--p2buf = '\0';

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Lower limit, Single Point
/* Purpose : This function configures the lower limit and enables the limit
/*           check at one of 20 test tones that can be defined via
/*           CONFigure:MULTitone:AF1Channel:TONE<nr>. The test tones are
/*           numbered by <nr> = 1 to 20.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultToneLimLow (ViSession      AudioNSigSession,
                                                       ViInt32        AFChannel,
                                                       ViInt32        ToneNumber,
                                                       ViReal64       limit_dB,
                                                       ViInt32        enable,
                                                       ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNumber, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_REAL64_RANGE(AudioNSigSession, limit_dB, -80, 80, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Sets the upper limit lines for one test tone
    sprintf(wrtBuf, "CONF:MULT:AF%ldC:TONE%ld:LIM:LINE:ASYM:LOW %.12f,%s", AFChannel
                                                                     , ToneNumber
                                                                     , limit_dB
                                                                     , mode_On_Off[enable]);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Default settings
/* Purpose : This function enables default setting for all parameters of the
/*           subsystem for AF1 or AF2 or both.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_DefMultLim (ViSession      AudioNSigSession,
                                               ViInt32        AFChannel,
                                               ViInt32        enable,
                                               ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, enable,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Set default settings
    if(AFChannel == RSCMU_ALL_AF)
        strcpy(wrtBuf, "DEF:MULT:LIM:LINE");
    else
        sprintf(wrtBuf, "DEF:MULT:AF%ldC:LIM:LINE", AFChannel);

    RETMSG(RSCMU_setEnum(AudioNSigSession, wrtBuf,
                         enable, mode_On_Off,
                         RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: Set Statistics Cycles
/* Purpose : This function sets the number of statistics cycles,the stop
/*           condition and the stepping mode for the Multitone Analysis.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultContRepStat (ViSession      AudioNSigSession,
                                                        ViInt32        AFChannel,
                                                        ViInt32        NbRepetition,
                                                        ViInt32        StopCondition,
                                                        ViInt32        StepMode,
                                                        ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, NbRepetition, RSCMU_MULT_MEAS_MIN, RSCMU_MULT_MEAS_MAX, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, StepMode,0,1, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Set statistics cycles
    sprintf (wrtBuf, "CONF:MULT:AF%ldC:CONT:REP %ld,%s,%s", AFChannel, NbRepetition, StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Configuration subarrays
/* Purpose : This command configures the READ:SUBarrays:MULTitone:AF1Channel..,
/*           FETCh:SUBarrays:MULTitone:AF1Channel.., and SAMPle:SUBarrays:
/*           MULTitone:AF1Channel commands. It restricts the measurement to up
/*           to 32 subranges where either all measurement results (the number
/*           of which is given by the <Samples> parameter) or a single
/*           statistical value is returned.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfSubMultRang (ViSession      AudioNSigSession,
                                                    ViInt32        AFChannel,
                                                    ViInt32        statisticsMode,
                                                    ViInt32        start_tone,
                                                    ViInt32        number_of_tones,
                                                    ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViString    StrStatMode[] = {"ALL","ARIT","MIN","MAX","IVAL"};
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, statisticsMode, 0, 4, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, start_tone, 1, 20, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, number_of_tones, 1, 20, RSCMU_ERROR_PARAMETER5, errorMessage);

    // Configures subarrays
    sprintf (wrtBuf, "CONF:SUB:MULT:AF%ldC %s,%ld,%ld", AFChannel
                                                   , StrStatMode[statisticsMode]
                                                   , start_tone
                                                   , number_of_tones);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*
 *-----------------------------------------------------------------------------
 *      MEASUREMENT CONTROL
 *
 *      This subsystem contains the functions for general control of the
 *      multitone measurement.
 *-----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Start new measurement
/* Purpose : This function starts the multitone measurement, setting it to
/*           the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_InitMult (ViSession      AudioNSigSession,
                                             ViInt32        AFChannel,
                                             ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Initiate measurement
    sprintf (wrtBuf, "INIT:MULT:AF%ldC", AFChannel);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Abort running measurement and switch off
/* Purpose : This function stops the multitone measurement, setting it to
/*           the status "OFF"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_AborMult (ViSession      AudioNSigSession,
                                             ViInt32        AFChannel,
                                             ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Initiate measurement
    sprintf (wrtBuf, "ABOR:MULT:AF%ldC", AFChannel);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: stop measurement after current statistic cycle
/* Purpose : This function stops the multitone measurement, setting it to
/*           the status "STOP"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_StopMult (ViSession      AudioNSigSession,
                                             ViInt32        AFChannel,
                                             ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession, RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Initiate measurement
    sprintf (wrtBuf, "STOP:MULT:AF%ldC", AFChannel);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Next Measurement Step
/* Purpose : This function continue the multitone measurement, setting it to
/*           the status "RUN"
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ContMult (ViSession      AudioNSigSession,
                                             ViInt32        AFChannel,
                                             ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00,RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Initiate measurement
    sprintf (wrtBuf, "CONT:MULT:AF%ldC", AFChannel);

    RETMSG(RSCMU_sendCmd(AudioNSigSession, wrtBuf));
}

/*===========================================================================*/
/* Function: AF analyzer status
/* Purpose : This function returns status of the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcMultStat (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32       *Status,
                                                 ViInt32       *NbCount,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    GET_BUFS( AudioNSigSession );

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00,RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Fetch the current AF generator status:
    sprintf (wrtBuf, "FETC:MULT:AF%ldC:STAT?", AFChannel);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf,
                               RD_BUFFER_SIZE, 0));

    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(AudioNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(AudioNSigSession, str, measurementStatusArr, Status));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(AudioNSigSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",NbCount))
        *NbCount = RSCMU_NONE;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the measurement
/*           is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfMultErep (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32        Mode,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "CONF:MULT:AF%ldC:EREP", AFChannel);

    RETMSG(RSCMU_setEnum(AudioNSigSession, wrtBuf, Mode, modeArr,
                         RSCMU_ERROR_PARAMETER3));
}

/*
 *-----------------------------------------------------------------------------
 * Subsystem MULTitone Results
 *
 * The subsystem MULTitone:...? measures the AF input level, returns the results
 * and compares them with the tolerance values. The subsystem corresponds to the
 * various output elements in the measurement menu Audio Multitone.
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Read scalar measured values
/* Purpose : This function  starts a measurement and output the audio level
/*           at test tone <nr>
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadMultTone (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32        ToneNumber,
                                                 ViReal64      *AF_level,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNumber, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(wrtBuf, "READ:MULT:AF%ldC:TONE%ld?", AFChannel, ToneNumber);

    RETMSG(RSCMU_getViReal64(AudioNSigSession, wrtBuf, AF_level,
                             RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Fetch scalar measured values
/* Purpose : This function outputs the audio level at test tone <nr>
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcMultTone (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32        ToneNumber,
                                                 ViReal64      *AF_level,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNumber, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(wrtBuf, "FETC:MULT:AF%ldC:TONE%ld?", AFChannel, ToneNumber);

    RETMSG(RSCMU_getViReal64(AudioNSigSession, wrtBuf, AF_level,
                             RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Sample scalar measured values
/* Purpose : This function reads out and outputs the audio level at test tone <nr>
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SampMultTone (ViSession      AudioNSigSession,
                                                 ViInt32        AFChannel,
                                                 ViInt32        ToneNumber,
                                                 ViReal64      *AF_level,
                                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF(AudioNSigSession);

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNumber, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(wrtBuf, "SAMP:MULT:AF%ldC:TONE%ld?", AFChannel, ToneNumber);

    RETMSG(RSCMU_getViReal64(AudioNSigSession, wrtBuf, AF_level,
                             RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Check limit
/* Purpose : This function indicates whether and in which way the error limits
/*           at tone <nr> (<nr> = 1 to 20) have been exceeded.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_CalcMultToneMatchLim (ViSession      AudioNSigSession,
                                                         ViInt32        AFChannel,
                                                         ViInt32        ToneNumber,
                                                         ViInt32       *Matching,
                                                         ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar     *str;
    GET_BUFS( AudioNSigSession );

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(AudioNSigSession, ToneNumber, 1, 20, RSCMU_ERROR_PARAMETER3, errorMessage);

    // Ask for limit check
    sprintf(wrtBuf, "CALC:MULT:AF%ldC:TONE%ld:MATC:LIM?", AFChannel, ToneNumber);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf, RD_BUFFER_SIZE,
                               0));

    // Extract the CMU response
//    rdBuf[strlen(rdBuf) - 1] = '\0';
    str = strtok (rdBuf, ",");
    return RSCMU_getIndexFromString(AudioNSigSession, str, matchingArr, Matching);
}

/*===========================================================================*/
/* Function: Check limit, Overall
/* Purpose : This function indicates whether and in which way the error limits
/*           at the 20 tone have been exceeded.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_CalcMultMatchLim (ViSession      AudioNSigSession,
                                                     ViInt32        AFChannel,
                                                     ViInt32        Matching[],
                                                     ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     str;
    ViInt32     i;
    GET_BUFS( AudioNSigSession );

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    // Ask for limit check
    sprintf(wrtBuf, "CALC:MULT:AF%ldC:MATC:LIM?", AFChannel);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf, RD_BUFFER_SIZE,0));
//    rdBuf[strlen(rdBuf) - 1] = '\0';
    str = strtok(rdBuf,",");

    for(i=0; i<20; i++)
    {
        CHECKRET(RSCMU_getIndexFromString(AudioNSigSession, str, matchingArr, &(Matching[i])));
        str = strtok(NULL, ",");
    }

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Read Array
/* Purpose : This function starts and returns the audio level at the 20 test tones
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadArrMult (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViReal64       AF_levels[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_WRT_BUF( AudioNSigSession );

    CHKNULLP(AF_levels, RSCMU_ERROR_PARAMETER2);

    sprintf (wrtBuf, "READ:ARR:MULT:AF%ldC?", AFChannel);

    CHECKERR (RSCMU_viWrite(AudioNSigSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(AudioNSigSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,AF_levels);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Fetch Array
/* Purpose : This function returns the audio level at the 20 test tones
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcArrMult (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViReal64       AF_levels[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_WRT_BUF( AudioNSigSession );

    CHKNULLP(AF_levels, RSCMU_ERROR_PARAMETER2);

    sprintf (wrtBuf, "FETC:ARR:MULT:AF%ldC?", AFChannel);

    CHECKERR (RSCMU_viWrite(AudioNSigSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(AudioNSigSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,AF_levels);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Sample Array
/* Purpose : This function returns the audio level at the 20 test tones
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SampArrMult (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViReal64       AF_levels[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_WRT_BUF( AudioNSigSession );

    CHKNULLP(AF_levels, RSCMU_ERROR_PARAMETER2);

    sprintf (wrtBuf, "SAMP:ARR:MULT:AF%ldC?", AFChannel);

    CHECKERR (RSCMU_viWrite(AudioNSigSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(AudioNSigSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,AF_levels);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Limit matching, Overall
/* Purpose : Any set bit of the two returned fields indicates that the limits
/*           at the corresponding point are exceeded.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_CalcArrMultMatchLim (ViSession      AudioNSigSession,
                                                        ViInt32        AFChannel,
                                                        ViInt32       *upper_limits,
                                                        ViInt32       *lower_limits,
                                                        ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_BUFS( AudioNSigSession );

    CHECKRETMSG(RSCMU_checkVersion(AudioNSigSession,RSCMU200_V3_00, RSCMU_ERROR_FW_VERSION_3_00));

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(wrtBuf, "CALC:ARR:MULT:AF%ldC:MATC:LIM?", AFChannel);
    CHECKRETMSG(RSCMU_getViString(AudioNSigSession, wrtBuf, rdBuf, RD_BUFFER_SIZE,0));

    sscanf(rdBuf,"%ld,%ld", upper_limits, lower_limits);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Read SubArrays
/* Purpose : This function starts and returns the audio level in the subranges
/*           defined by means of the CONFigure:SUBarrays:MULTitone:AF1Channel
/*           command. In the default setting of the configuration command the
/*           READ:SUBarrays..., FETCh:SUBarrays..., and SAMPle:SUBarrays...
/*           command group is equivalent to the READ:ARRay...,FETCh:ARRay...,
/*           and SAMPle:ARRay... command group described above.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadSubMult (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViReal64       AF_levels[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_WRT_BUF( AudioNSigSession );

    CHKNULLP(AF_levels, RSCMU_ERROR_PARAMETER2);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "READ:SUB:MULT:AF%ldC?", AFChannel);

    CHECKERR (RSCMU_viWrite(AudioNSigSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(AudioNSigSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,AF_levels);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Fetch SubArrays
/* Purpose : This returns the audio level in the subranges defined by means of
/*           the CONFigure:SUBarrays:MULTitone:AF1Channel command. In the
/*           default setting of the configuration command the READ:SUBarrays...,
/*           FETCh:SUBarrays..., and SAMPle:SUBarrays... command group is
/*           equivalent to the READ:ARRay...,FETCh:ARRay..., and SAMPle:ARRay...
/*           command group described above.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcSubMult (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViReal64       AF_levels[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_WRT_BUF( AudioNSigSession );

    CHKNULLP(AF_levels, RSCMU_ERROR_PARAMETER2);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "FETC:SUB:MULT:AF%ldC?", AFChannel);

    CHECKERR (RSCMU_viWrite(AudioNSigSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(AudioNSigSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,AF_levels);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Sample SubArrays
/* Purpose : This function reads out and returns the audio level in the subranges
/*           defined by means of the CONFigure:SUBarrays:MULTitone:AF1Channel
/*           command. In the default setting of the configuration command the
/*           READ:SUBarrays..., FETCh:SUBarrays..., and SAMPle:SUBarrays...
/*           command group is equivalent to the READ:ARRay...,FETCh:ARRay...,
/*           and SAMPle:ARRay... command group described above.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_SampSubMult (ViSession      AudioNSigSession,
                                                ViInt32        AFChannel,
                                                ViReal64       AF_levels[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     hugeBuf = VI_NULL;
    GET_WRT_BUF( AudioNSigSession );

    CHKNULLP(AF_levels, RSCMU_ERROR_PARAMETER2);
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(AudioNSigSession, AFChannel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (wrtBuf, "SAMP:SUB:MULT:AF%ldC?", AFChannel);

    CHECKERR (RSCMU_viWrite(AudioNSigSession, wrtBuf, strlen(wrtBuf)));

    CHECKERR(RSCMU_ReadDataUnknownLength(AudioNSigSession, &hugeBuf, VI_NULL));
    if (RSCMU_Status < VI_SUCCESS)
        if (hugeBuf) free(hugeBuf);

    RSCMU_atofArray (hugeBuf,AF_levels);

    if (hugeBuf) free(hugeBuf);

    return RSCMU_Status;
}

ViStatus _VI_FUNC RSCMU_Audio_NSig_MMIVersion (ViSession      AudioNSigSession,
                                                ViChar        Version[],
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_WRT_BUF( AudioNSigSession );

    sprintf (wrtBuf, "SYST:VERS:SW:MMI?");

    CHECKERR (RSCMU_getViString(AudioNSigSession, wrtBuf,Version,RD_BUFFER_SIZE,VI_NULL));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: InitThd                                                
/* Purpose : This function starts a new measurement of the THD.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_InitThd (ViSession instrumentHandle,
                                      ViInt32 channel, ViChar errorMessage[])
{
    ViChar  buffer[RD_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "INIT:THD:AF%ldC", channel);

    RETMSG (RSCMU_sendCmd (instrumentHandle, buffer));          
}

/*===========================================================================*/
/* Function: AborThd                                                
/* Purpose : This function aborts running measurement and switches off.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_AborThd (ViSession instrumentHandle,
                                      ViInt32 channel, ViChar errorMessage[])
{
    ViChar  buffer[RD_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "ABOR:THD:AF%ldC", channel);

    RETMSG (RSCMU_sendCmd (instrumentHandle, buffer));          
}

/*===========================================================================*/
/* Function: StopThd                                                
/* Purpose : This function stops AF measurement after current evaluation period.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_StopThd (ViSession instrumentHandle,
                                      ViInt32 channel, ViChar errorMessage[])
{
    ViChar  buffer[RD_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "STOP:THD:AF%ldC", channel);

    RETMSG (RSCMU_sendCmd (instrumentHandle, buffer));          
}

/*===========================================================================*/
/* Function: ContThd                                                
/* Purpose : This function runs next measurement step (only stepping mode).
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ContThd (ViSession instrumentHandle,
                                      ViInt32 channel, ViChar errorMessage[])
{
    ViChar  buffer[RD_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "CONT:THD:AF%ldC", channel);

    RETMSG (RSCMU_sendCmd (instrumentHandle, buffer));          
}

/*===========================================================================*/
/* Function: FetcThdStat                                                
/* Purpose : This function returns the status of the measurement and the number 
/*           of the current statistics cycle.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcThdStat (ViSession instrumentHandle,
                                          ViInt32 channel,
                                          ViInt32 *measurementStatus,
                                          ViInt32 *counter_for_current_stat_cycle,
                                          ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE] = "";
    ViChar      TableOfDataValues[100][100];
    ViChar      buffer[RD_BUFFER_SIZE] = "";      

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "FETC:THD:AF%ldC:STAT?", channel);
                                                    
    // Fetch the status of the Power/t measurement:
    if ((RSCMU_Status = RSCMU_Query (instrumentHandle, buffer, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
        return RSCMU_Status;

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", TableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, TableOfDataValues, 0)) < 0)
        return RSCMU_Status;

    // Get the current value of the statistics counter (the number of the current statistics cycle):
    if ((RSCMU_Status = RSCMU_GetCountingNo (counter_for_current_stat_cycle, TableOfDataValues, 1)) < 0)
        return RSCMU_Status;

    return RSCMU_Status;    
}

/*===========================================================================*/
/* Function: ConfThdErep                                                
/* Purpose : This function defines the events generated when the measurement 
/*           is terminated or stopped.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfThdErep (ViSession instrumentHandle,
                                          ViInt32 channel,
                                          ViInt32 eventReporting,
                                          ViChar errorMessage[])
{
    ViChar      buffer[RD_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "CONF:THD:AF%ldC:EREP", channel);
  
    RETMSG (RSCMU_setEnum(instrumentHandle,
                          buffer,
                          eventReporting,
                          modeArr,
                          RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: ConfThdContRep                                                
/* Purpose : This function defines the repetition mode, the stop condition and 
/*           the stepping mode for the spectrum measurement.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ConfThdContRep (ViSession instrumentHandle,
                                             ViInt32 channel,
                                             ViInt32 repetitionMode,
                                             ViInt32 multipleMeasNbRepetition,
                                             ViInt32 stopCondition,
                                             ViInt32 stepMode,
                                             ViChar errorMessage[])
{
    ViChar    wrtBuf[WR_BUFFER_SIZE] = "";
  
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, repetitionMode, RSCMU_CONTINUOUS, RSCMU_MULTIPLE, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, multipleMeasNbRepetition, 1, 1000, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, stopCondition, RSCMU_NONE, RSCMU_STOP_ON, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, stepMode, RSCMU_NONE, RSCMU_STEP, RSCMU_ERROR_PARAMETER6, errorMessage);

    if (repetitionMode == RSCMU_MULTIPLE)          
        sprintf (wrtBuf, "CONF:THD:AF%ldC:CONT:REP %ld,%s,%s", channel, multipleMeasNbRepetition, StrStopCondition[stopCondition], StrStepMode[stepMode]);
    else                  
        sprintf (wrtBuf, "CONF:THD:AF%ldC:CONT:REP %s,%s,%s", channel, StrRepetition[repetitionMode], StrStopCondition[stopCondition], StrStepMode[stepMode]);

    RETMSG (RSCMU_sendCmd (instrumentHandle, wrtBuf));
}

/*===========================================================================*/
/* Function: ContThdHarmonics                                                
/* Purpose : This function enables the harmonics no. 2 to 9 for the THD 
/*           calculation. The audio levels are measured for all harmonics below 
/*           21 kHz. The first harmonic frequency is always enabled.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ContThdHarmonics (ViSession instrumentHandle,
                                               ViInt32 channel,
                                               ViInt32 harmonics,
                                               ViBoolean enable,
                                               ViChar errorMessage[])
{
    ViChar    wrtBuf[WR_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, harmonics, 2, 9, RSCMU_ERROR_PARAMETER3, errorMessage);
         
    sprintf (wrtBuf, "CONF:THD:AF%ldC:HARM%ld",channel, harmonics);
    
    RETMSG ( RSCMU_setEnum (instrumentHandle,
                            wrtBuf,
                            enable,
                            mode_On_Off,
                            RSCMU_ERROR_PARAMETER4));                  
}

/*===========================================================================*/
/* Function: ContThdTdefinition                                                
/* Purpose : This function configures the generator signal. The first harmonic 
/*           d1 is measured at the generator signal frequency.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ContThdTdefinition (ViSession instrumentHandle,
                                                 ViInt32 channel,
                                                 ViReal64 frequency,
                                                 ViReal64 level,
                                                 ViChar errorMessage[])
{
    ViChar    wrtBuf[WR_BUFFER_SIZE] = "";

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 500.0, 3000.0, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_REAL64_RANGE(instrumentHandle, level, 0.0, 5.0, RSCMU_ERROR_PARAMETER4, errorMessage);
         
    sprintf (wrtBuf, "CONF:THD:AF%ldC:TDEF %Lf,%Lf",channel, frequency, level);
    
    RETMSG ( RSCMU_sendCmd (instrumentHandle, wrtBuf));    
}

/*===========================================================================*/
/* Function: ReadThdHarmonics                                                
/* Purpose : This function starts a new measurement and returns the audio 
/*           levels at the first harmonic frequency d1 (<nr> = 1) and at the 
/*           harmonic frequencies <nr> = 2 to 9. All results are normalized to 
/*           level of the first harmonic, so the first result is always 0 dB.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadThdHarmonics (ViSession instrumentHandle,
                                               ViInt32 channel,
                                               ViInt32 harmonics,
                                               ViReal64 *harmonicsLevel,
                                               ViChar errorMessage[])
{
    ViChar  buffer[RD_BUFFER_SIZE] = "";
    
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, harmonics, 1, 9, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (buffer, "READ:SCAL:THD:AF%ldC:HARM%ld?", channel, harmonics);

    return RSCMU_getViReal64 (instrumentHandle,
                                  buffer,
                                  harmonicsLevel,
                                  0);
}

/*===========================================================================*/
/* Function: ReadThdHarmonics                                                
/* Purpose : This function reads measurement results (unsynchronized), the 
/*           audio levels at the first harmonic frequency d1 (<nr> = 1) and at 
/*           the harmonic frequencies <nr> = 2 to 9. All results are normalized 
/*           to level of the first harmonic, so the first result is always 0 dB.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcThdHarmonics (ViSession instrumentHandle,
                                               ViInt32 channel,
                                               ViInt32 harmonics,
                                               ViReal64 *harmonicsLevel,
                                               ViChar errorMessage[])
{
    ViChar  buffer[RD_BUFFER_SIZE] = "";
    
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, harmonics, 1, 9, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf (buffer, "FETC:SCAL:THD:AF%ldC:HARM%ld?", channel, harmonics);

    return RSCMU_getViReal64 (instrumentHandle,
                                  buffer,
                                  harmonicsLevel,
                                  0);
}

/*===========================================================================*/
/* Function: ReadThd                                               
/* Purpose : This function starts a new measurement and returns the Total 
/*           Harmonic Distortion (THD) as a percentage and as an equivalent 
/*           dB-value.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadThd (ViSession instrumentHandle,
                                      ViInt32 channel,ViReal64 *THDIn_dB,ViChar errorMessage[])
{
    ViChar      buffer[RD_BUFFER_SIZE] = "";
    
    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    
    sprintf (buffer, "READ:SCAL:THD:AF%ldC:THD?", channel);

    RETMSG (RSCMU_getViReal64 (instrumentHandle,
                               buffer,
                               THDIn_dB,
                               RSCMU_ERROR_PARAMETER3));
    
}

/*===========================================================================*/
/* Function: FetcThd                                               
/* Purpose : This function reads measurement results (unsynchronized), the Total 
/*           Harmonic Distortion (THD) as a percentage and as an equivalent 
/*           dB-value.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcThd (ViSession instrumentHandle,
                                      ViInt32 channel, ViReal64 *THDIn_dB, ViChar errorMessage[])
{
    ViChar      buffer[RD_BUFFER_SIZE] = "";
    
     // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    
    sprintf (buffer, "FETC:SCAL:THD:AF%ldC:THD?", channel);

    RETMSG (RSCMU_getViReal64 (instrumentHandle,
                               buffer,
                               THDIn_dB,
                               RSCMU_ERROR_PARAMETER3));
}

/*===========================================================================*/
/* Function: ReadArrThd                                               
/* Purpose : This function starts a new measurement and returns the audio level 
/*           for all harmonics d2 to d9.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_ReadArrThd (ViSession instrumentHandle,
                                         ViInt32 channel,
                                         ViReal64 harmonicsLevels[],
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE] = "";
    ViChar      buffer[RD_BUFFER_SIZE] = "";
    ViChar*     str;
    ViInt32     i;        

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "READ:ARR:THD:AF%ldC?", channel);

    CHECKRETMSG (RSCMU_getViString (instrumentHandle,
                                    buffer,
                                    rdBuf,              
                                    RD_BUFFER_SIZE,
                                    0));
                                
    GET_VI_REAL64(harmonicsLevels[0], rdBuf);
    for (i = 1; i <= 7; i++)
    {
        GET_VI_REAL64(harmonicsLevels[i], NULL);
    }
    
    RETMSG (VI_SUCCESS);
}

/*===========================================================================*/
/* Function: FetcArrThd                                               
/* Purpose : This function reads measurement results (unsynchronized), the 
/*           audio level for all harmonics d2 to d9.
/*           
/* Firmware: V4.37
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_FetcArrThd (ViSession instrumentHandle,
                                         ViInt32 channel,
                                         ViReal64 harmonicsLevels[],
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE] = "";
    ViChar      buffer[RD_BUFFER_SIZE] = "";
    ViChar*     str;
    ViInt32     i;        

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf (buffer, "FETC:ARR:THD:AF%ldC?", channel);

    CHECKRETMSG (RSCMU_getViString (instrumentHandle,
                                    buffer,
                                    rdBuf,              
                                    RD_BUFFER_SIZE,
                                    0));
                                
    GET_VI_REAL64(harmonicsLevels[0], rdBuf);
    for (i = 1; i <= 7; i++)
    {
        GET_VI_REAL64(harmonicsLevels[i], VI_NULL);
    }
    
    RETMSG (VI_SUCCESS);    
}

/*
 * ----------------------------------------------------------------------------
 * Close
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Close
/* Purpose : This function closes the communication session with the Audio
/*           Function Group and deallocates all system resources allocated to
/*           the Audio Non-Signalling Session Handle.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Close_Audio_NSig (ViSession       AudioNSigSession,
                                          ViChar _VI_FAR  errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    struct RSCMU_Audio_NSig_Globals *p_globalvarAudioNSig;

    // Queries the specified attribute for the Audio Non-Signalling object:
    if ((RSCMU_Status = viGetAttribute (AudioNSigSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarAudioNSig)) < 0)
        return RSCMU_Base_errorMessage (AudioNSigSession, RSCMU_Status, errorMessage);

    // Deallocate the space pointed by p_globalvarAudioNSig:
    if (p_globalvarAudioNSig)
        free (p_globalvarAudioNSig);

    // Deallocate all system resources allocated to the Audio Non-Signalling Session:
    CHECKRET(viClose (AudioNSigSession));

  return RSCMU_Status;
}

/*****************************************************************************/
/*= CLASS: UTILITY ==========================================================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Reset
/* Purpose : This function resets the instrument.
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_Reset (ViSession instrSession,
                                ViChar    errorMessage[])
{
    ViUInt32    timeout;
    ViStatus    RSCMU_Status;
    ViInt32     dummy_opc;

    CHECKRET( viGetAttribute( instrSession, VI_ATTR_TMO_VALUE, &timeout ) );
    CHECKRET( viSetAttribute( instrSession, VI_ATTR_TMO_VALUE, 60000 ) );

    CHECKERR( RSCMU_getViInt32(instrSession, "*CLS;STAT:PRES;*RST;*OPC?", &dummy_opc,0 ));

    CHECKRET( viSetAttribute( instrSession, VI_ATTR_TMO_VALUE, timeout ) );

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Error Query
/* Purpose : This function queries the instrument error queue, and returns
/*           the result.
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_ErrorQuery (ViSession  instrSession,
                                     ViInt32    *errorCode,
                                     ViChar      errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    GET_RD_BUF( instrSession );

    /*-----------------------------------------------------------------------*/
    /* Parse the error message and return the Error Code and Error String.
    /* This code makes the following assumption: The String returned from
    /* the instrument is in the format set forth in the SCPI 1994 Command
    /* Reference which is <Error/Event Number,"Error/Event Descriptor">,
    /* for example  0,"No Error" or -410,"Query Interrupted".
    /*----------------------------------------------------------------------*/
    CHECKRETMSG(RSCMU_getViString(instrSession, "SYST:ERR?",
                                rdBuf, RD_BUFFER_SIZE,0));
    sscanf (rdBuf, "%ld,\"%[^\"]", errorCode, errorMessage);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Current Version
/* Purpose : This function translates the Current Version value from the CMU
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_CurrentVersion (ViSession instrSession, ViInt32 *CurrentVersion, ViChar _VI_FAR  message[])
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    ViInt32     num_cmu200_fw_versions = 0;
    ViInt32 i, MainVersion, Extension, FullVersion;
    struct RSCMU_Base_Globals     *p_globalvarBase;

    //================================================================
    // RSCMU_Version is used in the RSCMU_CurrentVersion function
    //================================================================
    typedef struct  RSCMU_Version
    {
        ViStatus stringVal;
        ViString stringName;
    } RSCMU_tVersion;

#define EXPAND2(id, value, text) {id, text},
    RSCMU_tVersion VersionDescArray[] = {
        RSCMU_LIST_OF_AVAILABLE_CMU200_FW_VERSIONS
    };

#undef EXPAND2
    CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

    num_cmu200_fw_versions = sizeof(VersionDescArray) / sizeof(RSCMU_tVersion);

    strcpy (message, "No Error");

    *CurrentVersion = -1;

    // if engineering version nothing to do
    if ((strstr (p_globalvarBase->RSCMU_tGlobal.CmuVersion, "X") == NULL) &&
        (strstr (p_globalvarBase->RSCMU_tGlobal.CmuVersion, "S") == NULL) &&
        (strstr (p_globalvarBase->RSCMU_tGlobal.CmuVersion, "Z") == NULL))
    {
        for (i = 0; i < num_cmu200_fw_versions; i++ )
        {
            if (strcmp (VersionDescArray[i].stringName, p_globalvarBase->RSCMU_tGlobal.CmuVersion) == 0)
            {
                *CurrentVersion = VersionDescArray[i].stringVal;
                return (RSCMU_Status);
            }
        }
        if (i == num_cmu200_fw_versions)
        {
            if (p_globalvarBase->RSCMU_tGlobal.RSCMU_Type == RSCBT)
              sscanf(p_globalvarBase->RSCMU_tGlobal.CmuVersion,"%*[^V]V%ld.%ld", &MainVersion,&Extension);
            else
              sscanf(p_globalvarBase->RSCMU_tGlobal.CmuVersion,"V%ld.%ld", &MainVersion,&Extension);

            FullVersion = MainVersion * 100 + Extension;

            if (FullVersion < VersionDescArray[0].stringVal)
                return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_FW_VERSION, message);
            else
                *CurrentVersion = FullVersion;
        }
    }
    else
    {
        // take per default the last version
        *CurrentVersion = VersionDescArray[num_cmu200_fw_versions - 1].stringVal;
        strcpy (p_globalvarBase->RSCMU_tGlobal.CmuVersion, VersionDescArray [num_cmu200_fw_versions - 1].stringName);
    }

    // Here the call was successful:
    return (0);
}

/*===========================================================================*/
/* Function: RSCMU_SetCmuVersion
/* Purpose : Forces the type of CMU and its Firmware version -
/*       to use only to deturn engineering version compatibility
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SetCmuVersion (ViSession  BaseSession,
                                      ViInt32    CmuType,
                                      ViString   CmuFWVersion,
                                      ViChar     errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViInt32   CurrentVersion = 0, i;
    ViSession sessionNb;
    struct  RSCMU_Base_Globals     *p_globalvarBase;
    struct  RSCMU_SESSION_Globals  *p_globalvarSession;

    CHECKRET(viGetAttribute (BaseSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

    strcpy (errorMessage, "No Error");

    p_globalvarBase->RSCMU_tGlobal.RSCMU_Type = CmuType;

    strcpy (p_globalvarBase->RSCMU_tGlobal.CmuVersion, CmuFWVersion);
    // Query the CMU version
    RSCMU_Status = RSCMU_CurrentVersion (BaseSession, &CurrentVersion, errorMessage);
    // Update the global structure
    p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version = CurrentVersion;

    for (i = 1 ; i < 30 ; i++)
    {
        sessionNb = p_globalvarBase->RSCMU_tSecAddr[i].SessionNb;
        if (sessionNb != RSCMU_OFF)
        {
            CHECKRET(viGetAttribute (sessionNb, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));
            p_globalvarSession->RSCMU_FW_Version = p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version;
        }
    }

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Error Message
/* Purpose : This function translates the error return value from the
/*           instrument driver into a user-readable string.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Base_errorMessage (ViSession       instrSession,
                                           ViStatus        statusCode,
                                           ViChar _VI_FAR  message[])
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    ViInt16 i;

    /*=======================================================================*/
    /* RSCMU_StringValPair is used in the RSCMU_Base_errorMessage function
    /*=======================================================================*/
    typedef struct  RSCMU_StringValPair
    {
        ViStatus stringVal;
        ViString stringName;
    } RSCMU_tStringValPair;

    #define EXPAND(id, value, text) {id, text},
    RSCMU_tStringValPair statusDescArray[] = {
        RSCMU_LIST_OF_SPECIFIC_ERROR_MESSAGES
    };

    #define NUM_BASE_ERRORS (sizeof(statusDescArray) / sizeof(RSCMU_tStringValPair))
  #undef EXPAND

    strcpy (message, "No Error");

    if (statusCode != VI_SUCCESS)
        RSCMU_Status = viStatusDesc (instrSession, statusCode, message);

    if (RSCMU_Status == VI_WARN_UNKNOWN_STATUS)
    {

        for( i=0; (i < NUM_BASE_ERRORS) && statusDescArray[i].stringName; i++ )
        {
            if (statusDescArray[i].stringVal == statusCode)
            {
                strcpy (message, statusDescArray[i].stringName);
                return (statusCode);
            }
        }
        sprintf (message, "Unknown Error 0x%08lX", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    else if (RSCMU_Status == VI_ERROR_USER_BUF)
    {
      sprintf (message, "The user buffer is not valid for the required size.");
      return (VI_ERROR_USER_BUF);
    }

    // Here the call was successful:
    return (statusCode);
}

/*===========================================================================*/
/* Function: RSCMU_Delay
/* Purpose : With a call to RSCMU_Delay, the current program is suspended
/*           from execution for the number of milliseconds specified by the
/*           argument seconds. RSCMU_Delay is accurate to a millisecond.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Delay (ViReal64 Number_Of_seconds)
{
    clock_t   StartTime = clock();

    while (((clock() - StartTime)) < (Number_Of_seconds*1000));

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: SCPI Version
/* Purpose : This function sets an Delay for STB Query in case of seriell Polling
/*           It is needed to synchronize with STB Setting Time of the CMU !!!
/*===========================================================================*/
#if 0
ViStatus _VI_FUNC RSCMU_SetCheckStatus_QueryDelay (ViSession instrSession,
                                                   ViReal64 statusByteQueryDelay)
{
  ViStatus    RSCMU_Status = VI_SUCCESS;

  d_DelayValue = statusByteQueryDelay;

  return RSCMU_Status;
}
#endif

/*===========================================================================*/
/* Function: SCPI Version
/* Purpose : This function queries the instrument error queue, and returns
/*           the result.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SCPIVersionQuery (ViSession instrSession,
                                          ViChar    ScpiVersion[],
                                          ViChar    errorMessage[])
{
    RETMSG(RSCMU_getViString(instrSession, "SYST:VERS?", ScpiVersion,
                             RD_BUFFER_SIZE, RSCMU_ERROR_PARAMETER2));
}

/*===========================================================================*/
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_Base_RevisionQuery (ViSession instrSession,
                                             ViChar    instrDriverRev[],
                                             ViChar    instrRev[],
                                             ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViInt32     i;
    GET_RD_BUF( instrSession );

    CHECKRETMSG(RSCMU_getViString(instrSession, "*IDN?", rdBuf, RD_BUFFER_SIZE,0));
    sscanf(rdBuf,"%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev);

    for(i = 0; instrRev[i]; i++)
        instrRev[i] = (ViChar) toupper(instrRev[i]);

    /*-----------------------------------------------------------------------*/
    /* Instrument driver revision:
    /* The format for instrDriverRev is a string with four fields separated by
    /* commas, the first field is the revision of the instrument driver, the
    /* second field is the month and year of the version's release, the third
    /* field is the time of the version's release, and the four field is the
    /* version of CVI used to write the current version.
    /*-----------------------------------------------------------------------*/
    strcpy (instrDriverRev, RSCMU_BASE_REVISION);
    strcat (instrDriverRev, ", ");
    /* From the complilation date of the current file */
    strcat (instrDriverRev, __DATE__);
    strcat (instrDriverRev, ", ");
    /* From the complilation time of the current file */
    strcat (instrDriverRev, __TIME__);
    //strcat (instrDriverRev, ", LW/CVI 5.01");

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_Trace_ON
/* Purpose : Set the trace referenced by 'filename' ON
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_Trace_ON (ViSession  BaseSession,
                                   ViString   filename,
                                   ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViInt16     i;
    ViSession   sessionNb;
    FILE        *targetFile;

    struct RSCMU_Base_Globals     *p_globalvarBase;
    struct RSCMU_SESSION_Globals  *p_globalvarSession;

    CHECKRET(viGetAttribute (BaseSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

    if ((targetFile = fopen(filename, "w")) == NULL)
        return RSCMU_Base_errorMessage (BaseSession, RSCMU_ERROR_OPEN_FILE, errorMessage);
    else
        p_globalvarBase->RSCMU_tGlobal.RSCMU_AddrTargetFile = (ViAddr) targetFile;

    p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE = RSCMU_ON;

    for (i = 1 ; i < 30 ; i++)
    {
        sessionNb = p_globalvarBase->RSCMU_tSecAddr[i].SessionNb;
        if (sessionNb != RSCMU_OFF)
            {
                CHECKRET(viGetAttribute (sessionNb, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));
                p_globalvarSession->RSCMU_TRACE = RSCMU_ON;
                p_globalvarSession->RSCMU_AddrTargetFile = (ViAddr) targetFile;
                p_globalvarSession->RSCMU_StateTraceFile = 1;
            }
    }

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_Trace_OFF
/* Purpose : Set the trace referenced by 'filename' OFF
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_Trace_OFF (ViSession  BaseSession,
                                    ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViInt16     i;
    ViSession   sessionNb;
    FILE        *targetFile;

    struct RSCMU_Base_Globals     *p_globalvarBase;
    struct RSCMU_SESSION_Globals  *p_globalvarSession;

    CHECKRET(viGetAttribute (BaseSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

  // if (p_globalvarBase->RSCMU_tGlobal.targetFile->state.InUse == 1)
    if (p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE == RSCMU_ON)
    {
        targetFile = p_globalvarBase->RSCMU_tGlobal.RSCMU_AddrTargetFile ;
        fclose(targetFile);
        p_globalvarBase->RSCMU_tGlobal.RSCMU_StateTraceFile = 0;
    }

  p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE = RSCMU_OFF;

    for (i = RSCMU_SECONDARY_ADDRESS_MIN ; i < RSCMU_SECONDARY_ADDRESS_MAX ; i++)
    {
        sessionNb = p_globalvarBase->RSCMU_tSecAddr[i].SessionNb;
        if (sessionNb != RSCMU_OFF)
        {
            CHECKRET(viGetAttribute (sessionNb, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));
            p_globalvarSession->RSCMU_TRACE = RSCMU_OFF;
            targetFile = NULL;
        }
    }
    return RSCMU_Status;
}

/*=========================================================================*/
/* Function: RSCMU_SetTimeoutAttribute
/* Purpose : This function specifies the minimum timeout value to use (in
/*           milliseconds) when accessing the device associated with the
/*           given session.
/*=========================================================================*/
ViStatus _VI_FUNC  RSCMU_SetTimeoutAttribute (ViSession instrSession,
                                              ViUInt32  Timeout,
                                              ViChar    errorMessage[])
{
    return viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, Timeout);
}

/*===========================================================================*/
/* Function: RSCMU_WriteExtTrace
/* Purpose : Write a buffer to the current trace file
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_WriteExtTrace   (ViSession  instrSession,
                                          ViChar     Buffer[],
                                          ViChar     errorMessage[])
{
  ViStatus  RSCMU_Status = VI_SUCCESS;
  ViUInt32  BytesWritten;
  FILE     *targetFile;
  struct RSCMU_SESSION_Globals  *p_globalvarSession;

  // Get the Globals
  if ((RSCMU_Status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession)) < 0)
    return RSCMU_Status;

  targetFile = p_globalvarSession->RSCMU_AddrTargetFile;
  BytesWritten = fwrite (Buffer, sizeof(ViChar), strlen(Buffer), targetFile);

  // If a write error is encountered:
  if (BytesWritten < strlen(Buffer))
    return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_WRITE_FILE, errorMessage);

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Go to remote without call drop                                  */
/* Purpose:  This function causes the signalling state to be maintained upon */
/*           a local to remote transition.                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SystGtrmComp (ViSession instrSession,
                                      ViBoolean enable,
                                      ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_checkVersion(instrSession, RSCMU200_V3_05, RSCMU_ERROR_FW_VERSION_3_05));

    RETMSG(RSCMU_setEnum(instrSession,"SYST:GTRM:COMP",
                         enable, mode_On_Off,
                         RSCMU_ERROR_PARAMETER2));
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Check Version                                                   */
/* Purpose:  This function checks the current CMU version                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_checkVersion (ViSession vi,
                             ViInt32    version,
                             ViStatus   errorCode)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    struct      RSCMU_SESSION_Globals    *p_globalvarBase;

    CHECKRET(viGetAttribute (vi, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));

    if (p_globalvarBase->RSCMU_FW_Version < version)
        return errorCode;

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: String to char (uint8)                                                 */
/* Purpose:  This function converts the string to ViReal64 value with NAN    */
/*           checking.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_atochar(ViString _VI_FAR str, ViChar* c)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    int tmp = 0;
    if (str)
        if (strncmp(str,"NAN",3))
        {
            tmp = atoi (str);
            if ((0 <=tmp) && (tmp <= 255))
            {
                *c = (char)tmp;
                return RSCMU_Status;
            }
        }

    return RSCMU_ERROR_NAN;
}
/*===========================================================================*/
/* Function: String to float array                                           */
/* Purpose:  This function converts the string to array of ViReal64 values   */
/*           with NAN checking. Values must be separated by commas           */
/*===========================================================================*/
ViStatus  _VI_FUNC RSCMU_atoCharArray(ViString _VI_FAR string, ViChar arr[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViInt32 i = 0;
    ViChar* str;

    str = strtok(string,",");
    while(str)
    {
        if((RSCMU_Status = RSCMU_atochar(str,&arr[i++]))<0);
           return RSCMU_Status;
    }
    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: String to float                                                 */
/* Purpose:  This function converts the string to ViReal64 value with NAN    */
/*           checking.                                                       */
/*===========================================================================*/
ViReal64 _VI_FUNC RSCMU_atof(ViString _VI_FAR str)
{
    if (str)
        if (strncmp(str,"NAN",3))
            return atof (str);

    return RSCMU_NAN;
}

/*===========================================================================*/
/* Function: String to int                                                   */
/* Purpose:  This function converts the string to ViReal64 value with NAN    */
/*           checking.                                                       */
/*===========================================================================*/
ViInt32 _VI_FUNC RSCMU_atol(ViString _VI_FAR str)
{
    if (str)
        if (strncmp (str, "NAN", 3))
            return atol (str);

    return RSCMU_NAN_VI_INT32;
}

/*===========================================================================*/
/* Function: String to float array                                           */
/* Purpose:  This function converts the string to array of ViReal64 values   */
/*           with NAN checking. Values must be separated by commas           */
/*===========================================================================*/
void _VI_FUNC RSCMU_atofArray(ViString _VI_FAR string, ViReal64 arr[])
{
    ViInt32 i = 0;
    ViChar* str;

    str = strtok(string,",");
    while(str)
    {
        arr[i++] = RSCMU_atof(str);
        str = strtok(NULL, ",");
    }
}
/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_setViInt32 (ViSession vi,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    if (RSCMU_invalidViInt32Range (value, min, max))
        return parameterErrorCode;

    sprintf (wrtBuf, "%s %ld", cmd, value);

    return RSCMU_sendCmd(vi, wrtBuf);
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_getViInt32 (ViSession vi,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKRET(RSCMU_viWrite(vi, cmd, strlen(cmd)));
    CHECKRET(RSCMU_viRead(vi,rdBuf,RD_BUFFER_SIZE));
    CHECKRET(RSCMU_sysStatus (vi, VI_NULL));

    *value = RSCMU_atol (rdBuf);
    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    if (max - min > 1.0e-14)
        if (RSCMU_invalidViReal64Range (value, min, max))
            return parameterErrorCode;

    sprintf(wrtBuf,"%s %.12f",cmd, value);

    return RSCMU_sendCmd(vi, wrtBuf);
}

/*===========================================================================*/
/* Function: Set ViReal64 with units                                         */
/* Purpose:  This function sets command with double parameter and with the   */
/*           unit .                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_setViReal64Unit (ViSession  vi,
                                       ViString cmd,
                                       ViReal64 value,
                                       ViReal64 min[],
                                       ViReal64 max[],
                                       ViInt32  unit,
                                       ViString unitArr[],
                                       ViStatus parameterErrorCode)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    CHECKRET (RSCMU_checkIndexInList (vi, unit, unitArr, parameterErrorCode-1));
    CHECKRET (RSCMU_checkIndexInRealList (vi, unit, min, parameterErrorCode));
    CHECKRET (RSCMU_checkIndexInRealList (vi, unit, max, parameterErrorCode));
    if (max[unit] - min[unit] > 1.0e-14)
        if (RSCMU_invalidViReal64Range (value, min[unit], max[unit]))
            return parameterErrorCode;

    sprintf(wrtBuf,"%s %.12f %s",cmd, value, unitArr[unit]);

    return RSCMU_sendCmd(vi, wrtBuf);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_getViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 *value,
                             ViStatus parameterErrorCode)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      buffer[RD_BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKRET(RSCMU_viWrite (vi, cmd, strlen(cmd)));
    CHECKRET(RSCMU_viRead (vi, buffer, RD_BUFFER_SIZE));

    *value = RSCMU_atof(buffer);

    CHECKRET(RSCMU_sysStatus (vi, VI_NULL));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_sendCmd (ViSession vi,
                        ViString cmd)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    CHECKRET(RSCMU_viWrite (vi, cmd, strlen(cmd)));
    CHECKRET(RSCMU_sysStatus(vi, VI_NULL));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_setViString (ViSession vi,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar*     wrtBuf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrtBuf = (ViChar*)malloc(strlen(cmd)+strlen(value)+2);
    p2buf = wrtBuf;
    p2buf += sprintf(wrtBuf,"%s %s",cmd, value);

    CHECKERR(RSCMU_viWrite (vi, wrtBuf, p2buf - wrtBuf));

    free(wrtBuf);

    CHECKRET(RSCMU_sysStatus(vi, VI_NULL));
    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_getViString (ViSession vi,
                            ViString _VI_FAR cmd,
                            ViString _VI_FAR value,
                            ViInt32 buffer_size,
                            ViStatus parameterErrorCode)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKRET(RSCMU_viWrite(vi, cmd, strlen(cmd)));
    CHECKRET(RSCMU_viRead(vi, value, buffer_size));

    CHECKRET(RSCMU_sysStatus (vi, VI_NULL));
    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_setEnum (ViSession vi,
                        ViString cmd,
                        ViInt32 value,
                        ViString _VI_FAR list[],
                        ViStatus parameterErrorCode)
{
    ViStatus RSCMU_Status = VI_SUCCESS;

    CHECKRET (RSCMU_checkIndexInList (vi, value, list, parameterErrorCode));

    return RSCMU_setViString (vi, cmd, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_getEnum (ViSession vi,
                        ViString cmd,
                        ViInt32 *value,
                        ViString _VI_FAR list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      buffer[RD_BUFFER_SIZE];

    CHECKRET(RSCMU_getViString(vi, cmd, buffer,RD_BUFFER_SIZE, parameterErrorCode));
//    buffer[strlen(buffer) - 1] = '\0';

    CHECKRET(RSCMU_getIndexFromString (vi, buffer, list, value));

    return RSCMU_Status;

}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_checkIndexInList (ViSession vi,
                                 ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;

    if (index < 0)
        return statusIfError;

    while (list[i++] != VI_NULL);
    if (index >= i)
        return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_checkIndexInRealList (ViSession vi,
                                 ViInt32 index,
                                 ViReal64 list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;

    if (index < 0)
        return statusIfError;

    while (list[i++] != RSCMU_NAN);
    if (index >= i)
        return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_getIndexFromString (ViSession vi,
                                   ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViUInt32    i,
                len;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = i;

    return (list[i]) ? RSCMU_Status : RSCMU_ERROR_INTERPRETING_RESPONSE;
}

/*=========================================================================*/
/* Function: Write to winbatch file                                        */
/* Purpose:  This function writes the string sent to the instrument to the */
/*           winbatch log file                                             */
/*=========================================================================*/
ViStatus _VI_FUNC RSCMU_WinBatchWrite (ViSession instrSession,
                         ViString _VI_FAR command)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    size_t FGrp_name_length = 0;
    ViChar trace_write_buf [WR_BUFFER_SIZE];
    ViChar* p2buf;
    ViUInt32    bytes_written;
    struct RSCMU_SESSION_Globals  *p_globalvarSession;

    CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));

    if (p_globalvarSession->RSCMU_TRACE)
    {
        // To write the command in a WinBatch File, we need the FGrp_Name.
        FGrp_name_length = strlen(p_globalvarSession->FGrp_Name);

        trace_write_buf[0] = '\n';
        // Extract the quotes given by the CMU before and after the FGrp_Name.
        if (p_globalvarSession->FGrp_Name[0] == 34)    // ASCII code for ".
        {
            memcpy(trace_write_buf+1, p_globalvarSession->FGrp_Name+1, FGrp_name_length-2);
            //RSCMU_copyString (FGrpBuf, p_globalvarSession->FGrp_Name, 1, FGrp_name_length-2);
            FGrp_name_length = FGrp_name_length - 2;
        }
        else
            strncpy (trace_write_buf+1, p_globalvarSession->FGrp_Name, FGrp_name_length);

        p2buf = trace_write_buf + FGrp_name_length + 1;
        sprintf(p2buf,":%s\n", command);
        // Open a Winbatch file and leave the current remote command
        //targetFile = p_globalvarSession->RSCMU_AddrTargetFile;

        bytes_written = fwrite (trace_write_buf, sizeof(ViChar), strlen(trace_write_buf), p_globalvarSession->RSCMU_AddrTargetFile);

          // If a write error is encountered:
        if (bytes_written < strlen(trace_write_buf))
            RSCMU_Status = RSCMU_ERROR_WRITE_FILE;
    }
    return RSCMU_Status;
}

/*=========================================================================*/
/* Function: Write to winbatch file                                        */
/* Purpose:  This function writes the response from the instrument to the  */
/*           winbatch log file                                             */
/*=========================================================================*/
ViStatus _VI_FUNC RSCMU_WinBatchRead (ViSession instrSession,
                         ViString _VI_FAR buffer)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    size_t FGrp_name_length = 0;
//  ViChar trace_write_buf [WR_BUFFER_SIZE];
    ViChar* trace_write_buf;
    ViChar* p2buf;
    ViUInt32    bytes_written;
    struct RSCMU_SESSION_Globals  *p_globalvarSession;

    CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));

    if (p_globalvarSession->RSCMU_TRACE)
    {
        // To write the command in a WinBatch File, we need the FGrp_Name.
        FGrp_name_length = strlen(p_globalvarSession->FGrp_Name);

        trace_write_buf = (ViChar*)malloc(FGrp_name_length+strlen(buffer)+1);

        trace_write_buf[0] = ';';
        // Extract the quotes given by the CMU before and after the FGrp_Name.
        if (p_globalvarSession->FGrp_Name[0] == 34)    // ASCII code for ".
        {
            memcpy(trace_write_buf+1, p_globalvarSession->FGrp_Name+1, FGrp_name_length-2);
            //RSCMU_copyString (FGrpBuf, p_globalvarSession->FGrp_Name, 1, FGrp_name_length-2);
            FGrp_name_length = FGrp_name_length - 2;
        }
        else
            strncpy (trace_write_buf+1, p_globalvarSession->FGrp_Name, FGrp_name_length);

//      p2buf = trace_write_buf + FGrp_name_length + 1;
        p2buf = trace_write_buf;
        sprintf(p2buf,":%s", buffer);
        // Open a Winbatch file and leave the current remote command
        //targetFile = p_globalvarSession->RSCMU_AddrTargetFile;

        bytes_written = fwrite (trace_write_buf, sizeof(ViChar), strlen(trace_write_buf), p_globalvarSession->RSCMU_AddrTargetFile);

          // If a write error is encountered:
        if (bytes_written < strlen(trace_write_buf))
            RSCMU_Status = RSCMU_ERROR_WRITE_FILE;

        free(trace_write_buf);

    }
    return RSCMU_Status;
}

/*=========================================================================*/
/* Function: Set Check Status                                              */
/* Purpose:  This function sets the bit for status checking                */
/*=========================================================================*/
ViStatus _VI_FUNC RSCMU_setCheckStatus(ViSession vi, ViBoolean checkStatus)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    struct RSCMU_Base_Globals* instrPtr;

    CHECKRET (viGetAttribute (vi, VI_ATTR_USER_DATA, &instrPtr));

/*===========================================================================*/
/*  new Error QueryHandling: Using the last second and third bit as *STB ON/OFF Flag
/*                           for all Commands without RSCMU_Write/_Read
/*                           Possible Status: OFF ,
/*                                            ON *STB after every Command,
/*                                            ON seriell Polling of status
/*===========================================================================*/

    switch (checkStatus)
        {
        case 0:

/*  65529 -> .RSCMU_Stb & 1111 1111 1111 1001  Off */
            instrPtr->RSCMU_tGlobal.RSCMU_Stb = instrPtr->RSCMU_tGlobal.RSCMU_Stb & 65529;

            break;
        case 1:
/*  65529 -> RSCMU_Stb & 1111 1111 1111 1001  Off */
/*  2 -> RSCMU_Stb | 0000 0000 0000 0010  ON *STB Standard*/
            instrPtr->RSCMU_tGlobal.RSCMU_Stb = instrPtr->RSCMU_tGlobal.RSCMU_Stb &65529;
            instrPtr->RSCMU_tGlobal.RSCMU_Stb = instrPtr->RSCMU_tGlobal.RSCMU_Stb |2;

            break;
        case 2:
/*  6 -> RSCMU_Stb | 0000 0000 0000 0110  ON seriel Polling*/
            instrPtr->RSCMU_tGlobal.RSCMU_Stb = instrPtr->RSCMU_tGlobal.RSCMU_Stb |6;

            break;
        default:

            break;
        }

    return RSCMU_Status;
}
/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function reads the system status byte.                   */
/*=========================================================================*/
ViStatus _VI_FUNC RSCMU_sysStatus (ViSession vi, ViUInt16* stb)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    ViUInt16 stbByte = 0;
    struct RSCMU_Base_Globals* instrPtr;
    ViUInt16 intf_type, i_temp;
    ViChar rdBuf[5];

    CHECKRET (viGetAttribute (vi, VI_ATTR_USER_DATA, &instrPtr));

/*===========================================================================*/
/*  new Error QueryHandling: value of Bit 2 and 3 to i_temp
/*===========================================================================*/
             i_temp = instrPtr->RSCMU_tGlobal.RSCMU_Stb;
             i_temp = instrPtr->RSCMU_tGlobal.RSCMU_Stb & 6;

/*===========================================================================*/
/*  new Error QueryHandling: Using the last second and third bit as *STB ON/OFF Flag
/*                           for all Commands without RSCMU_Write/_Read
/*                           Possible Status: OFF ,
/*                                            ON *STB after every Command,
/*                                            ON seriell Polling of status
/*===========================================================================*/
    switch (i_temp)
        {
/*===========================================================================*/
/*  new Error QueryHandling: value of Bit 2 and 3 => 00       OFF
/*===========================================================================*/
        case 0:

            break;
/*===========================================================================*/
/*  new Error QueryHandling: value of Bit 2 and 3 => 01       ON  *STB standard
/*===========================================================================*/
        case 2:
        CHECKRET (viGetAttribute (vi, VI_ATTR_INTF_TYPE, &intf_type));
            memset (rdBuf, 0, 5);
            CHECKRET (RSCMU_WinBatchWrite(vi,"*STB?"));
            CHECKRET (viPrintf (vi, "*STB?\r\n"));
            CHECKRET (viRead (vi, (ViPBuf)rdBuf, 4, VI_NULL));
            CHECKRET (RSCMU_WinBatchRead(vi,rdBuf));
            sscanf (rdBuf,  "%hu", &stbByte);
    /*--------------- Check if any error or message bit was asserted -*/

            if (stbByte & 0x4)
            RSCMU_Status = RSCMU_ERROR_STATUS_BYTE;

    if(stb)
        *stb = stbByte;

    return RSCMU_Status;

/*===========================================================================*/
/*  new Error QueryHandling: value of Bit 2 and 3 => 11       ON seriell Polling
/*===========================================================================*/
        case 6:
        CHECKRET (viGetAttribute (vi, VI_ATTR_INTF_TYPE, &intf_type));
        if(intf_type == VI_INTF_ASRL)
        {
            /*--------------- Read the Status Byte ---------------*/
            memset (rdBuf, 0, 5);
            CHECKRET (RSCMU_WinBatchWrite(vi,"*STB?"));
            CHECKRET (viPrintf (vi, "*STB?\r\n"));
            CHECKRET (viRead (vi, (ViPBuf)rdBuf, 4, VI_NULL));
            CHECKRET (RSCMU_WinBatchRead(vi,rdBuf));
            sscanf (rdBuf,  "%hu", &stbByte);
        }
        else
        {
                CHECKRET(viReadSTB(vi,&stbByte));
        }

/*--------------- Check if any error or message bit was asserted -*/

            if (stbByte & 0x4)
            RSCMU_Status = RSCMU_ERROR_STATUS_BYTE;

    if(stb)
        *stb = stbByte;

    return RSCMU_Status;

        default:
    // nothing happens, STB Flag not initialized
            break;
        }

    return RSCMU_Status;

}

/******************************************************************************
 * Function: Read Data With Unknown length
 * Description: This function allocate buffer and read data from the instrument
 * WARNING:  You must deallocate buffer, if any pointer has been returned.
 *           This function does not work correctly, when you read bin data via
 *           serial interface
 *****************************************************************************/
ViStatus _VI_FUNC RSCMU_ReadDataUnknownLength(ViSession instrSession,
                                            ViChar **outBuffer,
                                            ViUInt32 *count)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = VI_NULL, *p2buffer;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = RD_BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
        {
        if ((RSCMU_Status = viRead (instrSession, (ViPBuf)p2buffer, RD_BUFFER_SIZE, &retCnt)) < 0)
            {
            free(buffer);
            *outBuffer = VI_NULL;
            return RSCMU_Status;
            }
        if (retCnt == RD_BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += RD_BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - RD_BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string on the end of data */
        }
       } while (RSCMU_Status == 0x3FFF0006); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count) *count = (actSize - RD_BUFFER_SIZE) + retCnt;

    CHECKRET(RSCMU_WinBatchRead(instrSession,*outBuffer));
    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_viWrite
/* Purpose : Send the data to the instrument and to the WinBatch file
/*
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_viWrite (ViSession  instrSession,
                                 ViChar     writeBuf[],
                                 ViUInt32   cnt)
{
    ViUInt16  intf_type;
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    wrtBuf [WR_BUFFER_SIZE];
    static    ViInt32 Previous_Sec_Address = -1;

    struct RSCMU_SESSION_Globals  *p_globalvarSession;

    // Get the Globals
    CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));
    CHECKRET(viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf_type));

    if (intf_type == VI_INTF_ASRL)
    {
        if (Previous_Sec_Address != p_globalvarSession->RSCMU_secondary_address)
        {
            sprintf(wrtBuf,"*SEC %ld",p_globalvarSession->RSCMU_secondary_address);
            CHECKRET(RSCMU_WinBatchWrite(instrSession, wrtBuf));
            CHECKRET(viPrintf (instrSession, "%s\r\n", wrtBuf));
            Previous_Sec_Address = p_globalvarSession->RSCMU_secondary_address;
        }
    }

    memcpy(wrtBuf, writeBuf, cnt);
    wrtBuf[cnt] = '\0';

    CHECKRET(RSCMU_WinBatchWrite(instrSession, wrtBuf));

    CHECKRET(viPrintf (instrSession, "%s\r\n", wrtBuf));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_viRead
/* Purpose : Reads the response from the instrument and puts it to the
/*           WinBatch file
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_viRead (ViSession  instrSession,
                                ViChar     readBuf[],
                                ViUInt32   cnt)
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViUInt32  rtnCnt;

    CHECKRET(viRead(instrSession, (ViPBuf)readBuf, cnt, &rtnCnt));

    if (isspace(readBuf[rtnCnt-1]))
        readBuf[rtnCnt-1] = '\0';
    else
        readBuf[rtnCnt] = '\0';

    CHECKRET(RSCMU_WinBatchRead(instrSession, readBuf));

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_getTabSecAdd
/* Purpose : Query at CMU to obtain a string containing Secondary Addresses
/*           and options associed to these addresses. This string is breaked
/*           into tokens and putted in the Globals of BASIC UNIT Session
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_getTabSecAdd (ViSession instrSession, ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;
    ViInt16     i;
    struct RSCMU_Base_Globals *p_globalvarBase;

    CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase));
    CHECKRETMSG(RSCMU_getViString(instrSession,"SYST:REM:ADDR:SEC?",rdBuf,
                                  RD_BUFFER_SIZE, 0));

    str = strtok(rdBuf,",");
    for (i = 0; i < 30; i++)
    {
        strcpy (p_globalvarBase->RSCMU_tSecAddr[i].FctGrp, str);
        if (((str = strtok(NULL,",")) == NULL) && (i < 29))
            return RSCMU_ERROR_INTERPRETING_RESPONSE;
    }

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_WriteFile
/* Purpose : Write a buffer given by the instrument to a specified file
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_WriteFile (ViSession instrSession,
                                   ViAddr    ptTargetFile,
                                   ViChar    Buffer[],
                                   ViChar    errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViUInt32    BytesWritten;
  FILE       *targetFile;

  targetFile = ptTargetFile;

  BytesWritten = fwrite (Buffer, sizeof(ViChar), strlen(Buffer), targetFile);

  // If a write error is encountered:
  if (BytesWritten < strlen(Buffer))
    return RSCMU_Base_errorMessage (instrSession, RSCMU_ERROR_WRITE_FILE, errorMessage);

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Initialize Clean Up
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_initCleanUp (ViSession  openRMSession,
                                     ViSession  openInstrSession,
                                     ViStatus   currentStatus)
{
  ViStatus RSCMU_Status = VI_SUCCESS;

  if ((RSCMU_Status = viClose (openInstrSession)) < 0)
    return RSCMU_Status;

  if ((RSCMU_Status = viClose (openRMSession)) < 0)
    return RSCMU_Status;

  openInstrSession = VI_NULL;

  currentStatus = RSCMU_Status;

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean
/* Purpose : This function checks a Boolean to see if it is equal to VI_TRUE
/*           or VI_FALSE. If the value is out of range, the return value is
/*           VI_TRUE, otherwise the return value is VI_FALSE.
/*===========================================================================*/
ViBoolean _VI_FUNC RSCMU_invalidViBooleanRange (ViBoolean val)
{
  return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16
/* Purpose : This function checks a short unsigned integer value to see if it
/*           lies between a minimum and maximum value.  If the value is out
/*           of range,  the return value is VI_TRUE, otherwise the return
/*           value is VI_FALSE.
/*===========================================================================*/
ViBoolean _VI_FUNC RSCMU_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
  return ((val < min || val >max) ? VI_TRUE : VI_FALSE);
}

ViBoolean _VI_FUNC RSCMU_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
  return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32
/* Purpose : This function checks a long signed integer value to see if it
/*           lies between a minimum and maximum value.  If the value is out
/*           of range,  the return value is VI_TRUE, otherwise the return
/*           value is VI_FALSE.
/*===========================================================================*/
ViBoolean _VI_FUNC RSCMU_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
  return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32
/* Purpose : This function checks a long unsigned integer value to see if it
/*           lies between a minimum and maximum value.  If the value is out
/*           of range,  the return value is VI_TRUE, otherwise the return
/*           value is VI_FALSE.
/*===========================================================================*/
ViBoolean _VI_FUNC RSCMU_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
  return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32
/* Purpose : This function checks a real (float) value to see if it lies
/*           between a minimum and maximum value.  If the value is out of
/*           range, the return value is VI_TRUE, otherwise the return value
/*           is VI_FALSE.
/*===========================================================================*/
ViBoolean _VI_FUNC RSCMU_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
  return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64
/* Purpose : This function checks a real (double) value to see if it lies
/*           between a minimum and maximum value.  If the value is out of
/*           range, the return value is VI_TRUE, otherwise the return value
/*           is VI_FALSE.
/*===========================================================================*/
ViBoolean _VI_FUNC RSCMU_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
  return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: RSCMU_RF_NSig_GetResourceName                                   */
/* Purpose:  This function returns resource descriptor for TDMA option.      */
/*           Session parameter must contain at least 256 elements ViChar[256]*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_GetResourceName (ViSession instrSession,
                                                 ViSession instrSessionBase,
                                                 ViChar _VI_FAR sessionName[])
{
  ViStatus    RSCMU_Status    = VI_SUCCESS;
  ViInt32     comp;

  struct RSCMU_Base_Globals  *p_globalvarBase;
  struct RSCMU_RF_NSig_Globals  *p_globalvarRF;

  /* Get a copy of the global data for base session. */
  CHECKRET(viGetAttribute( instrSessionBase
                                     , VI_ATTR_USER_DATA
                                     , (ViAddr)&p_globalvarBase
                                     ));
  /* Get a copy of the global data for this session. */
  CHECKRET(viGetAttribute( instrSession
                                     , VI_ATTR_USER_DATA
                                     , (ViAddr)&p_globalvarRF
                                     ));

  // Hardware Connection
  comp = strncmp("GPIB", p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, 4);

  if( comp == 0 )
  {
    sprintf( sessionName
            , "%s::%ld::%ld::INSTR"
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address
            , p_globalvarRF->RSCMU_tGlobal.RSCMU_secondary_address
           );
  }
  else
  {
    sprintf( sessionName
            , "%s::INSTR"
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection
           );
  }

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_Audio_NSig_GetResourceName                                */
/* Purpose:  This function returns resource descriptor for TDMA option.      */
/*           Session parameter must contain at least 256 elements ViChar[256]*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_Audio_NSig_GetResourceName (ViSession instrSession,
                                                    ViSession instrSessionBase,
                                                    ViChar _VI_FAR sessionName[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViInt32     comp;

  struct RSCMU_Base_Globals  *p_globalvarBase;
  struct RSCMU_Audio_NSig_Globals  *p_globalvarRF;

  /* Get a copy of the global data for base session. */
  CHECKRET(viGetAttribute( instrSessionBase
                                     , VI_ATTR_USER_DATA
                                     , (ViAddr)&p_globalvarBase
                                     ));

  /* Get a copy of the global data for this session. */
  CHECKRET(viGetAttribute( instrSession
                                     , VI_ATTR_USER_DATA
                                     , (ViAddr)&p_globalvarRF
                                     ));

  // Hardware Connection
  comp = strncmp( "GPIB", p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, 4);

  if( comp == 0 )
  {
    sprintf( sessionName
            , "%s::%ld::%ld::INSTR"
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address
            , p_globalvarRF->RSCMU_tGlobal.RSCMU_secondary_address
           );
  }
  else
  {
    sprintf( sessionName
            , "%s::INSTR"
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection
           );
  }

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_SendControlCommand
/* Purpose : This function send a command to the CMU
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SendControlCommand (ViSession instrSession,
                                            ViInt32   StartCmd,
                                            ViInt32   Object,
                                            ViInt32   Modulation,
                                            ViChar    errorMessage[])

{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    if ((Object == RSCMU_POWMSL)||(Object == RSCMU_NPOW))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_05,RSCMU_ERROR_FW_VERSION_3_05));

    if (Object == RSCMU_POWABUR)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_00,RSCMU_ERROR_FW_VERSION_3_00));

    if (Object == RSCMU_WPOW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    sprintf (wrtBuf, "%s%s%s", StrCommand[StartCmd], StrObject[Object], StrModulation[Modulation]);
    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Event reporting of the measurement
/* Purpose : This function defines the events generated when the
/*           measurement is terminated or stopped.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_ConfReporting (ViSession  instrSession,
                                       ViInt32    Object,
                                       ViInt32    Modulation,
                                       ViInt32    Report,
                                       ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Report, 0, 3, RSCMU_ERROR_PARAMETER2, errorMessage);

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    if (Object == RSCMU_POWMSL)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_05,RSCMU_ERROR_FW_VERSION_3_05));

    if (Object == RSCMU_POWABUR)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_00,RSCMU_ERROR_FW_VERSION_3_00));

    if (Object == RSCMU_WPOW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    sprintf (wrtBuf, "CONF%s%s:EREP %s", StrObject[Object], StrModulation[Modulation], modeArr[Report]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Power/t Measurement Status
/* Purpose : This function returns the status of the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_FetcStatus (ViSession  instrSession,
                                    ViInt32    Object,
                                    ViInt32    Modulation,
                                    ViInt32   *MeasurementStatus,
                                    ViInt32   *CountingNo,
                                    ViInt32   *StatisticNo,
                                    ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      rdBuf[RD_BUFFER_SIZE], wrtBuf[WR_BUFFER_SIZE];
    ViChar*     str;

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    if (Object == RSCMU_POWMSL)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_05,RSCMU_ERROR_FW_VERSION_3_05));

    if (Object == RSCMU_POWABUR)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_00,RSCMU_ERROR_FW_VERSION_3_00));

    if (Object == RSCMU_WPOW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    // Fetch the status of the Power/t measurement:
    sprintf(wrtBuf,"FETC%s%s:STAT?", StrObject[Object], StrModulation[Modulation]);
    CHECKRETMSG(RSCMU_getViString(instrSession, wrtBuf, rdBuf, RD_BUFFER_SIZE,0));
    if (!(str = strtok(rdBuf,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    // Get the status of the measurement:
    CHECKRET(RSCMU_getIndexFromString(instrSession, rdBuf, measurementStatusArr, MeasurementStatus));

    if (!(str = strtok(NULL,",")))
        return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);

    if(!sscanf(str,"%ld",CountingNo))
        *CountingNo = RSCMU_NONE;

    // Get the current evaluation period within a statistics cycle:
    if (Object < RSCMU_POWSLOT)
    {
        if (!(str = strtok(NULL,",")))
            return RSCMU_Base_errorMessage(instrSession, RSCMU_ERROR_INTERPRETING_RESPONSE, errorMessage);
        if (strcmp(str,"NONE") != 0)
            sscanf(str,"%ld",StatisticNo);
        else
            *StatisticNo = RSCMU_NONE;
    }

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Set Scope of Measurement
/* Purpose : This function restricts the type of measured values and
/*           determines the number of bursts within a statistics cycle.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_ConfControl (ViSession instrSession,
                                     ViInt32   Object,
                                     ViInt32   Modulation,
                                     ViInt32   MeasuredValuesType,
                                     ViInt32   NbBurst,
                                     ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, MeasuredValuesType,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, NbBurst, 1, 1000, RSCMU_ERROR_PARAMETER3, errorMessage);

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    if (Object == RSCMU_POWMSL)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_05,RSCMU_ERROR_FW_VERSION_3_05));

    sprintf(wrtBuf,"CONF%s%s:CONT %s,%ld", StrObject[Object], StrModulation[Modulation], StrScopeMeas[MeasuredValuesType], NbBurst);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Set Tests Repetition
/* Purpose : This function sets the number of statistics cycles,the stop
/*           condition and the stepping mode for the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_ConfControlRep (ViSession  instrSession,
                                        ViInt32    Object,
                                        ViInt32    Modulation,
                                        ViInt32    Repetition,
                                        ViInt32    StopCondition,
                                        ViInt32    StepMode,
                                        ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, Repetition,0,1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    if (Object == RSCMU_POWMSL)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_05,RSCMU_ERROR_FW_VERSION_3_05));

    if (Object == RSCMU_NPOW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_07,RSCMU_ERROR_FW_VERSION_3_07));

    if (Object == RSCMU_WPOW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    if (Object == RSCMU_SPECMSW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_50,RSCMU_ERROR_FW_VERSION_3_50));

    // Configure  tests cycles:
    sprintf (wrtBuf, "CONF%s%s:CONT:REP %s,%s,%s", StrObject[Object], StrModulation[Modulation], StrRepetition[Repetition], StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Repetition Mode
/* Purpose : This function defines the number of test cycles and, if
/*           required, an abort condition for the measurement.
/*           Note: In the case of READ commands (READ:...), the <Repetition>
/*           parameter has no effect; the measurement is always aborted after
/*           a single shot.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_ConfContRepStat (ViSession  instrSession,
                                         ViInt32    Object,
                                         ViInt32    Modulation,
                                         ViInt32    NbRepetition,
                                         ViInt32    StopCondition,
                                         ViInt32    StepMode,
                                         ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrSession, NbRepetition, RSCMU_MULT_MEAS_MIN, RSCMU_MULT_MEAS_MAX, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StopCondition,0,1, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrSession, StepMode,0,1, RSCMU_ERROR_PARAMETER4, errorMessage);

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    if (Object == RSCMU_WPOW)
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    // Configure  tests cycles:
    sprintf (wrtBuf, "CONF%s%s:CONT:REP %ld,%s,%s", StrObject[Object], StrModulation[Modulation], NbRepetition, StrStopCondition[StopCondition], StrStepMode[StepMode]);

    RETMSG(RSCMU_sendCmd(instrSession, wrtBuf));
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : If used with ON, this function sets all the parameters of the
/*           subsystem to their default values. (OFF has no effect).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_ConfContDef (ViSession instrSession,
                                     ViInt32   Object,
                                     ViInt32   Modulation,
                                     ViInt32   DefaultValues,
                                     ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    if ((Modulation == RSCMU_EPSK)||(Object > RSCMU_POWMPR))
        CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V2_15,RSCMU_ERROR_FW_VERSION_2_15));

    sprintf (wrtBuf, "CONF%s%s:CONT:DEF", StrObject[Object], StrModulation[Modulation]);
    RETMSG(RSCMU_setEnum(instrSession, wrtBuf, DefaultValues, mode_On_Off,
                         RSCMU_ERROR_PARAMETER4));
}

/*===========================================================================*/
/* Function: Measurement scalar for narrow-band power
/* Purpose : This function starts the measurement and outputs the results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_Npow (ViSession  instrSession,
                                      ViInt32    command,
                                      ViReal64  *AvgPowCurr,
                                      ViReal64  *MinPowCurr,
                                      ViReal64  *MaxPowCurr,
                                      ViReal64  *AvgPowAvg,
                                      ViReal64  *MinPowAvg,
                                      ViReal64  *MaxPowAvg,
                                      ViReal64  *Frequency,
                                      ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE], rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;

    sprintf (wrtBuf, "%s:NPOW?", StrCommand[command]);
    CHECKRETMSG(RSCMU_getViString(instrSession,wrtBuf,rdBuf,RD_BUFFER_SIZE,0));

    // Get the Average Power of Current Burst
    GET_VI_REAL64(*AvgPowCurr,rdBuf);
    // Get the Minimum Power of Current Burst
    GET_VI_REAL64(*MinPowCurr,NULL);
    // Get the Maximum Power of Current Burst
    GET_VI_REAL64(*MaxPowCurr,NULL);
    // Get the Average Power of Average Burst
    GET_VI_REAL64(*AvgPowAvg,NULL);
    // Get the Minimum Power of Average Burst
    GET_VI_REAL64(*MinPowAvg,NULL);
    // Get the Maximum Power of Average Burst
    GET_VI_REAL64(*MaxPowAvg,NULL);
    // Get the frequency
    GET_VI_REAL64(*Frequency,NULL);

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Measurement scalar for RF Wide-band power
/* Purpose : This function starts the measurement and outputs the results.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_RF_NSig_Wpow (ViSession  instrSession,
                                      ViInt32    command,
                                      ViReal64  *Power,
                                      ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE], rdBuf[RD_BUFFER_SIZE];
    ViChar*     str;

    CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

    sprintf (wrtBuf, "%s:WPOW?", StrCommand[command]);
    CHECKRETMSG(RSCMU_getViString(instrSession,wrtBuf,rdBuf,RD_BUFFER_SIZE,0));

    // Get the Measured Power Value
    GET_VI_REAL64(*Power,rdBuf);

    return RSCMU_Status;
}

/*###########################################################################*/
/*                                                                           */
/*  VXI PLUG & PLAY COMPATIBILITY FUNCTIONS                                  */
/*                                                                           */
/*  These functions provide a VXIplug&play compatibily of this instrument    */
/*  driver with regard to the following documents:                           */
/*                                                                           */
/*  VPP-3.1: Instrument Drivers                                              */
/*      Architecture and Design                                              */
/*      Specification                                                        */
/*      Revision 4.0                                                         */
/*      February 5, 1996                                                     */
/*                                                                           */
/*  VPP-3.2: Instrument Driver                                               */
/*      Functional Body Specification                                        */
/*      Revision 4.0                                                         */
/*      February 2, 1996                                                     */
/*                                                                           */
/*  By: Jan Sima, ELCOM, a.s., Czech Republic                                */
/*                                                                           */
/*###########################################################################*/
#ifndef _REMOVE_VXIPNP_FUNCTIONS_

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_init (ViRsrc     resourceName,
                              ViBoolean  IDQuery,
                              ViBoolean  resetDevice,
                              ViPSession instrSession)
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViString  p2str;
    ViInt32   board = 0,
              primaryAddr = 0,
              secondaryAddr = 0,
              interface = 0;
    ViChar    errorMessage[WR_BUFFER_SIZE];
    ViSession rmHandle;

    p2str = resourceName;

    /* --- Set standard "C" locale --- */
    {
        char *locale;

        locale = setlocale(LC_ALL, "C");
    }

  if (strncmp (p2str, "GPIB", 4) == 0)
  {
    interface = RSCMU_GPIB;

    p2str += 4;

    if (p2str[0] >= '0' && p2str[0] <= '9')
      board = (p2str++)[0] - '0';

    if (strncmp (p2str, "::", 2) != 0)
      return VI_ERROR_INV_RSRC_NAME;

    if (sscanf (p2str += 2, "%ld", &primaryAddr) != 1)
      return VI_ERROR_INV_RSRC_NAME;

    p2str += (primaryAddr > 9) ? 2 : 1;

    if (strlen (p2str) != 0 && strncmp (p2str, "::", 2) != 0)
      return VI_ERROR_INV_RSRC_NAME;

    if (strlen (p2str) != 0) {
      if (sscanf (p2str += 2, "%ld", &secondaryAddr) != 1)
        return VI_ERROR_INV_RSRC_NAME;

      p2str += (secondaryAddr > 9) ? 2 : 1;

      if (strlen (p2str) != 0 && strcmp (p2str, "::INSTR") != 0)
        return VI_ERROR_INV_RSRC_NAME;
    }
  }
  else
  {
    return VI_ERROR_INV_RSRC_NAME;
  }

  if (secondaryAddr == 0) {
    CHECKRET(RSCMU_Init_Base (instrSession, interface, board, primaryAddr,
                       IDQuery, resetDevice, errorMessage));
  }
  else
  {
    CHECKRET(viOpenDefaultRM (&rmHandle));
  CHECKRET(viOpen (rmHandle, resourceName, VI_NULL, 3000, instrSession));
    CHECKRET(viGpibControlREN (*instrSession, VI_GPIB_REN_ASSERT));
  }

    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Init RF Non Signalling                                          */
/* Purpose:  This function initializes the RF non signalling module's        */
/*           global data structures. This function is used in VEE where this */
/*           can not be a part of the VXIpnp init function.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_initRFNSig (ViSession RFNSigSession,
                                    ViSession BaseSession)
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViSession   globalSession = 0;
  ViInt32     secondAddr = RSCMU_SECONDARY_ADDRESS_INVALID, comp = -1;
  ViReal64    Flag = RSCMU_OFF;
  struct RSCMU_Base_Globals *p_globalvarBase;
  struct RSCMU_RF_NSig_Globals *p_globalvarRF;
  ViChar    errorMessage[WR_BUFFER_SIZE];

    /* --- Set standard "C" locale --- */
    {
        char *locale;

        locale = setlocale(LC_ALL, "C");
    }

  // Init the Error Message
  strcpy (errorMessage, "No Error");

  // Get the current options (= Function Group Names) in the CMU.
  if ((RSCMU_Status = viGetAttribute (BaseSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Base_errorMessage (BaseSession, RSCMU_Status, errorMessage);

  //Find the Secondary address of function Group 'RF_NSig'
  while ((comp != 0) && (secondAddr <= RSCMU_SECONDARY_ADDRESS_MAX))
  {
    secondAddr++;
    comp = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp, "\"RF_NSig\"", 9);
    if (comp == 0)
    {
      Flag = RSCMU_ON;
    }
  }

  // If not found try to assign or error
  if (Flag == RSCMU_OFF)
      return RSCMU_Base_errorMessage (BaseSession, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);

  // Allocate space for "struct RSCMU_RF_NSig_Globals"
  if ((p_globalvarRF = (struct RSCMU_RF_NSig_Globals *)malloc(sizeof(struct RSCMU_RF_NSig_Globals))) == 0)
    return RSCMU_initCleanUp (globalSession, RFNSigSession, RSCMU_Status);

  memset ( p_globalvarRF, 0, sizeof ( struct RSCMU_RF_NSig_Globals ) );
  if ((RSCMU_Status = viSetAttribute(RFNSigSession, VI_ATTR_USER_DATA,(ViAttrState)p_globalvarRF)) < 0)
  {
    free( p_globalvarRF );
    return RSCMU_initCleanUp (globalSession, RFNSigSession, RSCMU_Status);
  }

  memcpy(&p_globalvarRF->RSCMU_tGlobal,&p_globalvarBase->RSCMU_tGlobal, sizeof (struct RSCMU_SESSION_Globals));

  // Set initial values of variables for function group 'RF_NSig'.
  strcpy (p_globalvarRF->RSCMU_tGlobal.FGrp_Name, "RF_NSig");
  strcpy (p_globalvarRF->RSCMU_tGlobal.RSCMU_SCPIConnection, p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);
  p_globalvarRF->RSCMU_tGlobal.RSCMU_primary_address = p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address;
  p_globalvarRF->RSCMU_tGlobal.RSCMU_secondary_address = secondAddr;
  p_globalvarRF->RSCMU_tGlobal.RSCMU_TRACE = p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE;
  p_globalvarRF->RSCMU_tGlobal.RSCMU_SessionNb = RFNSigSession;

  if (p_globalvarRF->RSCMU_tGlobal.RSCMU_TRACE == RSCMU_ON)
    p_globalvarRF->RSCMU_tGlobal.RSCMU_AddrTargetFile = p_globalvarBase->RSCMU_tGlobal.RSCMU_AddrTargetFile;

  // Record the execution of this Init Function.
  p_globalvarBase->RSCMU_tSecAddr[secondAddr].SessionNb = RFNSigSession;

  // Timeout set to RSCMU_TMO_VALUE defined in rscmu200.h (10 s)
  CHECKRET(viSetAttribute (RFNSigSession, VI_ATTR_TMO_VALUE, RSCMU_TMO_VALUE));
  CHECKRET(RSCMU_setCheckStatus(RFNSigSession, VI_TRUE));

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Init Audio Non Signalling                                       */
/* Purpose:  This function initializes the Audio non signalling module's     */
/*           global data structures. This function is used in VEE where this */
/*           can not be a part of the VXIpnp init function.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_initAudioNSig (ViSession AudioNSigSession,
                                   ViSession BaseSession)
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViSession   rmSession = 0; // Resource Manager Session
  ViInt32     secondAddr = RSCMU_SECONDARY_ADDRESS_INVALID, comp = -1;
  ViBoolean Flag = RSCMU_OFF;
  struct RSCMU_Base_Globals        *p_globalvarBase;
  struct RSCMU_Audio_NSig_Globals  *p_globalvarAudioNSig;
  ViChar    errorMessage[WR_BUFFER_SIZE];

    /* --- Set standard "C" locale --- */
    {
        char *locale;

        locale = setlocale(LC_ALL, "C");
    }

  // Reset the Error Message
  strcpy (errorMessage, "No Error");

  // Queries the specified attribute for the Base object:
  if ((RSCMU_Status = viGetAttribute (BaseSession, VI_ATTR_USER_DATA,(ViAddr) &p_globalvarBase)) < 0)
    return RSCMU_Base_errorMessage (BaseSession, RSCMU_Status, errorMessage);

  // Find the secondary address for the Audio Function Group:
  while ((comp != 0) && (secondAddr <= RSCMU_SECONDARY_ADDRESS_MAX))
  {
    secondAddr++;
    comp = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp, "\"AUDIO_NSig\"", 12);
    if (comp == 0)
    {
      Flag = RSCMU_ON;
    }
  }

  // If not found try to assign or error
  if (Flag == RSCMU_OFF)
      return RSCMU_Base_errorMessage (BaseSession, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);

  // Allocate space for the RSCMU_Audio_NSig_Globals object:
  if ((p_globalvarAudioNSig = (struct RSCMU_Audio_NSig_Globals *)malloc(sizeof(struct RSCMU_Audio_NSig_Globals))) == 0)
    return RSCMU_initCleanUp (rmSession, AudioNSigSession, RSCMU_Status);

  memset ( p_globalvarAudioNSig, 0, sizeof ( struct RSCMU_Audio_NSig_Globals ) );
  if ((RSCMU_Status = viSetAttribute(AudioNSigSession, VI_ATTR_USER_DATA,(ViAttrState)p_globalvarAudioNSig)) < 0)
  {
    free( p_globalvarAudioNSig );
    return RSCMU_initCleanUp (rmSession, AudioNSigSession, RSCMU_Status);
  }

  memcpy(&p_globalvarAudioNSig->RSCMU_tGlobal,&p_globalvarBase->RSCMU_tGlobal, sizeof (struct RSCMU_SESSION_Globals));
  // Set values of variables for the Audio Function Group:
  strcpy (p_globalvarAudioNSig->RSCMU_tGlobal.FGrp_Name, "AUDIO_NSig");
  strcpy (p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_SCPIConnection, p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_primary_address = p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address;
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_secondary_address = secondAddr;
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_TRACE = p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE;
  p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_SessionNb = AudioNSigSession;

  if (p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_TRACE == RSCMU_ON)
    p_globalvarAudioNSig->RSCMU_tGlobal.RSCMU_AddrTargetFile = p_globalvarBase->RSCMU_tGlobal.RSCMU_AddrTargetFile;

  // Record the open session:
  p_globalvarBase->RSCMU_tSecAddr[secondAddr].SessionNb = AudioNSigSession;

  // Timeout set to RSCMU_TMO_VALUE defined in rscmu200.h (10 s)
  CHECKRET(viSetAttribute (AudioNSigSession, VI_ATTR_TMO_VALUE, RSCMU_TMO_VALUE));

  CHECKRET(RSCMU_setCheckStatus(AudioNSigSession, VI_TRUE));
  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Get Instrument Handle                                           */
/* Purpose:  This function passes the instrument handle from the first       */
/*           parameter to the second parameter. This function is used in VEE */
/*           where it allows to get the current instrument handle.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetInstrumentHandle (ViSession instrSession,
                                             ViSession *instrSessionOut)
{
    *instrSessionOut = instrSession;
    return 0;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_reset (ViSession instrSession)
{
    ViChar      errorMessage[WR_BUFFER_SIZE];

    return RSCMU_Reset (instrSession, errorMessage);
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_self_test (ViSession instrSession,
                                    ViPInt16 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    return VI_WARN_NSUP_SELF_TEST;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    return RSCMU_ErrorQuery (instrSession, errorCode, errorMessage);
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    return RSCMU_Base_errorMessage (instrSession, statusCode, errMessage);
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViChar  errorMessage[WR_BUFFER_SIZE];

    return RSCMU_Base_RevisionQuery (instrSession, driverRev, instrRev, errorMessage);
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_close (ViSession instrSession)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    ViChar  errorMessage[WR_BUFFER_SIZE];
    ViUInt16 secAddr;

    CHECKRET(viGetAttribute (instrSession, VI_ATTR_GPIB_SECONDARY_ADDR, &secAddr));

    if (secAddr == 0) {
        CHECKRET(RSCMU_Close_Base (instrSession, errorMessage));
    } else {
        struct RSCMU_RF_NSig_Globals *p_globalvar;

        CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvar));

        // Deallocate the space pointed by p_globalvar
        if (p_globalvar)
            free (p_globalvar);

        // Deallocate all system resources
        CHECKRET(viClose (instrSession));

    }
    return RSCMU_Status;
}

#endif

/*###########################################################################*/
/*                                                                           */
/*  UNSUPPORTED FUNCTIONS                                                    */
/*                                                                           */
/*  These functions provide backwards compatibility with old driver versions */
/*  and are no longer supported.                                             */
/*                                                                           */
/*###########################################################################*/
/*=========================================================================*/
/* Function: RSCMU_Query
/* Purpose : Writes command string to the CMU and reads the resulting output,
/*           this write/read function is intended for use with commands that
/*           generate a response from the instrument.
/*           The operation first: writes the command string designated by the
/*           RSCMU_Write function to the instrument; then read the data from
/*           the instrument's response by the RSCMU_Read function.
/*=========================================================================*/
ViStatus _VI_FUNC  RSCMU_Query (ViSession  instrSession,
                                ViChar     wrtBuf[],
                                ViChar     rdBuf[],
                                ViBoolean  STBQuery,
                                ViChar     errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViUInt16    stb;

    CHECKRETMSG(RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));

    if (strstr(wrtBuf,"?"))
    {
        CHECKRETMSG(RSCMU_viRead(instrSession,rdBuf,RD_BUFFER_SIZE));
        //rdBuf[strlen(rdBuf) - 1] = '\0';
    }

    if (STBQuery)
    {
        RSCMU_Status = RSCMU_sysStatus(instrSession, &stb);
//      sprintf(rdBuf + strlen(rdBuf),";%hu", stb);
    }

/*
  // Write data to the CMU
  CHECKRETMSG(RSCMU_Write (instrSession, wrtBuf, STBQuery, strlen(wrtBuf), errorMessage));

  // Read the resulting output
  CHECKRETMSG(RSCMU_Read (instrSession, rdBuf, RD_BUFFER_SIZE, errorMessage));
*/

// V3.08

  return RSCMU_Status;
}

/*=========================================================================*/
/* Function: RSCMU_Query_Array
/* Purpose : Writes command string to the CMU and reads the resulting output,
/*           this write/read function is intended for use with commands that
/*           generate a response from the instrument.
/*           The operation first: writes the command string designated by the
/*           RSCMU_Write function to the instrument; then read the array data
/*           from the instrument's response by the RSCMU_Read function.
/*========================================================================*/
ViStatus _VI_FUNC RSCMU_Query_Array (ViSession instrSession,
                                     ViChar    wrtBuf[],
                                     ViChar    rdBuf[],
                                     ViBoolean STBQuery,
                                     ViChar    errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViUInt16    stb;

    CHECKRETMSG(RSCMU_viWrite(instrSession, wrtBuf, strlen(wrtBuf)));
 /*  JMG 030110
    if (strstr(wrtBuf,"?"))
    {
        RSCMU_Status = RSCMU_ReadDataUnknownLength(instrSession,&hugeBuf,&count);
        if (hugeBuf && RSCMU_Status >= 0)
        {
            // patch by as 14.02.03
            count++;  // one bype longer for the strin terminating \0
            // patch  as end
            memcpy (rdBuf, hugeBuf, count < RD_BUFFER_SIZE_ARRAY ? count : RD_BUFFER_SIZE_ARRAY);
        }
        if (hugeBuf) free ( hugeBuf );

        if (RSCMU_Status < 0)
            return RSCMU_Base_errorMessage (instrSession,RSCMU_Status,errorMessage);

        /* remove \n at the end of the string /

        rdBuf[strlen(rdBuf) - 1] = '\0';
    }
*/
    if (strstr(wrtBuf,"?"))
    {
        CHECKRETMSG(RSCMU_viRead(instrSession,rdBuf,RD_BUFFER_SIZE_ARRAY));
        /* remove \n at the end of the string */
//        rdBuf[strlen(rdBuf) - 1] = '\0';
    }

    if (STBQuery)
    {
        RSCMU_Status = RSCMU_sysStatus(instrSession, &stb);
//      sprintf(rdBuf + strlen(rdBuf),";%hu", stb);
    }

  return RSCMU_Status;
}

/*=========================================================================*/
/* Function: RSCMU_WriteToRSCMU
/* Purpose : This function send the specified data string to the instrument.
/*=========================================================================*/
ViStatus _VI_FUNC RSCMU_WriteToRSCMU (ViSession  instrSession,
                                      ViChar     wrtBuf[],
                                      ViChar     errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Write data to the CMU
  if ((RSCMU_Status = RSCMU_Write (instrSession, wrtBuf, RSCMU_STB_ON, strlen(wrtBuf), errorMessage)) < 0)
    return RSCMU_Base_errorMessage (instrSession, RSCMU_Status, errorMessage);

  return RSCMU_Status;
}

/*=========================================================================*/
/* Function: RSCMU_ReadFromRSCMU
/* Purpose : This function reads the instrument read buffer.
/*=========================================================================*/
ViStatus _VI_FUNC  RSCMU_ReadFromRSCMU (ViSession  instrSession,
                                        ViChar     rdBuf[],
                                        ViChar     errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;

  // Read the CMU read-buffer
  if ((RSCMU_Status = RSCMU_Read (instrSession, rdBuf, RD_BUFFER_SIZE, errorMessage)) < 0)
    return RSCMU_Base_errorMessage (instrSession, RSCMU_Status, errorMessage);

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_SepBufSTB
/* Purpose : this function extract from 'Buffer' the response to *STB? and
/*           returns it.
/*===========================================================================*/
static ViStatus _VI_FUNC RSCMU_SepBufSTB  (ViSession  instrSession,
                                            ViChar     Buffer[],
                                            ViChar     BufSTB[])
{
  ViStatus      RSCMU_Status = VI_SUCCESS;
  ViInt16       count = 0;
  ViChar        *adr;
  ViChar        *prevAdr;

  adr = strtok(Buffer, ";");
  prevAdr = adr;

  while (adr)
  {
    prevAdr = adr;
    adr = strtok (NULL, ";");
    count++;
  }
  if(count)
    strcpy (BufSTB, prevAdr);

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_Write
/* Purpose : Add to the user's command the request ;*STB? to see if an error
/*           occurs. It puts the resulted command in a WinBatch file to
/*           record the User's scenario. (= TRACE)
/*           This function is called instead of viWrite. (everytime we
/*           need it.)
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_Write (ViSession  instrSession,
                               ViChar     writeBuf[],
                               ViBoolean  QuerySTB,
                               ViUInt32   cnt,
                               ViChar _VI_FAR errorMessage[])
{
  ViStatus  RSCMU_Status = VI_SUCCESS;
  ViChar    STBWriteBuf [WR_BUFFER_SIZE] = "";
  ViChar    COMWriteBuf [WR_BUFFER_SIZE] = "";
  ViChar    traceWriteBuf [WR_BUFFER_SIZE] = "";
  ViChar    tempWriteBuf [WR_BUFFER_SIZE] = "";
  ViChar    FGrpBuf[WR_BUFFER_SIZE] = "";
  ViUInt32  rtnCnt;
  static    ViInt32 Previous_Sec_Address = -1;
  size_t    FGrp_name_length;
//  FILE     *targetFile;

  struct RSCMU_SESSION_Globals  *p_globalvarSession;

  // Get the Globals
  if ((RSCMU_Status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession)) < 0)
    return RSCMU_Status;

  if (strncmp ("ASRL", p_globalvarSession->RSCMU_SCPIConnection, 4) == 0)
  {
    if (Previous_Sec_Address != p_globalvarSession->RSCMU_secondary_address)
    {
      sprintf (COMWriteBuf, "*SEC %ld\r\n", p_globalvarSession->RSCMU_secondary_address);
      if ((RSCMU_Status = viWrite (instrSession, (ViBuf) STBWriteBuf, cnt, &rtnCnt)) < 0)
        return RSCMU_Base_errorMessage (instrSession, RSCMU_Status, errorMessage);
      Previous_Sec_Address = p_globalvarSession->RSCMU_secondary_address;
    }
  }
  else
    sprintf (COMWriteBuf, "%s", "");

  // Check if there is a query *STB?
  if (QuerySTB == RSCMU_STB_ON) // if *STB? is asked ...
  {
    // ... add ";*STB?" and the characters EndOfChar and EndOfLine.
    sprintf (STBWriteBuf, "%s%s;*STB?\r\n", COMWriteBuf, writeBuf);
    //cnt = cnt + 7; // missed the last character
    cnt = cnt + 8 + strlen (COMWriteBuf);

/*===========================================================================*/
/*  new Error QueryHandling: Using the last bit as *STB ON/OFF Flag
/*                           (last bit is only flag for RSCMU_Write/_Read)
/*                           1 -> 0000 0000 0000 0001  STB On
/*===========================================================================*/
    p_globalvarSession->RSCMU_Stb = p_globalvarSession->RSCMU_Stb | 1;
  }
  else
  {
    // ... else add EndOfChar and EndOfLine at writeBuf.
    sprintf (STBWriteBuf, "%s%s\r\n", COMWriteBuf, writeBuf);
    cnt = cnt + 2 + strlen (COMWriteBuf);

/*===========================================================================*/
/*  new Error QueryHandling: Using the last bit as *STB ON/OFF Flag
/*                           (last bit is only flag for RSCMU_Write/_Read)
/*                           65534 -> 1111 1111 1111 1110  STB Off
/*===========================================================================*/
    p_globalvarSession->RSCMU_Stb = p_globalvarSession->RSCMU_Stb & 65534;
  }

  // Call to the function which leave the command in the SessionTraceFile
  if (p_globalvarSession->RSCMU_TRACE)
  {
    // To write the command in a WinBatch File, we need the FGrp_Name.
    FGrp_name_length = strlen(p_globalvarSession->FGrp_Name);

    // Extract the quotes given by the CMU before and after the FGrp_Name.
    if (p_globalvarSession->FGrp_Name[0] == 34)    // ASCII code for ".
    {
      RSCMU_copyString (FGrpBuf, p_globalvarSession->FGrp_Name, 1, FGrp_name_length-2);
      FGrp_name_length = FGrp_name_length - 2;
    }
    else
      strncpy (FGrpBuf, p_globalvarSession->FGrp_Name, FGrp_name_length);

    // extract '\r\n' at the end
    strncpy (tempWriteBuf, STBWriteBuf, strlen (STBWriteBuf) - 2);
    // modify the string to have a good WinBatch file format.
    sprintf (traceWriteBuf, "\n%s: %s\n", FGrpBuf, tempWriteBuf);

    // Open a Winbatch file and leave the current remote command
    //targetFile = p_globalvarSession->RSCMU_AddrTargetFile;

   if ((RSCMU_Status = RSCMU_WriteFile (instrSession, p_globalvarSession->RSCMU_AddrTargetFile, traceWriteBuf, errorMessage)) < 0)
   return RSCMU_Status;
  }

  if ((RSCMU_Status = viWrite (instrSession, (ViBuf) STBWriteBuf, cnt, &rtnCnt)) < 0)
    return RSCMU_Base_errorMessage (instrSession, RSCMU_Status, errorMessage);

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_Read
/* Purpose : This function reads the response given by the CMU after a call
/*           at RSCMU_Write ("user's command; *STB?")
/*           It puts this response in a WinBatch file to record the User's
/*           scenario, and returns the statusbyte. The statusbyte is then
/*           evaluated by a call to "RSCMU_register"
/*           This function is called instead of viRead. (everytime we
/*           need it.)
/*===========================================================================*/
ViStatus _VI_FUNC  RSCMU_Read  (ViSession  instrSession,
                               ViChar     rdBuf[],
                               ViUInt32   cnt,
                               ViChar     _VI_FAR errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  size_t      posPtVirg, lengthBuffer, FGrp_name_length ;
  ViChar      TmpBuf [RD_BUFFER_SIZE_ARRAY] = "";
  ViChar      STBBuf [512] = "";
  ViChar      FGrpBuf[512] = "";
  ViUInt32    rtnCnt;
  ViInt32     reg_STB, errorCode;
  ViUInt16    i_temp;
//  FILE        *targetFile;

  struct RSCMU_SESSION_Globals *p_globalvarSession;

  // Get the Globals
  CHECKRET(viGetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAddr) &p_globalvarSession));

  // Read data from the CMU. The data read is stored in the specified rdBuffer.
  if ((RSCMU_Status = viRead (instrSession, (ViPBuf) rdBuf, cnt, &rtnCnt)) < 0)
    return RSCMU_Base_errorMessage (instrSession, RSCMU_Status, errorMessage);

  // \0 added at the end of the data read buffer from the instrument.
  rdBuf[rtnCnt-1]='\0';

  // Get the length of the read buffer:
  lengthBuffer = strlen (rdBuf);

  // Locate the position of the semicolon character in the read buffer:
  posPtVirg = strcspn (rdBuf, ";");

  // Call to the function which leaves the command string in the SessionTraceFile
  if (p_globalvarSession->RSCMU_TRACE)
  {
    FGrp_name_length = strlen(p_globalvarSession->FGrp_Name);

    // Extract the quotes given by the CMU before and after the FGrp_Name.
    if (p_globalvarSession->FGrp_Name[0] == 34)  // ASCII code for '"'.
    {
      RSCMU_copyString (FGrpBuf, p_globalvarSession->FGrp_Name, 1, FGrp_name_length-2);
      FGrp_name_length = FGrp_name_length - 2;
    }
    else
      strncpy (FGrpBuf, p_globalvarSession->FGrp_Name, FGrp_name_length);

    // Modify the string to have a WinBatch file format output:
    sprintf (TmpBuf, ";%s: %s", FGrpBuf, rdBuf);

    // Open a Winbatch file and leave the current response of CMU.
    //targetFile = p_globalvarSession->RSCMU_AddrTargetFile;

    CHECKRET(RSCMU_WriteFile (instrSession, p_globalvarSession->RSCMU_AddrTargetFile, TmpBuf, errorMessage));
  }

  // If the Status Byte register is required, extract the returned value from the RdBuf:
  if (posPtVirg != lengthBuffer)
  {

/*===========================================================================*/
/*  new Error QueryHandling: Test if STB On
/*===========================================================================*/
    i_temp= p_globalvarSession->RSCMU_Stb;
    i_temp = i_temp & 1;
    if (i_temp)
    // Extract the response at *STB?
    {
      if ((RSCMU_Status = RSCMU_SepBufSTB (instrSession, rdBuf, STBBuf))<0)
      {

/*===========================================================================*/
/*  new Error QueryHandling: Using the last bit as *STB ON/OFF Flag
/*                           (last bit is only flag for RSCMU_Write/_Read)
/*                           65534 -> 1111 1111 1111 1110  STB Off
/*===========================================================================*/
    p_globalvarSession->RSCMU_Stb = p_globalvarSession->RSCMU_Stb & 65534;
        return RSCMU_Status;
      }
    }
  }
  else
  {
    // The rdBuffer is the "response" at  a configuration with *STB?...
    strcpy (STBBuf, rdBuf);
  }
/*===========================================================================*/
/*  new Error QueryHandling: Test if STB On
/*===========================================================================*/
    i_temp= p_globalvarSession->RSCMU_Stb;
    i_temp = i_temp & 1;
    if (i_temp)
  {

/*===========================================================================*/
/*  new Error QueryHandling: Using the last bit as *STB ON/OFF Flag
/*                           (last bit is only flag for RSCMU_Write/_Read)
/*                           65534 -> 1111 1111 1111 1110  STB Off
/*===========================================================================*/
    p_globalvarSession->RSCMU_Stb = p_globalvarSession->RSCMU_Stb & 65534;
    // Check if there is an error
    reg_STB = strtoul(STBBuf, NULL, 0);
    if ((reg_STB != 0) && (reg_STB != 16))
      CHECKRET(RSCMU_ErrorQuery (instrSession, &errorCode, errorMessage));
  }

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Separate Data Values
/* Purpose : This function splits multiple data in a string into a table of
/*           data values, which are separated by one specified delimiter
/*           character.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_SeparateDataValues (ViChar     StringToSeparate[],
                                           ViChar     Delimiters[],
                                           ViChar     TableOfDataValues[100][100])
{
  ViStatus      RSCMU_Status = VI_SUCCESS;
  ViChar       *DataValue;
  ViInt16       Index = 0;

  DataValue = strtok (StringToSeparate, Delimiters);

  while (DataValue)
  {
    strcpy (TableOfDataValues[Index++], DataValue);
    DataValue = strtok (NULL, Delimiters);
  }

  while (Index <100)
    strcpy (TableOfDataValues[Index++], "");

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_ReturnCMUResponse
/* Purpose : This function is call to return the values of CMU respone
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_ReturnCMUResponse (ViInt32  count,
                                          ViChar   valuesTmp[669][20],
                                          ViReal64 values[])
{
  ViStatus      RSCMU_Status = VI_SUCCESS;
  ViInt32 i;

  for (i=0; i < (count); i++)
  {
    if (strncmp (valuesTmp[i], "NAN",3) != 0)
      sscanf (valuesTmp[i], "%le", &values[i]);
    else  values[i] = RSCMU_NAN;
  }

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function:
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetViInt32Value (ViChar    TableOfDataValues[100][100],
                                         ViInt32   Index,
                                         ViInt32   *dValue)
{
  ViReal64  lfValue;

  if (strncmp (TableOfDataValues[Index], "NAN",3) != 0)
  {
    if (sscanf (TableOfDataValues[Index], "%le", &lfValue) < 0)
      return RSCMU_ERROR_EOF;

    lfValue += 0.001; // To avoid a problem due to the fraction part
    *dValue = (ViInt32) lfValue;
  }
  else
    *dValue = RSCMU_NAN_VI_INT32;

  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: RSCMU_GetViReal64Value
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetViReal64Value (ViChar     TableOfDataValues[100][100],
                                          ViInt32    Index,
                                          ViReal64   *Value)
{
  if (strncmp (TableOfDataValues[Index], "NAN", 3) != 0)
  {
    if (sscanf (TableOfDataValues[Index], "%le", Value) < 0)
      return RSCMU_ERROR_EOF;
  }
  else
    *Value = RSCMU_NAN;

  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: RSCMU_GetLimitCheckValue
/* Purpose :
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetLimitCheckValue (ViChar     TableOfDataValues[100][100],
                                            ViInt32    Index,
                                            ViInt32    *LimitCheckValue)

{
  ViStatus      RSCMU_Status = VI_SUCCESS;

  if (strcmp (TableOfDataValues[Index], "NMAU") == 0)
    *LimitCheckValue = RSCMU_NMAU;
  else if (strcmp (TableOfDataValues[Index], "NMAL") == 0)
    *LimitCheckValue = RSCMU_NMAL;
  else if (strcmp (TableOfDataValues[Index], "INV") == 0)
    *LimitCheckValue = RSCMU_INV;
  else if (strcmp (TableOfDataValues[Index], "OK") == 0)
    *LimitCheckValue = RSCMU_OK;
  else
    return RSCMU_ERROR_INVALID_LIMIT_CHECK;

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Measurement Status
/* Purpose : This function is call to get the status of the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetMeasStatVal (ViInt32 *MeasurementStatus,
                                        ViChar   TableOfDataValues[100][100],
                                        ViInt32  Index)
{
  if (strncmp ("OFF", TableOfDataValues[Index], 3) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_OFF;
  else if (strncmp ("RUN",  TableOfDataValues[Index], 3) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_RUN;
  else if (strncmp ("STOP", TableOfDataValues[Index], 4) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_STOP;
  else if (strncmp ("ERR",  TableOfDataValues[Index], 3) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_ERR;
  else if (strncmp ("STEP", TableOfDataValues[Index], 4) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_STEP;
  else if (strncmp ("RDY",  TableOfDataValues[Index], 3) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_RDY;
  else
    return RSCMU_ERROR_MEAS_STAT_UNKNOWN;

  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Sensitivity Measurement Status
/* Purpose : This function is call to get the status of the sensitivity
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetSensMeasStatVal (ViInt32 *MeasurementStatus,
                                            ViChar   TableOfDataValues[100][100],
                                            ViInt32  Index)
{
  if (strncmp ("PASS", TableOfDataValues[Index], 4) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_PASS;
  else if (strncmp ("FAIL", TableOfDataValues[Index], 4) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_FAIL;
  else if (strncmp ("INV",  TableOfDataValues[Index], 3) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_INV;
  else if (strncmp ("TMER", TableOfDataValues[Index], 4) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_TMER;
  else if (strncmp ("TLOW", TableOfDataValues[Index], 4) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_TMER;  // keep compatibility with FW < 3.05
  else if (strncmp ("IMP",  TableOfDataValues[Index], 3) == 0)
    *MeasurementStatus = RSCMU_MEAS_STAT_IMP;
  else
    return RSCMU_ERROR_MEAS_STAT_UNKNOWN;

  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Counting No
/* Purpose : This function is call to get the number of the current statistics
/*           cycle.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetCountingNo (ViInt32 *CountingNo,
                                      ViChar   TableOfDataValues[100][100],
                                      ViInt32  Index)
{
  // Get the number of the current statistics cycle:
  if (strcmp (TableOfDataValues[Index], "NONE") != 0)
    sscanf (TableOfDataValues[Index], "%ld", CountingNo);
  else
    *CountingNo = RSCMU_NONE;

  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Statistic No
/* Purpose : This function is call to get the number of the current evaluation
/*           period.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetStatisticNo (ViInt32  *StatisticNo,
                                        ViChar    TableOfDataValues[100][100],
                                        ViInt32   Index)
{
  // Get the counter for current evaluation period within a cycle
  if (strcmp (TableOfDataValues[Index], "NONE") != 0)
    sscanf (TableOfDataValues[Index], "%ld", StatisticNo);
  else
    *StatisticNo = RSCMU_NONE;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: RSCMU_GetMatchingValue
/* Purpose : This function returns the value matching.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_GetMatchingValue (ViInt32 *Matching,
                                         ViChar   tab[100][100],
                                         ViInt32  Index)
{
  ViStatus      RSCMU_Status = VI_SUCCESS;

  // For All Measured Values
  if (strncmp ("NMAU", tab[Index], 4) == 0)
    *Matching = RSCMU_NMAU;
  else if (strncmp ("NAN", tab[Index], 3) == 0)
    *Matching = RSCMU_NAN_VI_INT32;
  else if (strncmp ("NMAL", tab[Index], 4) == 0)
    *Matching = RSCMU_NMAL;
  else if (strncmp ("INV", tab[Index], 3) == 0)
    *Matching = RSCMU_INV;
  else if (strncmp ("OK", tab[Index], 2) == 0)
    *Matching = RSCMU_OK;
  // The following messages may be output for the value Matching:
  else if (strncmp ("MATC", tab[Index], 4) == 0)
    *Matching = RSCMU_MATCH_MATC;
  else if (strncmp ("NMAT", tab[Index], 4) == 0)
    *Matching = RSCMU_MATCH_NMAT;
  else if (strncmp ("INV", tab[Index], 3) == 0)
    *Matching = RSCMU_MATCH_INV;
  else if (strncmp ("NTSC", tab[Index], 4) == 0)
    *Matching = RSCMU_MATCH_NTSC;
  else if (strncmp ("OUT", tab[Index], 3) == 0)
    *Matching = RSCMU_MATCH_OUT;
  else if (strncmp ("NTR", tab[Index], 3) == 0)
    *Matching = RSCMU_MATCH_NTR;
  else if (strncmp ("BNF", tab[Index], 3) == 0)
    *Matching = RSCMU_MATCH_BNF;
  else if (strncmp ("OFF", tab[Index], 3) == 0)
    *Matching = RSCMU_MATCH_OFF;
  else if (strncmp ("NRAM", tab[Index], 4) == 0)
    *Matching = RSCMU_MATCH_NRAM;
  else if (strncmp ("OFLW", tab[Index], 4) == 0)
    *Matching = RSCMU_MATCH_OFLOW;
  else if (strncmp ("UFLW", tab[Index], 4) == 0)
    *Matching = RSCMU_MATCH_UFLOW;

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: RSCMU_GetGenStatVal
/* Purpose : This function returns the generator status integer value.
/*===========================================================================*/
ViInt32 _VI_FUNC RSCMU_GetGenStatVal (ViChar _VI_FAR  GeneratorStatus[])
{
  // Return the generator status integer value:
  if (strncmp ("OFF", GeneratorStatus, 3) == 0)
    return RSCMU_GEN_STAT_OFF; // Generator is in the OFF state
  else if (strncmp ("RUN", GeneratorStatus, 3) == 0)
    return RSCMU_GEN_STAT_RUN; // Generator is running
  else if (strncmp ("ERR", GeneratorStatus, 3) == 0)
    return RSCMU_GEN_STAT_ERR; // Generator is in the OFF state because it could
                               // not be started.
  // Here, generator status unknown:
  return -1;
}

/*===========================================================================*/
/* Function: RSCMU_scanStringBurst
/* Purpose : This function is call to extract Values of Bursts...
/*           It breaks a string into tokens, which are separated
/*           by a comma.
/*           The resulted strings are putted in 'tabResponse'.
/*===========================================================================*/
ViInt32 _VI_FUNC RSCMU_scanStringBurst (ViChar Buffer[],
                                        ViChar tabResponse[669][20])
{
  ViInt16       count = 0;
  ViChar        *adr;

  adr = strtok(Buffer, ",");

  while (adr)
  {
    strcpy (tabResponse[count], adr);
    adr = strtok (NULL, ",");
    count++;
  }
  return count;
}

/*===========================================================================*/
/* Function: RSCMU_copyString
/* Purpose : Copy "Maximum_Bytes" Bytes from a Source_String (and from a
/*             Source_Index) to a Target_String (from the Index 0).
/*===========================================================================*/
ViStatus _VI_FUNC RSCMU_copyString  (ViChar   Target_String[],
                                     ViChar   Source_String[],
                                     ViUInt32 Source_Index,
                                     ViUInt32 Maximum_Bytes)
{
  ViStatus  RSCMU_Status = VI_SUCCESS;
  ViUInt16 i;

  for (i=0 ; i < (Maximum_Bytes); i++)
    Target_String[i] = Source_String[Source_Index+i];

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Default Settings
/* Purpose : this function is called by the global functions for the default
/*           settings.
/*===========================================================================*/
ViStatus RSCMU_RF_DefaultSettings (ViSession  instrSession,
                                   ViChar    *StrObject,
                                   ViInt32    DefaultValue,
                                   ViStatus   StatusCode,
                                   ViChar     errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_BUFFER_SIZE]= "";

  //Versions check
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

  // Check if the function is correctly used:
  CHECK_VI_BOOLEAN_RANGE(instrSession, (ViBoolean)DefaultValue, StatusCode, errorMessage);

  // Sets the default value for the string command
  sprintf (wrtBuf, "DEF:%s %s", StrObject, mode_On_Off[DefaultValue]);
  if ((RSCMU_Status = RSCMU_Query (instrSession, wrtBuf, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
    return RSCMU_Status;

  return RSCMU_Status;
}

/*===========================================================================*/
/* Function: Burst SubRanges
/* Purpose : This function configurates the READ..., FETCh... and
/*           SAMPle:SUBArrays:... commands.
/*           If the requested number of results exceeds the measurement
/*           range, the count parameter will be corrected by the system.
/*===========================================================================*/

ViStatus _VI_FUNC RSCMU_RF_ConfSubarrays (ViSession instrSession,
                                          ViInt32   Object,
                                          ViInt32   Statistics,
                                          ViReal64  FirstMeasurementFreq,
                                          ViInt32   MeasureResultsNumber,
                                          ViChar    errorMessage[])
{
  ViStatus    RSCMU_Status = VI_SUCCESS;
  ViChar      wrtBuf[WR_BUFFER_SIZE] = "", rdBuf[RD_BUFFER_SIZE]= "";

  //Versions check
  CHECKRETMSG(RSCMU_checkVersion(instrSession,RSCMU200_V3_10,RSCMU_ERROR_FW_VERSION_3_10));

  // Check if the function is correctly used:
  CHECK_VI_INT32_RANGE(instrSession, Statistics, 0, 4, RSCMU_ERROR_PARAMETER2, errorMessage);

  sprintf (wrtBuf, "CONF:SUB%s %s,%.12f,%ld", StrObject[Object], StrStat[Statistics], FirstMeasurementFreq,MeasureResultsNumber);

  if ((RSCMU_Status = RSCMU_Query (instrSession, wrtBuf, rdBuf, RSCMU_STB_ON, errorMessage)) < 0)
    return RSCMU_Status;

  return RSCMU_Status;
}

ViStatus _VI_FUNC RSCMU_FwVersionStringToIntegers(ViString versionString, ViInt32* fgVersion, ViInt32* baseVersion)
    {

    ViStatus   RSCMU_Status = VI_SUCCESS;
    ViChar*    fgVersionString;
    ViChar     fgVersionStringCp[RD_BUFFER_SIZE];
    ViChar*    baseVersionString;
    ViChar     baseVersionStringCp[RD_BUFFER_SIZE];
    ViInt32    mainVers                          = 0;
    ViInt32    subVers                           = 0;
    ViInt32    letterVers1                       = 0;
    ViInt32    letterVers2                       = 0;

    unsigned int i=0;
    unsigned int len =0;

        for ( i=0; i<strlen ((char*)versionString); i++)
            versionString[i] = (ViChar)toupper(versionString[i]);

        // Split the total string into functiongroup and base part
        GET_VI_STRING(fgVersionString,versionString);
        GET_VI_STRING(baseVersionString,NULL);
        baseVersionString[strlen(baseVersionString)] = '\0';
        strcpy(fgVersionStringCp,fgVersionString);
        strcpy(baseVersionStringCp,baseVersionString);

    // remove the leading and the trailing ' " ' from fgroup part
        REMOVE_STRING_SIGNS(fgVersionString);
        REMOVE_STRING_SIGNS(baseVersionString);
        if (fgVersionString[0] == 'V')
            fgVersionString++;
//    string++;
//    string[strlen(string)-1] = '\0'

        // split fgversion
        mainVers = atol(strtok((char*) fgVersionString, "."));
        subVers = atol(strtok((char*) NULL, "."));

        // if there is a trailling letter, get it
//printf("Rest des FGVersionsStrings: %s\n",fgVersionString);
        len = strlen ((char*)fgVersionStringCp);
        for ( i=0; i<len; i++)
        {
            if (i >0)
                if (isalpha(fgVersionStringCp[i]) && (isdigit(fgVersionStringCp[i-1])) )
                {
                    letterVers1 = (ViInt32) fgVersionStringCp[i]-64; //ascii A = 65 --> A => 1
                    if (i<len)
                        if (isalpha(fgVersionStringCp[i+1]))
                            letterVers2 = (ViInt32) fgVersionStringCp[i+1]-64; //ascii A = 65 --> A => 1
                    i = len;
                }
        }
        if (letterVers1 == 0) // if no letter, use 99 al highest possible letter version
            letterVers1 = 99;
        if (letterVers2 == 0) // if no letter, use 99 al highest possible letter version
            letterVers2 = 99;

        // calculate the version number
        *fgVersion = mainVers*1000000 + subVers * 10000 + letterVers1*100 + letterVers2;

        // split base version
        mainVers   = 0;
        subVers    = 0;
        letterVers1 = 0;
        letterVers2 = 0;

        mainVers = atol(strtok((char*) baseVersionString, "."));

        subVers = atol(strtok((char*) NULL, "."));

        // if there is a trailling letter, get it
        len = strlen ((char*)baseVersionStringCp);
        for ( i=0; i<len; i++)
        {
            if (i >0)
                if (isalpha(baseVersionStringCp[i]) && (isdigit(baseVersionStringCp[i-1])) )
                {
                    letterVers1 = (ViInt32) baseVersionStringCp[i]-64; //ascii A = 65 --> A => 1
                    if (i<len)
                        if (isalpha(baseVersionStringCp[i+1]))
                            letterVers2 = (ViInt32) baseVersionStringCp[i+1]-64; //ascii A = 65 --> A => 1
                    i = len;
                }
        }
        if (letterVers1 == 0)  // if no letter, use 99 as highest possible letter version
            letterVers1 = 99;
        if (letterVers2 == 0)  // if no letter, use 99 as highest possible letter version
            letterVers2 = 99;

        // calculate the version number
        *baseVersion = mainVers*1000000 + subVers * 10000 + letterVers1 * 100 + letterVers2;

        return (RSCMU_Status);
    }

ViStatus _VI_FUNC RSCMU_DrvVersionStringToIntegers(ViString fgVersionString, ViInt32* fgVersion, ViInt32* baseVersion)
{
  ViStatus   RSCMU_Status = VI_SUCCESS;
  ViChar     baseVersionString[RD_BUFFER_SIZE]  = "";
  ViInt32    mainVers                           = 0;
  ViInt32    subVers                            = 0;
  ViInt32    releaseCandidate                   = 0;
  ViChar*    version_string;

        version_string = malloc (strlen(fgVersionString)+1);
        memcpy(version_string,fgVersionString,strlen(fgVersionString)+1);
        mainVers          = atol(strtok((char*) version_string, "."));
        subVers           = atol(strtok((char*) NULL, "."));
        releaseCandidate  = atol(strtok((char*) NULL, "."));
        free(version_string);

        // calculate the version number
        *fgVersion = mainVers*1000000 + subVers * 10000 + releaseCandidate * 100;

        mainVers          = 0;
        subVers           = 0;
        releaseCandidate  = 0;

        strcpy(baseVersionString, RSCMU_BASE_VERSION);

        mainVers          = atol(strtok((char*) baseVersionString, "."));
        subVers           = atol(strtok((char*) NULL, "."));
        releaseCandidate  = atol(strtok((char*) NULL, "."));

        // calculate the version number
        *baseVersion = mainVers*1000000 + subVers * 10000 + releaseCandidate * 100;

        return (RSCMU_Status);
}

/***************************************************************
**   Converts a unsigned Integer(max. 32 bit) to a zero termiated string
**   The given char array needs to have a length of  at least 33 (for the trailing \0)
***************************************************************/
ViStatus _VI_FUNC RSCMU_Dec2Bin(ViUInt32 dec, ViChar* bin, ViChar* encloseString)
{
    ViChar  r[35] = "0";
    ViChar  s[35] = "0";
    int i         = 0;
    int j         = 0;

//  bin[0]        = 48;
    if (dec == 0)
    {
        sprintf(bin,"%s%s%s",encloseString,s,encloseString);
        return VI_SUCCESS;
    }
    while(dec > 0)
    {
        r[i] = (char)(48+ (dec % 2));    // 48 = ascii( 0 )
        i++;
        dec = dec/2;
    }
    i--;
    for (j = 0; j <= i; j++)
    {
        s[j] = r[i-j];
    }
    bin[j] = 0;
    sprintf(bin,"%s%s%s",encloseString,s,encloseString);
    return VI_SUCCESS;
}

